#line 1 "ad_riscv.cpp"
//
// Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
// Copyright (c) 2014, 2020, Red Hat Inc. All rights reserved.
// Copyright (c) 2020, 2022, Huawei Technologies Co., Ltd. All rights reserved.
// DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
//
// This code is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 2 only, as
// published by the Free Software Foundation.
//
// This code is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// version 2 for more details (a copy is included in the LICENSE file that
// accompanied this code).
//
// You should have received a copy of the GNU General Public License version
// 2 along with this work; if not, write to the Free Software Foundation,
// Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
// or visit www.oracle.com if you need additional information or have any
// questions.
//
//

// Machine Generated File.  Do Not Edit!

#include "precompiled.hpp"
#include "adfiles/adGlobals_riscv.hpp"
#include "adfiles/ad_riscv.hpp"
#include "memory/allocation.inline.hpp"
#include "code/codeCache.hpp"
#include "code/compiledIC.hpp"
#include "code/nativeInst.hpp"
#include "code/vmreg.inline.hpp"
#include "gc/shared/collectedHeap.inline.hpp"
#include "oops/compiledICHolder.hpp"
#include "oops/compressedOops.hpp"
#include "oops/markWord.hpp"
#include "oops/method.hpp"
#include "oops/oop.inline.hpp"
#include "opto/c2_MacroAssembler.hpp"
#include "opto/cfgnode.hpp"
#include "opto/intrinsicnode.hpp"
#include "opto/locknode.hpp"
#include "opto/opcodes.hpp"
#include "opto/regalloc.hpp"
#include "opto/regmask.hpp"
#include "opto/runtime.hpp"
#include "runtime/safepointMechanism.hpp"
#include "runtime/sharedRuntime.hpp"
#include "runtime/stubRoutines.hpp"
#include "utilities/growableArray.hpp"
#include "utilities/powerOfTwo.hpp"

//SourceForm

#line 1023 "src/hotspot/cpu/riscv/riscv.ad"


// Derived RegMask with conditionally allocatable registers

RegMask _ANY_REG32_mask;
RegMask _ANY_REG_mask;
RegMask _PTR_REG_mask;
RegMask _NO_SPECIAL_REG32_mask;
RegMask _NO_SPECIAL_REG_mask;
RegMask _NO_SPECIAL_PTR_REG_mask;

void reg_mask_init() {

  _ANY_REG32_mask = _ALL_REG32_mask;
  _ANY_REG32_mask.Remove(OptoReg::as_OptoReg(x0->as_VMReg()));

  _ANY_REG_mask = _ALL_REG_mask;
  _ANY_REG_mask.SUBTRACT(_ZR_REG_mask);

  _PTR_REG_mask = _ALL_REG_mask;
  _PTR_REG_mask.SUBTRACT(_ZR_REG_mask);

  _NO_SPECIAL_REG32_mask = _ALL_REG32_mask;
  _NO_SPECIAL_REG32_mask.SUBTRACT(_NON_ALLOCATABLE_REG32_mask);

  _NO_SPECIAL_REG_mask = _ALL_REG_mask;
  _NO_SPECIAL_REG_mask.SUBTRACT(_NON_ALLOCATABLE_REG_mask);

  _NO_SPECIAL_PTR_REG_mask = _ALL_REG_mask;
  _NO_SPECIAL_PTR_REG_mask.SUBTRACT(_NON_ALLOCATABLE_REG_mask);

  // x27 is not allocatable when compressed oops is on
  if (UseCompressedOops) {
    _NO_SPECIAL_REG32_mask.Remove(OptoReg::as_OptoReg(x27->as_VMReg()));
    _NO_SPECIAL_REG_mask.SUBTRACT(_HEAPBASE_REG_mask);
    _NO_SPECIAL_PTR_REG_mask.SUBTRACT(_HEAPBASE_REG_mask);
  }

  // x8 is not allocatable when PreserveFramePointer is on
  if (PreserveFramePointer) {
    _NO_SPECIAL_REG32_mask.Remove(OptoReg::as_OptoReg(x8->as_VMReg()));
    _NO_SPECIAL_REG_mask.SUBTRACT(_FP_REG_mask);
    _NO_SPECIAL_PTR_REG_mask.SUBTRACT(_FP_REG_mask);
  }
}

void PhaseOutput::pd_perform_mach_node_analysis() {
}

int MachNode::pd_alignment_required() const {
  return 1;
}

int MachNode::compute_padding(int current_offset) const {
  return 0;
}

// is_CAS(int opcode, bool maybe_volatile)
//
// return true if opcode is one of the possible CompareAndSwapX
// values otherwise false.
bool is_CAS(int opcode, bool maybe_volatile)
{
  switch (opcode) {
    // We handle these
    case Op_CompareAndSwapI:
    case Op_CompareAndSwapL:
    case Op_CompareAndSwapP:
    case Op_CompareAndSwapN:
    case Op_ShenandoahCompareAndSwapP:
    case Op_ShenandoahCompareAndSwapN:
    case Op_CompareAndSwapB:
    case Op_CompareAndSwapS:
    case Op_GetAndSetI:
    case Op_GetAndSetL:
    case Op_GetAndSetP:
    case Op_GetAndSetN:
    case Op_GetAndAddI:
    case Op_GetAndAddL:
      return true;
    case Op_CompareAndExchangeI:
    case Op_CompareAndExchangeN:
    case Op_CompareAndExchangeB:
    case Op_CompareAndExchangeS:
    case Op_CompareAndExchangeL:
    case Op_CompareAndExchangeP:
    case Op_WeakCompareAndSwapB:
    case Op_WeakCompareAndSwapS:
    case Op_WeakCompareAndSwapI:
    case Op_WeakCompareAndSwapL:
    case Op_WeakCompareAndSwapP:
    case Op_WeakCompareAndSwapN:
    case Op_ShenandoahWeakCompareAndSwapP:
    case Op_ShenandoahWeakCompareAndSwapN:
    case Op_ShenandoahCompareAndExchangeP:
    case Op_ShenandoahCompareAndExchangeN:
      return maybe_volatile;
    default:
      return false;
  }
}

// predicate controlling translation of CAS
//
// returns true if CAS needs to use an acquiring load otherwise false
bool needs_acquiring_load_reserved(const Node *n)
{
  assert(n != NULL && is_CAS(n->Opcode(), true), "expecting a compare and swap");

  LoadStoreNode* ldst = n->as_LoadStore();
  if (n != NULL && is_CAS(n->Opcode(), false)) {
    assert(ldst != NULL && ldst->trailing_membar() != NULL, "expected trailing membar");
  } else {
    return ldst != NULL && ldst->trailing_membar() != NULL;
  }
  // so we can just return true here
  return true;
}
#define __ _masm.

// advance declarations for helper functions to convert register
// indices to register objects

// the ad file has to provide implementations of certain methods
// expected by the generic code
//
// REQUIRED FUNCTIONALITY

//=============================================================================

// !!!!! Special hack to get all types of calls to specify the byte offset
//       from the start of the call to the point where the return address
//       will point.

int MachCallStaticJavaNode::ret_addr_offset()
{
  // jal
  return 1 * NativeInstruction::instruction_size;
}

int MachCallDynamicJavaNode::ret_addr_offset()
{
  return 7 * NativeInstruction::instruction_size; // movptr, jal
}

int MachCallRuntimeNode::ret_addr_offset() {
  // for generated stubs the call will be
  //   jal(addr)
  // or with far branches
  //   jal(trampoline_stub)
  // for real runtime callouts it will be 11 instructions
  // see riscv_enc_java_to_runtime
  //   la(t1, retaddr)                ->  auipc + addi
  //   la(t0, RuntimeAddress(addr))   ->  lui + addi + slli + addi + slli + addi
  //   addi(sp, sp, -2 * wordSize)    ->  addi
  //   sd(t1, Address(sp, wordSize))  ->  sd
  //   jalr(t0)                       ->  jalr
  CodeBlob *cb = CodeCache::find_blob(_entry_point);
  if (cb != NULL) {
    return 1 * NativeInstruction::instruction_size;
  } else {
    return 11 * NativeInstruction::instruction_size;
  }
}

//
// Compute padding required for nodes which need alignment
//

// With RVC a call instruction may get 2-byte aligned.
// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallStaticJavaDirectNode::compute_padding(int current_offset) const
{
  // to make sure the address of jal 4-byte aligned.
  return align_up(current_offset, alignment_required()) - current_offset;
}

// With RVC a call instruction may get 2-byte aligned.
// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallDynamicJavaDirectNode::compute_padding(int current_offset) const
{
  // skip the movptr in MacroAssembler::ic_call():
  // lui + addi + slli + addi + slli + addi
  // Though movptr() has already 4-byte aligned with or without RVC,
  // We need to prevent from further changes by explicitly calculating the size.
  const int movptr_size = 6 * NativeInstruction::instruction_size;
  current_offset += movptr_size;
  // to make sure the address of jal 4-byte aligned.
  return align_up(current_offset, alignment_required()) - current_offset;
}

//=============================================================================

#ifndef PRODUCT
void MachBreakpointNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  assert_cond(st != NULL);
  st->print("BREAKPOINT");
}
#endif

void MachBreakpointNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  C2_MacroAssembler _masm(&cbuf);
  Assembler::CompressibleRegion cr(&_masm);
  __ ebreak();
}

uint MachBreakpointNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}

//=============================================================================

#ifndef PRODUCT
  void MachNopNode::format(PhaseRegAlloc*, outputStream* st) const {
    st->print("nop \t# %d bytes pad for loops and calls", _count);
  }
#endif

  void MachNopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc*) const {
    C2_MacroAssembler _masm(&cbuf);
    Assembler::CompressibleRegion cr(&_masm); // nops shall be 2-byte under RVC for alignment purposes.
    for (int i = 0; i < _count; i++) {
      __ nop();
    }
  }

  uint MachNopNode::size(PhaseRegAlloc*) const {
    return _count * (UseRVC ? NativeInstruction::compressed_instruction_size : NativeInstruction::instruction_size);
  }

//=============================================================================
const RegMask& MachConstantBaseNode::_out_RegMask = RegMask::Empty;

int ConstantTable::calculate_table_base_offset() const {
  return 0;  // absolute addressing, no offset
}

bool MachConstantBaseNode::requires_postalloc_expand() const { return false; }
void MachConstantBaseNode::postalloc_expand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  ShouldNotReachHere();
}

void MachConstantBaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  // Empty encoding
}

uint MachConstantBaseNode::size(PhaseRegAlloc* ra_) const {
  return 0;
}

#ifndef PRODUCT
void MachConstantBaseNode::format(PhaseRegAlloc* ra_, outputStream* st) const {
  assert_cond(st != NULL);
  st->print("-- \t// MachConstantBaseNode (empty encoding)");
}
#endif

#ifndef PRODUCT
void MachPrologNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  assert_cond(st != NULL && ra_ != NULL);
  Compile* C = ra_->C;

  int framesize = C->output()->frame_slots() << LogBytesPerInt;

  if (C->output()->need_stack_bang(framesize)) {
    st->print("# stack bang size=%d\n\t", framesize);
  }

  st->print("sd  fp, [sp, #%d]\n\t", - 2 * wordSize);
  st->print("sd  ra, [sp, #%d]\n\t", - wordSize);
  if (PreserveFramePointer) { st->print("sub  fp, sp, #%d\n\t", 2 * wordSize); }
  st->print("sub sp, sp, #%d\n\t", framesize);

  if (C->stub_function() == NULL && BarrierSet::barrier_set()->barrier_set_nmethod() != NULL) {
    st->print("ld  t0, [guard]\n\t");
    st->print("membar LoadLoad\n\t");
    st->print("ld  t1, [xthread, #thread_disarmed_offset]\n\t");
    st->print("beq t0, t1, skip\n\t");
    st->print("jalr #nmethod_entry_barrier_stub\n\t");
    st->print("j skip\n\t");
    st->print("guard: int\n\t");
    st->print("skip:\n\t");
  }
}
#endif

void MachPrologNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  assert_cond(ra_ != NULL);
  Compile* C = ra_->C;
  C2_MacroAssembler _masm(&cbuf);

  // n.b. frame size includes space for return pc and fp
  const int framesize = C->output()->frame_size_in_bytes();

  // insert a nop at the start of the prolog so we can patch in a
  // branch if we need to invalidate the method later
  __ nop();

  assert_cond(C != NULL);

  if (C->clinit_barrier_on_entry()) {
    assert(!C->method()->holder()->is_not_initialized(), "initialization should have been started");

    Label L_skip_barrier;

    __ mov_metadata(t1, C->method()->holder()->constant_encoding());
    __ clinit_barrier(t1, t0, &L_skip_barrier);
    __ far_jump(RuntimeAddress(SharedRuntime::get_handle_wrong_method_stub()));
    __ bind(L_skip_barrier);
  }

  int bangsize = C->output()->bang_size_in_bytes();
  if (C->output()->need_stack_bang(bangsize)) {
    __ generate_stack_overflow_check(bangsize);
  }

  __ build_frame(framesize);

  if (C->stub_function() == NULL) {
    BarrierSetAssembler* bs = BarrierSet::barrier_set()->barrier_set_assembler();
    bs->nmethod_entry_barrier(&_masm);
  }

  if (VerifyStackAtCalls) {
    Unimplemented();
  }

  C->output()->set_frame_complete(cbuf.insts_size());

  if (C->has_mach_constant_base_node()) {
    // NOTE: We set the table base offset here because users might be
    // emitted before MachConstantBaseNode.
    ConstantTable& constant_table = C->output()->constant_table();
    constant_table.set_table_base_offset(constant_table.calculate_table_base_offset());
  }
}

uint MachPrologNode::size(PhaseRegAlloc* ra_) const
{
  assert_cond(ra_ != NULL);
  return MachNode::size(ra_); // too many variables; just compute it
                              // the hard way
}

int MachPrologNode::reloc() const
{
  return 0;
}

//=============================================================================

#ifndef PRODUCT
void MachEpilogNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  assert_cond(st != NULL && ra_ != NULL);
  Compile* C = ra_->C;
  assert_cond(C != NULL);
  int framesize = C->output()->frame_size_in_bytes();

  st->print("# pop frame %d\n\t", framesize);

  if (framesize == 0) {
    st->print("ld  ra, [sp,#%d]\n\t", (2 * wordSize));
    st->print("ld  fp, [sp,#%d]\n\t", (3 * wordSize));
    st->print("add sp, sp, #%d\n\t", (2 * wordSize));
  } else {
    st->print("add  sp, sp, #%d\n\t", framesize);
    st->print("ld  ra, [sp,#%d]\n\t", - 2 * wordSize);
    st->print("ld  fp, [sp,#%d]\n\t", - wordSize);
  }

  if (do_polling() && C->is_method_compilation()) {
    st->print("# test polling word\n\t");
    st->print("ld t0, [xthread,#%d]\n\t", in_bytes(JavaThread::polling_word_offset()));
    st->print("bgtu sp, t0, #slow_path");
  }
}
#endif

void MachEpilogNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  assert_cond(ra_ != NULL);
  Compile* C = ra_->C;
  C2_MacroAssembler _masm(&cbuf);
  assert_cond(C != NULL);
  int framesize = C->output()->frame_size_in_bytes();

  __ remove_frame(framesize);

  if (StackReservedPages > 0 && C->has_reserved_stack_access()) {
    __ reserved_stack_check();
  }

  if (do_polling() && C->is_method_compilation()) {
    Label dummy_label;
    Label* code_stub = &dummy_label;
    if (!C->output()->in_scratch_emit_size()) {
      code_stub = &C->output()->safepoint_poll_table()->add_safepoint(__ offset());
    }
    __ relocate(relocInfo::poll_return_type);
    __ safepoint_poll(*code_stub, true /* at_return */, false /* acquire */, true /* in_nmethod */);
  }
}

uint MachEpilogNode::size(PhaseRegAlloc *ra_) const {
  assert_cond(ra_ != NULL);
  // Variable size. Determine dynamically.
  return MachNode::size(ra_);
}

int MachEpilogNode::reloc() const {
  // Return number of relocatable values contained in this instruction.
  return 1; // 1 for polling page.
}
const Pipeline * MachEpilogNode::pipeline() const {
  return MachNode::pipeline_class();
}

//=============================================================================

// Figure out which register class each belongs in: rc_int, rc_float or
// rc_stack.
enum RC { rc_bad, rc_int, rc_float, rc_vector, rc_stack };

static enum RC rc_class(OptoReg::Name reg) {

  if (reg == OptoReg::Bad) {
    return rc_bad;
  }

  // we have 30 int registers * 2 halves
  // (t0 and t1 are omitted)
  int slots_of_int_registers = RegisterImpl::max_slots_per_register * (RegisterImpl::number_of_registers - 2);
  if (reg < slots_of_int_registers) {
    return rc_int;
  }

  // we have 32 float register * 2 halves
  int slots_of_float_registers = FloatRegisterImpl::max_slots_per_register * FloatRegisterImpl::number_of_registers;
  if (reg < slots_of_int_registers + slots_of_float_registers) {
    return rc_float;
  }

  // we have 32 vector register * 4 halves
  int slots_of_vector_registers = VectorRegisterImpl::max_slots_per_register * VectorRegisterImpl::number_of_registers;
  if (reg < slots_of_int_registers + slots_of_float_registers + slots_of_vector_registers) {
    return rc_vector;
  }

  // Between vector regs & stack is the flags regs.
  assert(OptoReg::is_stack(reg), "blow up if spilling flags");

  return rc_stack;
}

uint MachSpillCopyNode::implementation(CodeBuffer *cbuf, PhaseRegAlloc *ra_, bool do_size, outputStream *st) const {
  assert_cond(ra_ != NULL);
  Compile* C = ra_->C;

  // Get registers to move.
  OptoReg::Name src_hi = ra_->get_reg_second(in(1));
  OptoReg::Name src_lo = ra_->get_reg_first(in(1));
  OptoReg::Name dst_hi = ra_->get_reg_second(this);
  OptoReg::Name dst_lo = ra_->get_reg_first(this);

  enum RC src_hi_rc = rc_class(src_hi);
  enum RC src_lo_rc = rc_class(src_lo);
  enum RC dst_hi_rc = rc_class(dst_hi);
  enum RC dst_lo_rc = rc_class(dst_lo);

  assert(src_lo != OptoReg::Bad && dst_lo != OptoReg::Bad, "must move at least 1 register");

  if (src_hi != OptoReg::Bad) {
    assert((src_lo & 1) == 0 && src_lo + 1 == src_hi &&
           (dst_lo & 1) == 0 && dst_lo + 1 == dst_hi,
           "expected aligned-adjacent pairs");
  }

  if (src_lo == dst_lo && src_hi == dst_hi) {
    return 0;            // Self copy, no move.
  }

  bool is64 = (src_lo & 1) == 0 && src_lo + 1 == src_hi &&
              (dst_lo & 1) == 0 && dst_lo + 1 == dst_hi;
  int src_offset = ra_->reg2offset(src_lo);
  int dst_offset = ra_->reg2offset(dst_lo);

  if (bottom_type()->isa_vect() != NULL) {
    uint ireg = ideal_reg();
    if (ireg == Op_VecA && cbuf) {
      C2_MacroAssembler _masm(cbuf);
      Assembler::CompressibleRegion cr(&_masm);
      int vector_reg_size_in_bytes = Matcher::scalable_vector_reg_size(T_BYTE);
      if (src_lo_rc == rc_stack && dst_lo_rc == rc_stack) {
        // stack to stack
        __ spill_copy_vector_stack_to_stack(src_offset, dst_offset,
                                            vector_reg_size_in_bytes);
      } else if (src_lo_rc == rc_vector && dst_lo_rc == rc_stack) {
        // vpr to stack
        __ spill(as_VectorRegister(Matcher::_regEncode[src_lo]), ra_->reg2offset(dst_lo));
      } else if (src_lo_rc == rc_stack && dst_lo_rc == rc_vector) {
        // stack to vpr
        __ unspill(as_VectorRegister(Matcher::_regEncode[dst_lo]), ra_->reg2offset(src_lo));
      } else if (src_lo_rc == rc_vector && dst_lo_rc == rc_vector) {
        // vpr to vpr
        __ vmv1r_v(as_VectorRegister(Matcher::_regEncode[dst_lo]), as_VectorRegister(Matcher::_regEncode[src_lo]));
      } else {
        ShouldNotReachHere();
      }
    }
  } else if (cbuf != NULL) {
    C2_MacroAssembler _masm(cbuf);
    Assembler::CompressibleRegion cr(&_masm);
    switch (src_lo_rc) {
      case rc_int:
        if (dst_lo_rc == rc_int) {  // gpr --> gpr copy
          if (!is64 && this->ideal_reg() != Op_RegI) { // zero extended for narrow oop or klass
            __ zero_extend(as_Register(Matcher::_regEncode[dst_lo]), as_Register(Matcher::_regEncode[src_lo]), 32);
          } else {
            __ mv(as_Register(Matcher::_regEncode[dst_lo]), as_Register(Matcher::_regEncode[src_lo]));
          }
        } else if (dst_lo_rc == rc_float) { // gpr --> fpr copy
          if (is64) {
            __ fmv_d_x(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                       as_Register(Matcher::_regEncode[src_lo]));
          } else {
            __ fmv_w_x(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                       as_Register(Matcher::_regEncode[src_lo]));
          }
        } else {                    // gpr --> stack spill
          assert(dst_lo_rc == rc_stack, "spill to bad register class");
          __ spill(as_Register(Matcher::_regEncode[src_lo]), is64, dst_offset);
        }
        break;
      case rc_float:
        if (dst_lo_rc == rc_int) {  // fpr --> gpr copy
          if (is64) {
            __ fmv_x_d(as_Register(Matcher::_regEncode[dst_lo]),
                       as_FloatRegister(Matcher::_regEncode[src_lo]));
          } else {
            __ fmv_x_w(as_Register(Matcher::_regEncode[dst_lo]),
                       as_FloatRegister(Matcher::_regEncode[src_lo]));
          }
        } else if (dst_lo_rc == rc_float) { // fpr --> fpr copy
          if (is64) {
            __ fmv_d(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                     as_FloatRegister(Matcher::_regEncode[src_lo]));
          } else {
            __ fmv_s(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                     as_FloatRegister(Matcher::_regEncode[src_lo]));
          }
        } else {                    // fpr --> stack spill
          assert(dst_lo_rc == rc_stack, "spill to bad register class");
          __ spill(as_FloatRegister(Matcher::_regEncode[src_lo]),
                   is64, dst_offset);
        }
        break;
      case rc_stack:
        if (dst_lo_rc == rc_int) {  // stack --> gpr load
          if (this->ideal_reg() == Op_RegI) {
            __ unspill(as_Register(Matcher::_regEncode[dst_lo]), is64, src_offset);
          } else { // // zero extended for narrow oop or klass
            __ unspillu(as_Register(Matcher::_regEncode[dst_lo]), is64, src_offset);
          }
        } else if (dst_lo_rc == rc_float) { // stack --> fpr load
          __ unspill(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                     is64, src_offset);
        } else {                    // stack --> stack copy
          assert(dst_lo_rc == rc_stack, "spill to bad register class");
          if (this->ideal_reg() == Op_RegI) {
            __ unspill(t0, is64, src_offset);
          } else { // zero extended for narrow oop or klass
            __ unspillu(t0, is64, src_offset);
          }
          __ spill(t0, is64, dst_offset);
        }
        break;
      default:
        ShouldNotReachHere();
    }
  }

  if (st != NULL) {
    st->print("spill ");
    if (src_lo_rc == rc_stack) {
      st->print("[sp, #%d] -> ", src_offset);
    } else {
      st->print("%s -> ", Matcher::regName[src_lo]);
    }
    if (dst_lo_rc == rc_stack) {
      st->print("[sp, #%d]", dst_offset);
    } else {
      st->print("%s", Matcher::regName[dst_lo]);
    }
    if (bottom_type()->isa_vect() != NULL) {
      int vsize = 0;
      if (ideal_reg() == Op_VecA) {
        vsize = Matcher::scalable_vector_reg_size(T_BYTE) * 8;
      } else {
        ShouldNotReachHere();
      }
      st->print("\t# vector spill size = %d", vsize);
    } else {
      st->print("\t# spill size = %d", is64 ? 64 : 32);
    }
  }

  return 0;
}

#ifndef PRODUCT
void MachSpillCopyNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  if (ra_ == NULL) {
    st->print("N%d = SpillCopy(N%d)", _idx, in(1)->_idx);
  } else {
    implementation(NULL, ra_, false, st);
  }
}
#endif

void MachSpillCopyNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  implementation(&cbuf, ra_, false, NULL);
}

uint MachSpillCopyNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}

//=============================================================================

#ifndef PRODUCT
void BoxLockNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  assert_cond(ra_ != NULL && st != NULL);
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_reg_first(this);
  st->print("add %s, sp, #%d\t# box lock",
            Matcher::regName[reg], offset);
}
#endif

void BoxLockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  C2_MacroAssembler _masm(&cbuf);

  assert_cond(ra_ != NULL);
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg    = ra_->get_encode(this);

  if (is_imm_in_range(offset, 12, 0)) {
    __ addi(as_Register(reg), sp, offset);
  } else if (is_imm_in_range(offset, 32, 0)) {
    __ li32(t0, offset);
    __ add(as_Register(reg), sp, t0);
  } else {
    ShouldNotReachHere();
  }
}

uint BoxLockNode::size(PhaseRegAlloc *ra_) const {
  // BoxLockNode is not a MachNode, so we can't just call MachNode::size(ra_).
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());

  if (is_imm_in_range(offset, 12, 0)) {
    return NativeInstruction::instruction_size;
  } else {
    return 3 * NativeInstruction::instruction_size; // lui + addiw + add;
  }
}

//=============================================================================

#ifndef PRODUCT
void MachUEPNode::format(PhaseRegAlloc* ra_, outputStream* st) const
{
  assert_cond(st != NULL);
  st->print_cr("# MachUEPNode");
  if (UseCompressedClassPointers) {
    st->print_cr("\tlwu t0, [j_rarg0, oopDesc::klass_offset_in_bytes()]\t# compressed klass");
    if (CompressedKlassPointers::shift() != 0) {
      st->print_cr("\tdecode_klass_not_null t0, t0");
    }
  } else {
    st->print_cr("\tld t0, [j_rarg0, oopDesc::klass_offset_in_bytes()]\t# compressed klass");
  }
  st->print_cr("\tbeq t0, t1, ic_hit");
  st->print_cr("\tj, SharedRuntime::_ic_miss_stub\t # Inline cache check");
  st->print_cr("\tic_hit:");
}
#endif

void MachUEPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const
{
  // This is the unverified entry point.
  C2_MacroAssembler _masm(&cbuf);

  Label skip;
  __ cmp_klass(j_rarg0, t1, t0, skip);
  __ far_jump(RuntimeAddress(SharedRuntime::get_ic_miss_stub()));
  __ bind(skip);
}

uint MachUEPNode::size(PhaseRegAlloc* ra_) const
{
  assert_cond(ra_ != NULL);
  return MachNode::size(ra_);
}

// REQUIRED EMIT CODE

//=============================================================================

// Emit exception handler code.
int HandlerImpl::emit_exception_handler(CodeBuffer& cbuf)
{
  // la_patchable t0, #exception_blob_entry_point
  // jr (offset)t0
  // or
  // j #exception_blob_entry_point
  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  C2_MacroAssembler _masm(&cbuf);
  address base = __ start_a_stub(size_exception_handler());
  if (base == NULL) {
    ciEnv::current()->record_failure("CodeCache is full");
    return 0;  // CodeBuffer::expand failed
  }
  int offset = __ offset();
  __ far_jump(RuntimeAddress(OptoRuntime::exception_blob()->entry_point()));
  assert(__ offset() - offset <= (int) size_exception_handler(), "overflow");
  __ end_a_stub();
  return offset;
}

// Emit deopt handler code.
int HandlerImpl::emit_deopt_handler(CodeBuffer& cbuf)
{
  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  C2_MacroAssembler _masm(&cbuf);
  address base = __ start_a_stub(size_deopt_handler());
  if (base == NULL) {
    ciEnv::current()->record_failure("CodeCache is full");
    return 0;  // CodeBuffer::expand failed
  }
  int offset = __ offset();

  __ auipc(ra, 0);
  __ far_jump(RuntimeAddress(SharedRuntime::deopt_blob()->unpack()));

  assert(__ offset() - offset <= (int) size_deopt_handler(), "overflow");
  __ end_a_stub();
  return offset;

}
// REQUIRED MATCHER CODE

//=============================================================================

const bool Matcher::match_rule_supported(int opcode) {
  if (!has_match_rule(opcode)) {
    return false;
  }

  switch (opcode) {
    case Op_CacheWB:           // fall through
    case Op_CacheWBPreSync:    // fall through
    case Op_CacheWBPostSync:
      if (!VM_Version::supports_data_cache_line_flush()) {
        return false;
      }
      break;

    case Op_StrCompressedCopy: // fall through
    case Op_StrInflatedCopy:   // fall through
    case Op_CountPositives:
      return UseRVV;

    case Op_EncodeISOArray:
      return UseRVV && SpecialEncodeISOArray;

    case Op_PopCountI:
    case Op_PopCountL:
      return UsePopCountInstruction;

    case Op_RotateRight:
    case Op_RotateLeft:
    case Op_CountLeadingZerosI:
    case Op_CountLeadingZerosL:
    case Op_CountTrailingZerosI:
    case Op_CountTrailingZerosL:
      return UseZbb;
  }

  return true; // Per default match rules are supported.
}

// Identify extra cases that we might want to provide match rules for vector nodes and
// other intrinsics guarded with vector length (vlen) and element type (bt).
const bool Matcher::match_rule_supported_vector(int opcode, int vlen, BasicType bt) {
  if (!match_rule_supported(opcode) || !vector_size_supported(bt, vlen)) {
    return false;
  }

  return op_vec_supported(opcode);
}

const bool Matcher::match_rule_supported_vector_masked(int opcode, int vlen, BasicType bt) {
  return false;
}

const RegMask* Matcher::predicate_reg_mask(void) {
  return NULL;
}

const TypeVectMask* Matcher::predicate_reg_type(const Type* elemTy, int length) {
  return NULL;
}

// Vector calling convention not yet implemented.
const bool Matcher::supports_vector_calling_convention(void) {
  return false;
}

OptoRegPair Matcher::vector_return_value(uint ideal_reg) {
  Unimplemented();
  return OptoRegPair(0, 0);
}

// Is this branch offset short enough that a short branch can be used?
//
// NOTE: If the platform does not provide any short branch variants, then
//       this method should return false for offset 0.
// |---label(L1)-----|
// |-----------------|
// |-----------------|----------eq: float-------------------
// |-----------------| // far_cmpD_branch   |   cmpD_branch
// |------- ---------|    feq;              |      feq;
// |-far_cmpD_branch-|    beqz done;        |      bnez L;
// |-----------------|    j L;              |
// |-----------------|    bind(done);       |
// |-----------------|--------------------------------------
// |-----------------| // so shortBrSize = br_size - 4;
// |-----------------| // so offs = offset - shortBrSize + 4;
// |---label(L2)-----|
bool Matcher::is_short_branch_offset(int rule, int br_size, int offset) {
  // The passed offset is relative to address of the branch.
  int shortBrSize = br_size - 4;
  int offs = offset - shortBrSize + 4;
  return (-4096 <= offs && offs < 4096);
}

// Vector width in bytes.
const int Matcher::vector_width_in_bytes(BasicType bt) {
  if (UseRVV) {
    // The MaxVectorSize should have been set by detecting RVV max vector register size when check UseRVV.
    // MaxVectorSize == VM_Version::_initial_vector_length
    return MaxVectorSize;
  }
  return 0;
}

// Limits on vector size (number of elements) loaded into vector.
const int Matcher::max_vector_size(const BasicType bt) {
  return vector_width_in_bytes(bt) / type2aelembytes(bt);
}
const int Matcher::min_vector_size(const BasicType bt) {
  return max_vector_size(bt);
}

// Vector ideal reg.
const uint Matcher::vector_ideal_reg(int len) {
  assert(MaxVectorSize >= len, "");
  if (UseRVV) {
    return Op_VecA;
  }

  ShouldNotReachHere();
  return 0;
}

const int Matcher::scalable_vector_reg_size(const BasicType bt) {
  return Matcher::max_vector_size(bt);
}

MachOper* Matcher::pd_specialize_generic_vector_operand(MachOper* original_opnd, uint ideal_reg, bool is_temp) {
  ShouldNotReachHere(); // generic vector operands not supported
  return NULL;
}

bool Matcher::is_reg2reg_move(MachNode* m) {
  ShouldNotReachHere(); // generic vector operands not supported
  return false;
}

bool Matcher::is_generic_vector(MachOper* opnd) {
  ShouldNotReachHere(); // generic vector operands not supported
  return false;
}

// Return whether or not this register is ever used as an argument.
// This function is used on startup to build the trampoline stubs in
// generateOptoStub.  Registers not mentioned will be killed by the VM
// call in the trampoline, and arguments in those registers not be
// available to the callee.
bool Matcher::can_be_java_arg(int reg)
{
  return
    reg ==  R10_num || reg == R10_H_num ||
    reg ==  R11_num || reg == R11_H_num ||
    reg ==  R12_num || reg == R12_H_num ||
    reg ==  R13_num || reg == R13_H_num ||
    reg ==  R14_num || reg == R14_H_num ||
    reg ==  R15_num || reg == R15_H_num ||
    reg ==  R16_num || reg == R16_H_num ||
    reg ==  R17_num || reg == R17_H_num ||
    reg ==  F10_num || reg == F10_H_num ||
    reg ==  F11_num || reg == F11_H_num ||
    reg ==  F12_num || reg == F12_H_num ||
    reg ==  F13_num || reg == F13_H_num ||
    reg ==  F14_num || reg == F14_H_num ||
    reg ==  F15_num || reg == F15_H_num ||
    reg ==  F16_num || reg == F16_H_num ||
    reg ==  F17_num || reg == F17_H_num;
}

bool Matcher::is_spillable_arg(int reg)
{
  return can_be_java_arg(reg);
}

uint Matcher::int_pressure_limit()
{
  // A derived pointer is live at CallNode and then is flagged by RA
  // as a spilled LRG. Spilling heuristics(Spill-USE) explicitly skip
  // derived pointers and lastly fail to spill after reaching maximum
  // number of iterations. Lowering the default pressure threshold to
  // (_NO_SPECIAL_REG32_mask.Size() minus 1) forces CallNode to become
  // a high register pressure area of the code so that split_DEF can
  // generate DefinitionSpillCopy for the derived pointer.
  uint default_int_pressure_threshold = _NO_SPECIAL_REG32_mask.Size() - 1;
  if (!PreserveFramePointer) {
    // When PreserveFramePointer is off, frame pointer is allocatable,
    // but different from other SOC registers, it is excluded from
    // fatproj's mask because its save type is No-Save. Decrease 1 to
    // ensure high pressure at fatproj when PreserveFramePointer is off.
    // See check_pressure_at_fatproj().
    default_int_pressure_threshold--;
  }
  return (INTPRESSURE == -1) ? default_int_pressure_threshold : INTPRESSURE;
}

uint Matcher::float_pressure_limit()
{
  // _FLOAT_REG_mask is generated by adlc from the float_reg register class.
  return (FLOATPRESSURE == -1) ? _FLOAT_REG_mask.Size() : FLOATPRESSURE;
}

bool Matcher::use_asm_for_ldiv_by_con(jlong divisor) {
  return false;
}

RegMask Matcher::divI_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for MODI projection of divmodI.
RegMask Matcher::modI_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for DIVL projection of divmodL.
RegMask Matcher::divL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for MODL projection of divmodL.
RegMask Matcher::modL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

const RegMask Matcher::method_handle_invoke_SP_save_mask() {
  return FP_REG_mask();
}

bool size_fits_all_mem_uses(AddPNode* addp, int shift) {
  assert_cond(addp != NULL);
  for (DUIterator_Fast imax, i = addp->fast_outs(imax); i < imax; i++) {
    Node* u = addp->fast_out(i);
    if (u != NULL && u->is_Mem()) {
      int opsize = u->as_Mem()->memory_size();
      assert(opsize > 0, "unexpected memory operand size");
      if (u->as_Mem()->memory_size() != (1 << shift)) {
        return false;
      }
    }
  }
  return true;
}

// Should the Matcher clone input 'm' of node 'n'?
bool Matcher::pd_clone_node(Node* n, Node* m, Matcher::MStack& mstack) {
  assert_cond(m != NULL);
  if (is_vshift_con_pattern(n, m)) { // ShiftV src (ShiftCntV con)
    mstack.push(m, Visit);           // m = ShiftCntV
    return true;
  }
  return false;
}

// Should the Matcher clone shifts on addressing modes, expecting them
// to be subsumed into complex addressing expressions or compute them
// into registers?
bool Matcher::pd_clone_address_expressions(AddPNode* m, Matcher::MStack& mstack, VectorSet& address_visited) {
  return clone_base_plus_offset_address(m, mstack, address_visited);
}


#line 1081 "ad_riscv.cpp"


//SourceForm

#line 35 "src/hotspot/cpu/riscv/riscv_v.ad"


  static void loadStore(C2_MacroAssembler masm, bool is_store,
                        VectorRegister reg, BasicType bt, Register base) {
    Assembler::SEW sew = Assembler::elemtype_to_sew(bt);
    masm.vsetvli(t0, x0, sew);
    if (is_store) {
      masm.vsex_v(reg, base, sew);
    } else {
      masm.vlex_v(reg, base, sew);
    }
  }

  bool op_vec_supported(int opcode) {
    switch (opcode) {
      // No multiply reduction instructions
      case Op_MulReductionVD:
      case Op_MulReductionVF:
      case Op_MulReductionVI:
      case Op_MulReductionVL:
      // Others
      case Op_Extract:
      case Op_ExtractB:
      case Op_ExtractC:
      case Op_ExtractD:
      case Op_ExtractF:
      case Op_ExtractI:
      case Op_ExtractL:
      case Op_ExtractS:
      case Op_ExtractUB:
      // Vector API specific
      case Op_AndReductionV:
      case Op_OrReductionV:
      case Op_XorReductionV:
      case Op_LoadVectorGather:
      case Op_StoreVectorScatter:
      case Op_VectorBlend:
      case Op_VectorCast:
      case Op_VectorCastB2X:
      case Op_VectorCastD2X:
      case Op_VectorCastF2X:
      case Op_VectorCastI2X:
      case Op_VectorCastL2X:
      case Op_VectorCastS2X:
      case Op_VectorInsert:
      case Op_VectorLoadConst:
      case Op_VectorLoadMask:
      case Op_VectorLoadShuffle:
      case Op_VectorMaskCmp:
      case Op_VectorRearrange:
      case Op_VectorReinterpret:
      case Op_VectorStoreMask:
      case Op_VectorTest:
      case Op_PopCountVI:
      case Op_PopCountVL:
        return false;
      default:
        return UseRVV;
    }
  }


#line 1149 "ad_riscv.cpp"


//SourceForm

#line 33 "src/hotspot/cpu/riscv/gc/z/z_riscv64.ad"


static void z_load_barrier(MacroAssembler& _masm, const MachNode* node, Address ref_addr, Register ref, Register tmp, int barrier_data) {
  if (barrier_data == ZLoadBarrierElided) {
    return;
  }
  ZLoadBarrierStubC2* const stub = ZLoadBarrierStubC2::create(node, ref_addr, ref, tmp, barrier_data);
  __ ld(tmp, Address(xthread, ZThreadLocalData::address_bad_mask_offset()));
  __ andr(tmp, tmp, ref);
  __ bnez(tmp, *stub->entry(), true /* far */);
  __ bind(*stub->continuation());
}

static void z_load_barrier_slow_path(MacroAssembler& _masm, const MachNode* node, Address ref_addr, Register ref, Register tmp) {
  ZLoadBarrierStubC2* const stub = ZLoadBarrierStubC2::create(node, ref_addr, ref, tmp, ZLoadBarrierStrong);
  __ j(*stub->entry());
  __ bind(*stub->continuation());
}


#line 1175 "ad_riscv.cpp"


#ifndef PRODUCT
void Compile::adlc_verification() {

  // Following assertions generated from definition section
  assert( DEFAULT_COST == 100, "Expect (100) to equal 100");
  assert( ALU_COST == 100, "Expect (1 * DEFAULT_COST) to equal 100");
  assert( LOAD_COST == 300, "Expect (3 * DEFAULT_COST) to equal 300");
  assert( STORE_COST == 100, "Expect (1 * DEFAULT_COST) to equal 100");
  assert( XFER_COST == 300, "Expect (3 * DEFAULT_COST) to equal 300");
  assert( BRANCH_COST == 100, "Expect (1 * DEFAULT_COST) to equal 100");
  assert( IMUL_COST == 1000, "Expect (10 * DEFAULT_COST) to equal 1000");
  assert( IDIVSI_COST == 3400, "Expect (34 * DEFAULT_COST) to equal 3400");
  assert( IDIVDI_COST == 6600, "Expect (66 * DEFAULT_COST) to equal 6600");
  assert( FMUL_SINGLE_COST == 500, "Expect (5 * DEFAULT_COST) to equal 500");
  assert( FMUL_DOUBLE_COST == 700, "Expect (7 * DEFAULT_COST) to equal 700");
  assert( FDIV_COST == 2000, "Expect (20 * DEFAULT_COST) to equal 2000");
  assert( FSQRT_COST == 2500, "Expect (25 * DEFAULT_COST) to equal 2500");
  assert( VOLATILE_REF_COST == 1000, "Expect (10 * DEFAULT_COST) to equal 1000");
  assert( VEC_COST == 200, "Expect (200) to equal 200");
}
#endif

// Map from machine-independent register number to register_save_policy
const        char register_save_policy[] = {
  'C', // R7
  'C', // R7_H
  'C', // R28
  'C', // R28_H
  'C', // R29
  'C', // R29_H
  'C', // R30
  'C', // R30_H
  'C', // R31
  'C', // R31_H
  'C', // R10
  'C', // R10_H
  'C', // R11
  'C', // R11_H
  'C', // R12
  'C', // R12_H
  'C', // R13
  'C', // R13_H
  'C', // R14
  'C', // R14_H
  'C', // R15
  'C', // R15_H
  'C', // R16
  'C', // R16_H
  'C', // R17
  'C', // R17_H
  'C', // R9
  'C', // R9_H
  'C', // R18
  'C', // R18_H
  'C', // R19
  'C', // R19_H
  'C', // R20
  'C', // R20_H
  'C', // R21
  'C', // R21_H
  'C', // R22
  'C', // R22_H
  'C', // R24
  'C', // R24_H
  'C', // R25
  'C', // R25_H
  'C', // R26
  'C', // R26_H
  'N', // R23
  'N', // R23_H
  'C', // R27
  'C', // R27_H
  'N', // R4
  'N', // R4_H
  'N', // R8
  'N', // R8_H
  'N', // R0
  'N', // R0_H
  'N', // R1
  'N', // R1_H
  'N', // R2
  'N', // R2_H
  'N', // R3
  'N', // R3_H
  'C', // F0
  'C', // F0_H
  'C', // F1
  'C', // F1_H
  'C', // F2
  'C', // F2_H
  'C', // F3
  'C', // F3_H
  'C', // F4
  'C', // F4_H
  'C', // F5
  'C', // F5_H
  'C', // F6
  'C', // F6_H
  'C', // F7
  'C', // F7_H
  'C', // F28
  'C', // F28_H
  'C', // F29
  'C', // F29_H
  'C', // F30
  'C', // F30_H
  'C', // F31
  'C', // F31_H
  'C', // F10
  'C', // F10_H
  'C', // F11
  'C', // F11_H
  'C', // F12
  'C', // F12_H
  'C', // F13
  'C', // F13_H
  'C', // F14
  'C', // F14_H
  'C', // F15
  'C', // F15_H
  'C', // F16
  'C', // F16_H
  'C', // F17
  'C', // F17_H
  'C', // F8
  'C', // F8_H
  'C', // F9
  'C', // F9_H
  'C', // F18
  'C', // F18_H
  'C', // F19
  'C', // F19_H
  'C', // F20
  'C', // F20_H
  'C', // F21
  'C', // F21_H
  'C', // F22
  'C', // F22_H
  'C', // F23
  'C', // F23_H
  'C', // F24
  'C', // F24_H
  'C', // F25
  'C', // F25_H
  'C', // F26
  'C', // F26_H
  'C', // F27
  'C', // F27_H
  'C', // V0
  'C', // V0_H
  'C', // V0_J
  'C', // V0_K
  'C', // V1
  'C', // V1_H
  'C', // V1_J
  'C', // V1_K
  'C', // V2
  'C', // V2_H
  'C', // V2_J
  'C', // V2_K
  'C', // V3
  'C', // V3_H
  'C', // V3_J
  'C', // V3_K
  'C', // V4
  'C', // V4_H
  'C', // V4_J
  'C', // V4_K
  'C', // V5
  'C', // V5_H
  'C', // V5_J
  'C', // V5_K
  'C', // V6
  'C', // V6_H
  'C', // V6_J
  'C', // V6_K
  'C', // V7
  'C', // V7_H
  'C', // V7_J
  'C', // V7_K
  'C', // V8
  'C', // V8_H
  'C', // V8_J
  'C', // V8_K
  'C', // V9
  'C', // V9_H
  'C', // V9_J
  'C', // V9_K
  'C', // V10
  'C', // V10_H
  'C', // V10_J
  'C', // V10_K
  'C', // V11
  'C', // V11_H
  'C', // V11_J
  'C', // V11_K
  'C', // V12
  'C', // V12_H
  'C', // V12_J
  'C', // V12_K
  'C', // V13
  'C', // V13_H
  'C', // V13_J
  'C', // V13_K
  'C', // V14
  'C', // V14_H
  'C', // V14_J
  'C', // V14_K
  'C', // V15
  'C', // V15_H
  'C', // V15_J
  'C', // V15_K
  'C', // V16
  'C', // V16_H
  'C', // V16_J
  'C', // V16_K
  'C', // V17
  'C', // V17_H
  'C', // V17_J
  'C', // V17_K
  'C', // V18
  'C', // V18_H
  'C', // V18_J
  'C', // V18_K
  'C', // V19
  'C', // V19_H
  'C', // V19_J
  'C', // V19_K
  'C', // V20
  'C', // V20_H
  'C', // V20_J
  'C', // V20_K
  'C', // V21
  'C', // V21_H
  'C', // V21_J
  'C', // V21_K
  'C', // V22
  'C', // V22_H
  'C', // V22_J
  'C', // V22_K
  'C', // V23
  'C', // V23_H
  'C', // V23_J
  'C', // V23_K
  'C', // V24
  'C', // V24_H
  'C', // V24_J
  'C', // V24_K
  'C', // V25
  'C', // V25_H
  'C', // V25_J
  'C', // V25_K
  'C', // V26
  'C', // V26_H
  'C', // V26_J
  'C', // V26_K
  'C', // V27
  'C', // V27_H
  'C', // V27_J
  'C', // V27_K
  'C', // V28
  'C', // V28_H
  'C', // V28_J
  'C', // V28_K
  'C', // V29
  'C', // V29_H
  'C', // V29_J
  'C', // V29_K
  'C', // V30
  'C', // V30_H
  'C', // V30_J
  'C', // V30_K
  'C', // V31
  'C', // V31_H
  'C', // V31_J
  'C', // V31_K
  'C' // no trailing comma // RFLAGS
};

// Map from machine-independent register number to c_reg_save_policy
const        char c_reg_save_policy[] = {
  'C', // R7
  'C', // R7_H
  'C', // R28
  'C', // R28_H
  'C', // R29
  'C', // R29_H
  'C', // R30
  'C', // R30_H
  'C', // R31
  'C', // R31_H
  'C', // R10
  'C', // R10_H
  'C', // R11
  'C', // R11_H
  'C', // R12
  'C', // R12_H
  'C', // R13
  'C', // R13_H
  'C', // R14
  'C', // R14_H
  'C', // R15
  'C', // R15_H
  'C', // R16
  'C', // R16_H
  'C', // R17
  'C', // R17_H
  'E', // R9
  'E', // R9_H
  'E', // R18
  'E', // R18_H
  'E', // R19
  'E', // R19_H
  'E', // R20
  'E', // R20_H
  'E', // R21
  'E', // R21_H
  'E', // R22
  'E', // R22_H
  'E', // R24
  'E', // R24_H
  'E', // R25
  'E', // R25_H
  'E', // R26
  'E', // R26_H
  'E', // R23
  'E', // R23_H
  'E', // R27
  'E', // R27_H
  'N', // R4
  'N', // R4_H
  'E', // R8
  'E', // R8_H
  'N', // R0
  'N', // R0_H
  'C', // R1
  'C', // R1_H
  'E', // R2
  'E', // R2_H
  'N', // R3
  'N', // R3_H
  'C', // F0
  'C', // F0_H
  'C', // F1
  'C', // F1_H
  'C', // F2
  'C', // F2_H
  'C', // F3
  'C', // F3_H
  'C', // F4
  'C', // F4_H
  'C', // F5
  'C', // F5_H
  'C', // F6
  'C', // F6_H
  'C', // F7
  'C', // F7_H
  'C', // F28
  'C', // F28_H
  'C', // F29
  'C', // F29_H
  'C', // F30
  'C', // F30_H
  'C', // F31
  'C', // F31_H
  'C', // F10
  'C', // F10_H
  'C', // F11
  'C', // F11_H
  'C', // F12
  'C', // F12_H
  'C', // F13
  'C', // F13_H
  'C', // F14
  'C', // F14_H
  'C', // F15
  'C', // F15_H
  'C', // F16
  'C', // F16_H
  'C', // F17
  'C', // F17_H
  'E', // F8
  'E', // F8_H
  'E', // F9
  'E', // F9_H
  'E', // F18
  'E', // F18_H
  'E', // F19
  'E', // F19_H
  'E', // F20
  'E', // F20_H
  'E', // F21
  'E', // F21_H
  'E', // F22
  'E', // F22_H
  'E', // F23
  'E', // F23_H
  'E', // F24
  'E', // F24_H
  'E', // F25
  'E', // F25_H
  'E', // F26
  'E', // F26_H
  'E', // F27
  'E', // F27_H
  'C', // V0
  'C', // V0_H
  'C', // V0_J
  'C', // V0_K
  'C', // V1
  'C', // V1_H
  'C', // V1_J
  'C', // V1_K
  'C', // V2
  'C', // V2_H
  'C', // V2_J
  'C', // V2_K
  'C', // V3
  'C', // V3_H
  'C', // V3_J
  'C', // V3_K
  'C', // V4
  'C', // V4_H
  'C', // V4_J
  'C', // V4_K
  'C', // V5
  'C', // V5_H
  'C', // V5_J
  'C', // V5_K
  'C', // V6
  'C', // V6_H
  'C', // V6_J
  'C', // V6_K
  'C', // V7
  'C', // V7_H
  'C', // V7_J
  'C', // V7_K
  'C', // V8
  'C', // V8_H
  'C', // V8_J
  'C', // V8_K
  'C', // V9
  'C', // V9_H
  'C', // V9_J
  'C', // V9_K
  'C', // V10
  'C', // V10_H
  'C', // V10_J
  'C', // V10_K
  'C', // V11
  'C', // V11_H
  'C', // V11_J
  'C', // V11_K
  'C', // V12
  'C', // V12_H
  'C', // V12_J
  'C', // V12_K
  'C', // V13
  'C', // V13_H
  'C', // V13_J
  'C', // V13_K
  'C', // V14
  'C', // V14_H
  'C', // V14_J
  'C', // V14_K
  'C', // V15
  'C', // V15_H
  'C', // V15_J
  'C', // V15_K
  'C', // V16
  'C', // V16_H
  'C', // V16_J
  'C', // V16_K
  'C', // V17
  'C', // V17_H
  'C', // V17_J
  'C', // V17_K
  'C', // V18
  'C', // V18_H
  'C', // V18_J
  'C', // V18_K
  'C', // V19
  'C', // V19_H
  'C', // V19_J
  'C', // V19_K
  'C', // V20
  'C', // V20_H
  'C', // V20_J
  'C', // V20_K
  'C', // V21
  'C', // V21_H
  'C', // V21_J
  'C', // V21_K
  'C', // V22
  'C', // V22_H
  'C', // V22_J
  'C', // V22_K
  'C', // V23
  'C', // V23_H
  'C', // V23_J
  'C', // V23_K
  'C', // V24
  'C', // V24_H
  'C', // V24_J
  'C', // V24_K
  'C', // V25
  'C', // V25_H
  'C', // V25_J
  'C', // V25_K
  'C', // V26
  'C', // V26_H
  'C', // V26_J
  'C', // V26_K
  'C', // V27
  'C', // V27_H
  'C', // V27_J
  'C', // V27_K
  'C', // V28
  'C', // V28_H
  'C', // V28_J
  'C', // V28_K
  'C', // V29
  'C', // V29_H
  'C', // V29_J
  'C', // V29_K
  'C', // V30
  'C', // V30_H
  'C', // V30_J
  'C', // V30_K
  'C', // V31
  'C', // V31_H
  'C', // V31_J
  'C', // V31_K
  'C' // no trailing comma // RFLAGS
};

// Map from machine-independent register number to register_save_type
const        int register_save_type[] = {
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_VecA,
  Op_RegFlags // no trailing comma
};


const        int   reduceOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  immI_rule,
  /*    9 */  immI0_rule,
  /*   10 */  immI_1_rule,
  /*   11 */  immI_M1_rule,
  /*   12 */  uimmI6_ge32_rule,
  /*   13 */  immI_le_4_rule,
  /*   14 */  immI_16_rule,
  /*   15 */  immI_24_rule,
  /*   16 */  immI_31_rule,
  /*   17 */  immI_63_rule,
  /*   18 */  immIAdd_rule,
  /*   19 */  immISub_rule,
  /*   20 */  immI5_rule,
  /*   21 */  immL5_rule,
  /*   22 */  immL_rule,
  /*   23 */  immL0_rule,
  /*   24 */  immP_rule,
  /*   25 */  immP0_rule,
  /*   26 */  immP_1_rule,
  /*   27 */  immByteMapBase_rule,
  /*   28 */  immI_16bits_rule,
  /*   29 */  immL_32bits_rule,
  /*   30 */  immL_M1_rule,
  /*   31 */  immL_pc_off_rule,
  /*   32 */  immLAdd_rule,
  /*   33 */  immLSub_rule,
  /*   34 */  immN_rule,
  /*   35 */  immN0_rule,
  /*   36 */  immNKlass_rule,
  /*   37 */  immD_rule,
  /*   38 */  immD0_rule,
  /*   39 */  immF_rule,
  /*   40 */  immF0_rule,
  /*   41 */  immIOffset_rule,
  /*   42 */  immLOffset_rule,
  /*   43 */  immIScale_rule,
  /*   44 */  iRegI_rule,
  /*   45 */  iRegINoSp_rule,
  /*   46 */  iRegI_R10_rule,
  /*   47 */  iRegI_R12_rule,
  /*   48 */  iRegI_R13_rule,
  /*   49 */  iRegI_R14_rule,
  /*   50 */  iRegL_rule,
  /*   51 */  iRegLNoSp_rule,
  /*   52 */  iRegL_R28_rule,
  /*   53 */  iRegL_R29_rule,
  /*   54 */  iRegL_R30_rule,
  /*   55 */  iRegP_rule,
  /*   56 */  iRegPNoSp_rule,
  /*   57 */  iRegP_R10_rule,
  /*   58 */  iRegP_R11_rule,
  /*   59 */  iRegP_R12_rule,
  /*   60 */  iRegP_R13_rule,
  /*   61 */  iRegP_R14_rule,
  /*   62 */  iRegP_R15_rule,
  /*   63 */  iRegP_R16_rule,
  /*   64 */  iRegP_R28_rule,
  /*   65 */  iRegN_rule,
  /*   66 */  iRegNNoSp_rule,
  /*   67 */  iRegIHeapbase_rule,
  /*   68 */  iRegL_R10_rule,
  /*   69 */  fRegF_rule,
  /*   70 */  fRegD_rule,
  /*   71 */  vReg_rule,
  /*   72 */  vReg_V1_rule,
  /*   73 */  vReg_V2_rule,
  /*   74 */  vReg_V3_rule,
  /*   75 */  vReg_V4_rule,
  /*   76 */  vReg_V5_rule,
  /*   77 */  javaThread_RegP_rule,
  /*   78 */  indirect_rule,
  /*   79 */  indOffI_rule,
  /*   80 */  indOffL_rule,
  /*   81 */  indirectN_rule,
  /*   82 */  indOffIN_rule,
  /*   83 */  indOffLN_rule,
  /*   84 */  thread_anchor_pc_rule,
  /*   85 */  stackSlotI_rule,
  /*   86 */  stackSlotF_rule,
  /*   87 */  stackSlotD_rule,
  /*   88 */  stackSlotL_rule,
  /*   89 */  iRegL2I_rule,
  /*   90 */  cmpOp_rule,
  /*   91 */  cmpOpU_rule,
  /*   92 */  cmpOpEqNe_rule,
  /*   93 */  cmpOpULtGe_rule,
  /*   94 */  cmpOpUEqNeLeGt_rule,
  /*   95 */  rFlagsReg_rule,
  /*   96 */  inline_cache_RegP_rule,
  // last operand
  /*   97 */  memory_rule,
  /*   98 */  iRegIorL2I_rule,
  /*   99 */  iRegIorL_rule,
  /*  100 */  iRegNorP_rule,
  /*  101 */  iRegILNP_rule,
  /*  102 */  iRegILNPNoSp_rule,
  /*  103 */  immIorL_rule,
  /*  104 */  vmemA_rule,
  // last operand class
  /*  105 */  _DecodeN_iRegN__rule,
  /*  106 */  _LoadB_memory__rule,
  /*  107 */  _LoadUB_memory__rule,
  /*  108 */  _LoadS_memory__rule,
  /*  109 */  _LoadUS_memory__rule,
  /*  110 */  _LoadI_memory__rule,
  /*  111 */  _ConvI2L__LoadI_memory___rule,
  /*  112 */  _Binary_iRegP_iRegP_rule,
  /*  113 */  _Binary_iRegL_iRegL_rule,
  /*  114 */  _Binary_iRegI_iRegI_rule,
  /*  115 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  116 */  _Binary_iRegN_iRegN_rule,
  /*  117 */  _ConvL2I_iRegL__rule,
  /*  118 */  _ConvI2L_iRegI__rule,
  /*  119 */  _RShiftI_iRegIorL2I_immI_31_rule,
  /*  120 */  _RShiftL_iRegL_immI_63_rule,
  /*  121 */  _CastP2X_iRegP__rule,
  /*  122 */  _Binary_fRegF_fRegF_rule,
  /*  123 */  _Binary_fRegD_fRegD_rule,
  /*  124 */  _NegF_fRegF__rule,
  /*  125 */  _NegD_fRegD__rule,
  /*  126 */  _Binary__NegF_fRegF__fRegF_rule,
  /*  127 */  _Binary_fRegF__NegF_fRegF__rule,
  /*  128 */  _Binary__NegD_fRegD__fRegD_rule,
  /*  129 */  _Binary_fRegD__NegD_fRegD__rule,
  /*  130 */  _ConvF2D_fRegF__rule,
  /*  131 */  _SqrtD__ConvF2D_fRegF___rule,
  /*  132 */  _ConvI2L_iRegIorL2I__rule,
  /*  133 */  _CastP2X__DecodeN_iRegN___rule,
  /*  134 */  _CmpI_iRegI_iRegI_rule,
  /*  135 */  _CmpU_iRegI_iRegI_rule,
  /*  136 */  _CmpL_iRegL_iRegL_rule,
  /*  137 */  _CmpUL_iRegL_iRegL_rule,
  /*  138 */  _CmpP_iRegP_iRegP_rule,
  /*  139 */  _CmpN_iRegN_iRegN_rule,
  /*  140 */  _CmpF_fRegF_fRegF_rule,
  /*  141 */  _CmpD_fRegD_fRegD_rule,
  /*  142 */  _CmpI_iRegI_immI0_rule,
  /*  143 */  _CmpU_iRegI_immI0_rule,
  /*  144 */  _CmpL_iRegL_immL0_rule,
  /*  145 */  _CmpUL_iRegL_immL0_rule,
  /*  146 */  _CmpP_iRegP_immP0_rule,
  /*  147 */  _CmpN_iRegN_immN0_rule,
  /*  148 */  _CmpP__DecodeN_iRegN__immP0_rule,
  /*  149 */  _Binary_cmpOp__CmpI_iRegI_iRegI_rule,
  /*  150 */  _Binary_iRegINoSp_iRegI_rule,
  /*  151 */  _Binary_cmpOpU__CmpU_iRegI_iRegI_rule,
  /*  152 */  _Binary_cmpOp__CmpL_iRegL_iRegL_rule,
  /*  153 */  _Binary_iRegLNoSp_iRegL_rule,
  /*  154 */  _Binary_cmpOpU__CmpUL_iRegL_iRegL_rule,
  /*  155 */  _PartialSubtypeCheck_iRegP_R14_iRegP_R10_rule,
  /*  156 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  157 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  158 */  _Binary_iRegP_R13_immI_le_4_rule,
  /*  159 */  _Binary_iRegP_R13_immI_1_rule,
  /*  160 */  _Binary_iRegP_R11_iRegP_R13_rule,
  /*  161 */  _Binary_vReg_vReg_rule,
  /*  162 */  _NegVF_vReg__rule,
  /*  163 */  _Binary__NegVF_vReg__vReg_rule,
  /*  164 */  _Binary_vReg__NegVF_vReg__rule,
  /*  165 */  _NegVD_vReg__rule,
  /*  166 */  _Binary__NegVD_vReg__vReg_rule,
  /*  167 */  _Binary_vReg__NegVD_vReg__rule,
  /*  168 */  _MulVB_vReg_vReg_rule,
  /*  169 */  _MulVS_vReg_vReg_rule,
  /*  170 */  _MulVI_vReg_vReg_rule,
  /*  171 */  _MulVL_vReg_vReg_rule,
  /*  172 */  _RShiftCntV_immI__rule,
  /*  173 */  _LShiftCntV_immI__rule,
  /*  174 */  _Binary_iRegP_R11_iRegI_R13_rule,
  /*  175 */  _LShiftI_iRegIorL2I_immI_24_rule,
  /*  176 */  _LShiftI_iRegIorL2I_immI_16_rule,
  /*  177 */  _LShiftL_iRegL_immIScale_rule,
  /*  178 */  _LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  179 */  _XorI_iRegI_immI_M1_rule,
  /*  180 */  _XorL_iRegL_immL_M1_rule,
  // last internally defined operand
  /*  181 */  iRegINoSp_rule,
  /*  182 */  iRegLNoSp_rule,
  /*  183 */  iRegPNoSp_rule,
  /*  184 */  iRegPNoSp_rule,
  /*  185 */  iRegPNoSp_rule,
  /*  186 */  iRegPNoSp_rule,
  /*  187 */  iRegNNoSp_rule,
  /*  188 */  iRegNNoSp_rule,
  /*  189 */  iRegNNoSp_rule,
  /*  190 */  fRegF_rule,
  /*  191 */  fRegF_rule,
  /*  192 */  fRegD_rule,
  /*  193 */  fRegD_rule,
  /*  194 */  rFlagsReg_rule,
  /*  195 */  rFlagsReg_rule,
  /*  196 */  rFlagsReg_rule,
  /*  197 */  fRegF_rule,
  /*  198 */  fRegD_rule,
  /*  199 */  iRegINoSp_rule,
  /*  200 */  iRegLNoSp_rule,
  /*  201 */  fRegF_rule,
  /*  202 */  fRegD_rule,
  /*  203 */  iRegPNoSp_rule,
  /*  204 */  iRegLNoSp_rule,
  /*  205 */  iRegPNoSp_rule,
  /*  206 */  iRegL_rule,
  /*  207 */  iRegI_rule,
  /*  208 */  iRegPNoSp_rule,
  /*  209 */  fRegF_rule,
  /*  210 */  fRegD_rule,
  /*  211 */  vReg_rule,
  /*  212 */  iRegINoSp_rule,
  /*  213 */  iRegINoSp_rule,
  /*  214 */  iRegINoSp_rule,
  /*  215 */  fRegF_rule,
  /*  216 */  iRegLNoSp_rule,
  /*  217 */  fRegD_rule,
  /*  218 */  iRegINoSp_rule,
  /*  219 */  fRegF_rule,
  /*  220 */  iRegLNoSp_rule,
  /*  221 */  fRegD_rule,
  /*  222 */  rFlagsReg_rule,
  /*  223 */  javaThread_RegP_rule,
  /*  224 */  rFlagsReg_rule,
  /*  225 */  rFlagsReg_rule,
  /*  226 */  vReg_rule,
  /*  227 */  vReg_rule,
  /*  228 */  vReg_rule,
  /*  229 */  vReg_rule,
  /*  230 */  vReg_rule,
  /*  231 */  vReg_rule,
  /*  232 */  vReg_rule,
  /*  233 */  vReg_rule,
  /*  234 */  vReg_rule,
  /*  235 */  vReg_rule,
  /*  236 */  vReg_rule,
  /*  237 */  vReg_rule,
  /*  238 */  vReg_rule,
  /*  239 */  vReg_rule,
  /*  240 */  iRegINoSp_rule,
  /*  241 */  iRegINoSp_rule,
  /*  242 */  iRegINoSp_rule,
  /*  243 */  iRegINoSp_rule,
  /*  244 */  iRegINoSp_rule,
  /*  245 */  iRegINoSp_rule,
  /*  246 */  iRegINoSp_rule,
  /*  247 */  iRegLNoSp_rule,
  /*  248 */  iRegINoSp_rule,
  /*  249 */  iRegLNoSp_rule,
  /*  250 */  iRegINoSp_rule,
  /*  251 */  iRegLNoSp_rule,
  /*  252 */  iRegINoSp_rule,
  /*  253 */  iRegLNoSp_rule,
  /*  254 */  iRegINoSp_rule,
  /*  255 */  iRegLNoSp_rule,
  /*  256 */  iRegINoSp_rule,
  /*  257 */  iRegLNoSp_rule,
  /*  258 */  iRegLNoSp_rule,
  /*  259 */  iRegLNoSp_rule,
  /*  260 */  iRegINoSp_rule,
  /*  261 */  iRegPNoSp_rule,
  /*  262 */  iRegNNoSp_rule,
  /*  263 */  iRegPNoSp_rule,
  /*  264 */  iRegNNoSp_rule,
  /*  265 */  fRegF_rule,
  /*  266 */  fRegD_rule,
  /*  267 */  Universe_rule,
  /*  268 */  Universe_rule,
  /*  269 */  Universe_rule,
  /*  270 */  Universe_rule,
  /*  271 */  Universe_rule,
  /*  272 */  Universe_rule,
  /*  273 */  Universe_rule,
  /*  274 */  Universe_rule,
  /*  275 */  Universe_rule,
  /*  276 */  Universe_rule,
  /*  277 */  Universe_rule,
  /*  278 */  Universe_rule,
  /*  279 */  Universe_rule,
  /*  280 */  Universe_rule,
  /*  281 */  Universe_rule,
  /*  282 */  Universe_rule,
  /*  283 */  Universe_rule,
  /*  284 */  iRegPNoSp_rule,
  /*  285 */  iRegINoSp_rule,
  /*  286 */  iRegINoSp_rule,
  /*  287 */  iRegINoSp_rule,
  /*  288 */  iRegINoSp_rule,
  /*  289 */  iRegINoSp_rule,
  /*  290 */  iRegINoSp_rule,
  /*  291 */  iRegINoSp_rule,
  /*  292 */  iRegINoSp_rule,
  /*  293 */  iRegINoSp_rule,
  /*  294 */  iRegINoSp_rule,
  /*  295 */  iRegINoSp_rule,
  /*  296 */  iRegINoSp_rule,
  /*  297 */  iRegINoSp_rule,
  /*  298 */  iRegINoSp_rule,
  /*  299 */  iRegINoSp_rule,
  /*  300 */  iRegLNoSp_rule,
  /*  301 */  iRegNNoSp_rule,
  /*  302 */  iRegPNoSp_rule,
  /*  303 */  iRegINoSp_rule,
  /*  304 */  iRegINoSp_rule,
  /*  305 */  iRegINoSp_rule,
  /*  306 */  iRegLNoSp_rule,
  /*  307 */  iRegNNoSp_rule,
  /*  308 */  iRegPNoSp_rule,
  /*  309 */  iRegINoSp_rule,
  /*  310 */  iRegINoSp_rule,
  /*  311 */  iRegINoSp_rule,
  /*  312 */  iRegINoSp_rule,
  /*  313 */  iRegINoSp_rule,
  /*  314 */  iRegINoSp_rule,
  /*  315 */  iRegINoSp_rule,
  /*  316 */  iRegINoSp_rule,
  /*  317 */  iRegINoSp_rule,
  /*  318 */  iRegINoSp_rule,
  /*  319 */  iRegINoSp_rule,
  /*  320 */  iRegINoSp_rule,
  /*  321 */  iRegINoSp_rule,
  /*  322 */  iRegLNoSp_rule,
  /*  323 */  iRegINoSp_rule,
  /*  324 */  iRegPNoSp_rule,
  /*  325 */  iRegINoSp_rule,
  /*  326 */  iRegLNoSp_rule,
  /*  327 */  iRegINoSp_rule,
  /*  328 */  iRegPNoSp_rule,
  /*  329 */  iRegLNoSp_rule,
  /*  330 */  Universe_rule,
  /*  331 */  iRegLNoSp_rule,
  /*  332 */  Universe_rule,
  /*  333 */  iRegINoSp_rule,
  /*  334 */  Universe_rule,
  /*  335 */  iRegINoSp_rule,
  /*  336 */  Universe_rule,
  /*  337 */  iRegLNoSp_rule,
  /*  338 */  Universe_rule,
  /*  339 */  iRegLNoSp_rule,
  /*  340 */  Universe_rule,
  /*  341 */  iRegINoSp_rule,
  /*  342 */  Universe_rule,
  /*  343 */  iRegINoSp_rule,
  /*  344 */  Universe_rule,
  /*  345 */  iRegINoSp_rule,
  /*  346 */  iRegINoSp_rule,
  /*  347 */  iRegINoSp_rule,
  /*  348 */  iRegPNoSp_rule,
  /*  349 */  iRegLNoSp_rule,
  /*  350 */  iRegPNoSp_rule,
  /*  351 */  iRegLNoSp_rule,
  /*  352 */  iRegLNoSp_rule,
  /*  353 */  iRegINoSp_rule,
  /*  354 */  iRegINoSp_rule,
  /*  355 */  iRegLNoSp_rule,
  /*  356 */  iRegLNoSp_rule,
  /*  357 */  iRegINoSp_rule,
  /*  358 */  iRegLNoSp_rule,
  /*  359 */  iRegINoSp_rule,
  /*  360 */  iRegLNoSp_rule,
  /*  361 */  iRegLNoSp_rule,
  /*  362 */  iRegINoSp_rule,
  /*  363 */  iRegINoSp_rule,
  /*  364 */  iRegLNoSp_rule,
  /*  365 */  iRegLNoSp_rule,
  /*  366 */  iRegINoSp_rule,
  /*  367 */  iRegLNoSp_rule,
  /*  368 */  iRegINoSp_rule,
  /*  369 */  iRegINoSp_rule,
  /*  370 */  iRegINoSp_rule,
  /*  371 */  iRegINoSp_rule,
  /*  372 */  iRegINoSp_rule,
  /*  373 */  iRegINoSp_rule,
  /*  374 */  iRegLNoSp_rule,
  /*  375 */  iRegLNoSp_rule,
  /*  376 */  iRegLNoSp_rule,
  /*  377 */  iRegLNoSp_rule,
  /*  378 */  iRegLNoSp_rule,
  /*  379 */  iRegLNoSp_rule,
  /*  380 */  iRegLNoSp_rule,
  /*  381 */  iRegINoSp_rule,
  /*  382 */  iRegLNoSp_rule,
  /*  383 */  fRegF_rule,
  /*  384 */  fRegD_rule,
  /*  385 */  fRegF_rule,
  /*  386 */  fRegD_rule,
  /*  387 */  fRegF_rule,
  /*  388 */  fRegD_rule,
  /*  389 */  fRegF_rule,
  /*  390 */  fRegD_rule,
  /*  391 */  fRegF_rule,
  /*  392 */  fRegD_rule,
  /*  393 */  fRegF_rule,
  /*  394 */  fRegF_rule,
  /*  395 */  fRegD_rule,
  /*  396 */  fRegD_rule,
  /*  397 */  fRegF_rule,
  /*  398 */  fRegF_rule,
  /*  399 */  fRegD_rule,
  /*  400 */  fRegD_rule,
  /*  401 */  fRegF_rule,
  /*  402 */  fRegF_rule,
  /*  403 */  fRegD_rule,
  /*  404 */  fRegD_rule,
  /*  405 */  fRegF_rule,
  /*  406 */  fRegD_rule,
  /*  407 */  fRegF_rule,
  /*  408 */  fRegD_rule,
  /*  409 */  iRegINoSp_rule,
  /*  410 */  iRegINoSp_rule,
  /*  411 */  iRegINoSp_rule,
  /*  412 */  iRegINoSp_rule,
  /*  413 */  iRegINoSp_rule,
  /*  414 */  iRegINoSp_rule,
  /*  415 */  iRegLNoSp_rule,
  /*  416 */  iRegLNoSp_rule,
  /*  417 */  iRegLNoSp_rule,
  /*  418 */  iRegLNoSp_rule,
  /*  419 */  iRegLNoSp_rule,
  /*  420 */  iRegLNoSp_rule,
  /*  421 */  iRegINoSp_rule,
  /*  422 */  iRegLNoSp_rule,
  /*  423 */  iRegINoSp_rule,
  /*  424 */  iRegINoSp_rule,
  /*  425 */  Universe_rule,
  /*  426 */  Universe_rule,
  /*  427 */  Universe_rule,
  /*  428 */  Universe_rule,
  /*  429 */  Universe_rule,
  /*  430 */  Universe_rule,
  /*  431 */  Universe_rule,
  /*  432 */  Universe_rule,
  /*  433 */  Universe_rule,
  /*  434 */  iRegLNoSp_rule,
  /*  435 */  iRegINoSp_rule,
  /*  436 */  iRegLNoSp_rule,
  /*  437 */  fRegD_rule,
  /*  438 */  fRegF_rule,
  /*  439 */  iRegINoSp_rule,
  /*  440 */  fRegF_rule,
  /*  441 */  iRegLNoSp_rule,
  /*  442 */  fRegF_rule,
  /*  443 */  iRegINoSp_rule,
  /*  444 */  fRegD_rule,
  /*  445 */  iRegLNoSp_rule,
  /*  446 */  fRegD_rule,
  /*  447 */  iRegINoSp_rule,
  /*  448 */  iRegINoSp_rule,
  /*  449 */  iRegNNoSp_rule,
  /*  450 */  iRegPNoSp_rule,
  /*  451 */  iRegPNoSp_rule,
  /*  452 */  iRegNNoSp_rule,
  /*  453 */  iRegPNoSp_rule,
  /*  454 */  stackSlotI_rule,
  /*  455 */  stackSlotF_rule,
  /*  456 */  stackSlotL_rule,
  /*  457 */  stackSlotD_rule,
  /*  458 */  iRegINoSp_rule,
  /*  459 */  iRegINoSp_rule,
  /*  460 */  iRegINoSp_rule,
  /*  461 */  iRegINoSp_rule,
  /*  462 */  iRegINoSp_rule,
  /*  463 */  iRegINoSp_rule,
  /*  464 */  iRegINoSp_rule,
  /*  465 */  Universe_rule,
  /*  466 */  Universe_rule,
  /*  467 */  Universe_rule,
  /*  468 */  Universe_rule,
  /*  469 */  Universe_rule,
  /*  470 */  Universe_rule,
  /*  471 */  Universe_rule,
  /*  472 */  Universe_rule,
  /*  473 */  Universe_rule,
  /*  474 */  Universe_rule,
  /*  475 */  Universe_rule,
  /*  476 */  Universe_rule,
  /*  477 */  Universe_rule,
  /*  478 */  Universe_rule,
  /*  479 */  Universe_rule,
  /*  480 */  Universe_rule,
  /*  481 */  Universe_rule,
  /*  482 */  Universe_rule,
  /*  483 */  Universe_rule,
  /*  484 */  Universe_rule,
  /*  485 */  Universe_rule,
  /*  486 */  Universe_rule,
  /*  487 */  Universe_rule,
  /*  488 */  Universe_rule,
  /*  489 */  Universe_rule,
  /*  490 */  Universe_rule,
  /*  491 */  Universe_rule,
  /*  492 */  Universe_rule,
  /*  493 */  Universe_rule,
  /*  494 */  Universe_rule,
  /*  495 */  Universe_rule,
  /*  496 */  Universe_rule,
  /*  497 */  Universe_rule,
  /*  498 */  Universe_rule,
  /*  499 */  Universe_rule,
  /*  500 */  Universe_rule,
  /*  501 */  Universe_rule,
  /*  502 */  Universe_rule,
  /*  503 */  Universe_rule,
  /*  504 */  Universe_rule,
  /*  505 */  Universe_rule,
  /*  506 */  Universe_rule,
  /*  507 */  Universe_rule,
  /*  508 */  Universe_rule,
  /*  509 */  Universe_rule,
  /*  510 */  Universe_rule,
  /*  511 */  Universe_rule,
  /*  512 */  Universe_rule,
  /*  513 */  Universe_rule,
  /*  514 */  Universe_rule,
  /*  515 */  Universe_rule,
  /*  516 */  Universe_rule,
  /*  517 */  Universe_rule,
  /*  518 */  Universe_rule,
  /*  519 */  Universe_rule,
  /*  520 */  Universe_rule,
  /*  521 */  Universe_rule,
  /*  522 */  Universe_rule,
  /*  523 */  Universe_rule,
  /*  524 */  Universe_rule,
  /*  525 */  Universe_rule,
  /*  526 */  Universe_rule,
  /*  527 */  Universe_rule,
  /*  528 */  Universe_rule,
  /*  529 */  Universe_rule,
  /*  530 */  Universe_rule,
  /*  531 */  Universe_rule,
  /*  532 */  iRegINoSp_rule,
  /*  533 */  iRegINoSp_rule,
  /*  534 */  iRegINoSp_rule,
  /*  535 */  iRegLNoSp_rule,
  /*  536 */  iRegLNoSp_rule,
  /*  537 */  iRegINoSp_rule,
  /*  538 */  Universe_rule,
  /*  539 */  Universe_rule,
  /*  540 */  Universe_rule,
  /*  541 */  Universe_rule,
  /*  542 */  Universe_rule,
  /*  543 */  iRegP_R15_rule,
  /*  544 */  iRegI_R10_rule,
  /*  545 */  iRegI_R10_rule,
  /*  546 */  iRegI_R10_rule,
  /*  547 */  iRegI_R10_rule,
  /*  548 */  iRegI_R10_rule,
  /*  549 */  iRegI_R10_rule,
  /*  550 */  iRegI_R10_rule,
  /*  551 */  iRegI_R10_rule,
  /*  552 */  iRegI_R10_rule,
  /*  553 */  iRegI_R10_rule,
  /*  554 */  iRegI_R10_rule,
  /*  555 */  iRegI_R10_rule,
  /*  556 */  Universe_rule,
  /*  557 */  Universe_rule,
  /*  558 */  iRegI_R10_rule,
  /*  559 */  iRegI_R10_rule,
  /*  560 */  iRegI_R10_rule,
  /*  561 */  iRegI_R10_rule,
  /*  562 */  Universe_rule,
  /*  563 */  Universe_rule,
  /*  564 */  Universe_rule,
  /*  565 */  iRegP_R10_rule,
  /*  566 */  Universe_rule,
  /*  567 */  Universe_rule,
  /*  568 */  Universe_rule,
  /*  569 */  vReg_rule,
  /*  570 */  Universe_rule,
  /*  571 */  vReg_rule,
  /*  572 */  vReg_rule,
  /*  573 */  vReg_rule,
  /*  574 */  vReg_rule,
  /*  575 */  vReg_rule,
  /*  576 */  vReg_rule,
  /*  577 */  vReg_rule,
  /*  578 */  vReg_rule,
  /*  579 */  vReg_rule,
  /*  580 */  vReg_rule,
  /*  581 */  vReg_rule,
  /*  582 */  vReg_rule,
  /*  583 */  vReg_rule,
  /*  584 */  vReg_rule,
  /*  585 */  vReg_rule,
  /*  586 */  vReg_rule,
  /*  587 */  vReg_rule,
  /*  588 */  vReg_rule,
  /*  589 */  vReg_rule,
  /*  590 */  vReg_rule,
  /*  591 */  vReg_rule,
  /*  592 */  vReg_rule,
  /*  593 */  vReg_rule,
  /*  594 */  vReg_rule,
  /*  595 */  vReg_rule,
  /*  596 */  vReg_rule,
  /*  597 */  vReg_rule,
  /*  598 */  vReg_rule,
  /*  599 */  vReg_rule,
  /*  600 */  vReg_rule,
  /*  601 */  vReg_rule,
  /*  602 */  vReg_rule,
  /*  603 */  vReg_rule,
  /*  604 */  vReg_rule,
  /*  605 */  vReg_rule,
  /*  606 */  vReg_rule,
  /*  607 */  vReg_rule,
  /*  608 */  vReg_rule,
  /*  609 */  vReg_rule,
  /*  610 */  vReg_rule,
  /*  611 */  vReg_rule,
  /*  612 */  vReg_rule,
  /*  613 */  vReg_rule,
  /*  614 */  vReg_rule,
  /*  615 */  vReg_rule,
  /*  616 */  vReg_rule,
  /*  617 */  vReg_rule,
  /*  618 */  vReg_rule,
  /*  619 */  vReg_rule,
  /*  620 */  vReg_rule,
  /*  621 */  vReg_rule,
  /*  622 */  iRegINoSp_rule,
  /*  623 */  iRegINoSp_rule,
  /*  624 */  iRegINoSp_rule,
  /*  625 */  iRegLNoSp_rule,
  /*  626 */  fRegF_rule,
  /*  627 */  fRegD_rule,
  /*  628 */  iRegINoSp_rule,
  /*  629 */  iRegINoSp_rule,
  /*  630 */  iRegINoSp_rule,
  /*  631 */  iRegLNoSp_rule,
  /*  632 */  iRegINoSp_rule,
  /*  633 */  iRegINoSp_rule,
  /*  634 */  iRegINoSp_rule,
  /*  635 */  iRegLNoSp_rule,
  /*  636 */  fRegF_rule,
  /*  637 */  fRegD_rule,
  /*  638 */  fRegF_rule,
  /*  639 */  fRegD_rule,
  /*  640 */  vReg_rule,
  /*  641 */  vReg_rule,
  /*  642 */  vReg_rule,
  /*  643 */  vReg_rule,
  /*  644 */  vReg_rule,
  /*  645 */  vReg_rule,
  /*  646 */  vReg_rule,
  /*  647 */  vReg_rule,
  /*  648 */  vReg_rule,
  /*  649 */  vReg_rule,
  /*  650 */  vReg_rule,
  /*  651 */  vReg_rule,
  /*  652 */  vReg_rule,
  /*  653 */  vReg_rule,
  /*  654 */  vReg_rule,
  /*  655 */  vReg_rule,
  /*  656 */  vReg_rule,
  /*  657 */  vReg_rule,
  /*  658 */  vReg_rule,
  /*  659 */  vReg_rule,
  /*  660 */  vReg_rule,
  /*  661 */  vReg_rule,
  /*  662 */  vReg_rule,
  /*  663 */  vReg_rule,
  /*  664 */  vReg_rule,
  /*  665 */  vReg_rule,
  /*  666 */  vReg_rule,
  /*  667 */  vReg_rule,
  /*  668 */  vReg_rule,
  /*  669 */  vReg_rule,
  /*  670 */  vReg_rule,
  /*  671 */  vReg_rule,
  /*  672 */  vReg_rule,
  /*  673 */  vReg_rule,
  /*  674 */  vReg_rule,
  /*  675 */  vReg_rule,
  /*  676 */  vReg_rule,
  /*  677 */  vReg_rule,
  /*  678 */  vReg_rule,
  /*  679 */  vReg_rule,
  /*  680 */  vReg_rule,
  /*  681 */  iRegI_R10_rule,
  /*  682 */  iRegI_R10_rule,
  /*  683 */  iRegI_R10_rule,
  /*  684 */  iRegI_R10_rule,
  /*  685 */  iRegI_R10_rule,
  /*  686 */  iRegI_R10_rule,
  /*  687 */  iRegI_R10_rule,
  /*  688 */  iRegI_R10_rule,
  /*  689 */  Universe_rule,
  /*  690 */  iRegI_R10_rule,
  /*  691 */  iRegI_R10_rule,
  /*  692 */  iRegI_R10_rule,
  /*  693 */  iRegI_R10_rule,
  /*  694 */  iRegI_R10_rule,
  /*  695 */  Universe_rule,
  /*  696 */  iRegINoSp_rule,
  /*  697 */  iRegLNoSp_rule,
  /*  698 */  iRegINoSp_rule,
  /*  699 */  iRegLNoSp_rule,
  /*  700 */  iRegINoSp_rule,
  /*  701 */  iRegLNoSp_rule,
  /*  702 */  iRegINoSp_rule,
  /*  703 */  iRegINoSp_rule,
  /*  704 */  iRegINoSp_rule,
  /*  705 */  iRegINoSp_rule,
  /*  706 */  iRegLNoSp_rule,
  /*  707 */  iRegINoSp_rule,
  /*  708 */  iRegLNoSp_rule,
  /*  709 */  iRegINoSp_rule,
  /*  710 */  iRegINoSp_rule,
  /*  711 */  iRegPNoSp_rule,
  /*  712 */  iRegPNoSp_rule,
  /*  713 */  iRegLNoSp_rule,
  /*  714 */  iRegLNoSp_rule,
  /*  715 */  iRegLNoSp_rule,
  /*  716 */  iRegLNoSp_rule,
  /*  717 */  iRegINoSp_rule,
  /*  718 */  iRegINoSp_rule,
  /*  719 */  iRegINoSp_rule,
  /*  720 */  iRegINoSp_rule,
  /*  721 */  iRegLNoSp_rule,
  /*  722 */  iRegLNoSp_rule,
  /*  723 */  iRegINoSp_rule,
  /*  724 */  iRegINoSp_rule,
  /*  725 */  iRegLNoSp_rule,
  /*  726 */  iRegLNoSp_rule,
  /*  727 */  iRegINoSp_rule,
  /*  728 */  iRegINoSp_rule,
  /*  729 */  iRegINoSp_rule,
  /*  730 */  iRegINoSp_rule,
  /*  731 */  iRegNNoSp_rule,
  /*  732 */  iRegPNoSp_rule,
  /*  733 */  iRegINoSp_rule,
  /*  734 */  iRegNNoSp_rule,
  /*  735 */  iRegPNoSp_rule,
  /*  736 */  iRegINoSp_rule,
  /*  737 */  iRegINoSp_rule,
  /*  738 */  iRegINoSp_rule,
  /*  739 */  iRegPNoSp_rule,
  /*  740 */  iRegINoSp_rule,
  /*  741 */  iRegINoSp_rule,
  /*  742 */  iRegINoSp_rule,
  /*  743 */  iRegINoSp_rule,
  /*  744 */  iRegPNoSp_rule,
  /*  745 */  iRegPNoSp_rule,
  /*  746 */  iRegPNoSp_rule,
  /*  747 */  iRegPNoSp_rule,
  // last instruction
  0 // no trailing comma
};

const        int   leftOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  0,
  /*   76 */  0,
  /*   77 */  0,
  /*   78 */  0,
  /*   79 */  iRegP_rule,
  /*   80 */  iRegP_rule,
  /*   81 */  iRegN_rule,
  /*   82 */  _DecodeN_iRegN__rule,
  /*   83 */  _DecodeN_iRegN__rule,
  /*   84 */  javaThread_RegP_rule,
  /*   85 */  0,
  /*   86 */  0,
  /*   87 */  0,
  /*   88 */  0,
  /*   89 */  iRegL_rule,
  /*   90 */  0,
  /*   91 */  0,
  /*   92 */  0,
  /*   93 */  0,
  /*   94 */  0,
  /*   95 */  0,
  /*   96 */  0,
  // last operand
  /*   97 */  0,
  /*   98 */  0,
  /*   99 */  0,
  /*  100 */  0,
  /*  101 */  0,
  /*  102 */  0,
  /*  103 */  0,
  /*  104 */  0,
  // last operand class
  /*  105 */  iRegN_rule,
  /*  106 */  memory_rule,
  /*  107 */  memory_rule,
  /*  108 */  memory_rule,
  /*  109 */  memory_rule,
  /*  110 */  memory_rule,
  /*  111 */  _LoadI_memory__rule,
  /*  112 */  iRegP_rule,
  /*  113 */  iRegL_rule,
  /*  114 */  iRegI_rule,
  /*  115 */  iRegI_R12_rule,
  /*  116 */  iRegN_rule,
  /*  117 */  iRegL_rule,
  /*  118 */  iRegI_rule,
  /*  119 */  iRegIorL2I_rule,
  /*  120 */  iRegL_rule,
  /*  121 */  iRegP_rule,
  /*  122 */  fRegF_rule,
  /*  123 */  fRegD_rule,
  /*  124 */  fRegF_rule,
  /*  125 */  fRegD_rule,
  /*  126 */  _NegF_fRegF__rule,
  /*  127 */  fRegF_rule,
  /*  128 */  _NegD_fRegD__rule,
  /*  129 */  fRegD_rule,
  /*  130 */  fRegF_rule,
  /*  131 */  _ConvF2D_fRegF__rule,
  /*  132 */  iRegIorL2I_rule,
  /*  133 */  _DecodeN_iRegN__rule,
  /*  134 */  iRegI_rule,
  /*  135 */  iRegI_rule,
  /*  136 */  iRegL_rule,
  /*  137 */  iRegL_rule,
  /*  138 */  iRegP_rule,
  /*  139 */  iRegN_rule,
  /*  140 */  fRegF_rule,
  /*  141 */  fRegD_rule,
  /*  142 */  iRegI_rule,
  /*  143 */  iRegI_rule,
  /*  144 */  iRegL_rule,
  /*  145 */  iRegL_rule,
  /*  146 */  iRegP_rule,
  /*  147 */  iRegN_rule,
  /*  148 */  _DecodeN_iRegN__rule,
  /*  149 */  cmpOp_rule,
  /*  150 */  iRegINoSp_rule,
  /*  151 */  cmpOpU_rule,
  /*  152 */  cmpOp_rule,
  /*  153 */  iRegLNoSp_rule,
  /*  154 */  cmpOpU_rule,
  /*  155 */  iRegP_R14_rule,
  /*  156 */  iRegP_R11_rule,
  /*  157 */  iRegP_R13_rule,
  /*  158 */  iRegP_R13_rule,
  /*  159 */  iRegP_R13_rule,
  /*  160 */  iRegP_R11_rule,
  /*  161 */  vReg_rule,
  /*  162 */  vReg_rule,
  /*  163 */  _NegVF_vReg__rule,
  /*  164 */  vReg_rule,
  /*  165 */  vReg_rule,
  /*  166 */  _NegVD_vReg__rule,
  /*  167 */  vReg_rule,
  /*  168 */  vReg_rule,
  /*  169 */  vReg_rule,
  /*  170 */  vReg_rule,
  /*  171 */  vReg_rule,
  /*  172 */  immI_rule,
  /*  173 */  immI_rule,
  /*  174 */  iRegP_R11_rule,
  /*  175 */  iRegIorL2I_rule,
  /*  176 */  iRegIorL2I_rule,
  /*  177 */  iRegL_rule,
  /*  178 */  _ConvI2L_iRegI__rule,
  /*  179 */  iRegI_rule,
  /*  180 */  iRegL_rule,
  // last internally defined operand
  /*  181 */  immI_rule,
  /*  182 */  immL_rule,
  /*  183 */  immP_rule,
  /*  184 */  immP0_rule,
  /*  185 */  immP_1_rule,
  /*  186 */  immByteMapBase_rule,
  /*  187 */  immN_rule,
  /*  188 */  immN0_rule,
  /*  189 */  immNKlass_rule,
  /*  190 */  immF_rule,
  /*  191 */  immF0_rule,
  /*  192 */  immD_rule,
  /*  193 */  immD0_rule,
  /*  194 */  memory_rule,
  /*  195 */  indirect_rule,
  /*  196 */  indirect_rule,
  /*  197 */  fRegF_rule,
  /*  198 */  fRegD_rule,
  /*  199 */  iRegIorL2I_rule,
  /*  200 */  iRegL_rule,
  /*  201 */  fRegF_rule,
  /*  202 */  fRegD_rule,
  /*  203 */  iRegL_rule,
  /*  204 */  iRegP_rule,
  /*  205 */  iRegPNoSp_rule,
  /*  206 */  iRegL_rule,
  /*  207 */  iRegI_rule,
  /*  208 */  iRegPNoSp_rule,
  /*  209 */  fRegF_rule,
  /*  210 */  fRegD_rule,
  /*  211 */  vReg_rule,
  /*  212 */  iRegI_rule,
  /*  213 */  iRegP_rule,
  /*  214 */  stackSlotF_rule,
  /*  215 */  stackSlotI_rule,
  /*  216 */  stackSlotD_rule,
  /*  217 */  stackSlotL_rule,
  /*  218 */  fRegF_rule,
  /*  219 */  iRegI_rule,
  /*  220 */  fRegD_rule,
  /*  221 */  iRegL_rule,
  /*  222 */  _PartialSubtypeCheck_iRegP_R14_iRegP_R10_rule,
  /*  223 */  0,
  /*  224 */  iRegP_rule,
  /*  225 */  iRegP_rule,
  /*  226 */  vReg_rule,
  /*  227 */  vReg_rule,
  /*  228 */  vReg_rule,
  /*  229 */  vReg_rule,
  /*  230 */  iRegIorL2I_rule,
  /*  231 */  iRegIorL2I_rule,
  /*  232 */  iRegIorL2I_rule,
  /*  233 */  iRegIorL2I_rule,
  /*  234 */  iRegIorL2I_rule,
  /*  235 */  iRegIorL2I_rule,
  /*  236 */  iRegIorL2I_rule,
  /*  237 */  iRegIorL2I_rule,
  /*  238 */  vReg_rule,
  /*  239 */  vReg_rule,
  /*  240 */  iRegIorL2I_rule,
  /*  241 */  iRegL_rule,
  /*  242 */  iRegIorL2I_rule,
  /*  243 */  iRegL_rule,
  /*  244 */  iRegIorL2I_rule,
  /*  245 */  iRegL_rule,
  /*  246 */  iRegI_rule,
  /*  247 */  iRegL_rule,
  /*  248 */  memory_rule,
  /*  249 */  _LoadB_memory__rule,
  /*  250 */  memory_rule,
  /*  251 */  _LoadUB_memory__rule,
  /*  252 */  memory_rule,
  /*  253 */  _LoadS_memory__rule,
  /*  254 */  memory_rule,
  /*  255 */  _LoadUS_memory__rule,
  /*  256 */  memory_rule,
  /*  257 */  _LoadI_memory__rule,
  /*  258 */  _ConvI2L__LoadI_memory___rule,
  /*  259 */  memory_rule,
  /*  260 */  memory_rule,
  /*  261 */  memory_rule,
  /*  262 */  memory_rule,
  /*  263 */  memory_rule,
  /*  264 */  memory_rule,
  /*  265 */  memory_rule,
  /*  266 */  memory_rule,
  /*  267 */  memory_rule,
  /*  268 */  memory_rule,
  /*  269 */  memory_rule,
  /*  270 */  memory_rule,
  /*  271 */  memory_rule,
  /*  272 */  memory_rule,
  /*  273 */  memory_rule,
  /*  274 */  memory_rule,
  /*  275 */  memory_rule,
  /*  276 */  memory_rule,
  /*  277 */  memory_rule,
  /*  278 */  memory_rule,
  /*  279 */  memory_rule,
  /*  280 */  memory_rule,
  /*  281 */  memory_rule,
  /*  282 */  memory_rule,
  /*  283 */  memory_rule,
  /*  284 */  indirect_rule,
  /*  285 */  indirect_rule,
  /*  286 */  indirect_rule,
  /*  287 */  indirect_rule,
  /*  288 */  indirect_rule,
  /*  289 */  indirect_rule,
  /*  290 */  indirect_rule,
  /*  291 */  indirect_rule,
  /*  292 */  indirect_rule,
  /*  293 */  indirect_rule,
  /*  294 */  indirect_rule,
  /*  295 */  indirect_rule,
  /*  296 */  indirect_rule,
  /*  297 */  indirect_rule,
  /*  298 */  indirect_rule,
  /*  299 */  indirect_rule,
  /*  300 */  indirect_rule,
  /*  301 */  indirect_rule,
  /*  302 */  indirect_rule,
  /*  303 */  indirect_rule,
  /*  304 */  indirect_rule,
  /*  305 */  indirect_rule,
  /*  306 */  indirect_rule,
  /*  307 */  indirect_rule,
  /*  308 */  indirect_rule,
  /*  309 */  indirect_rule,
  /*  310 */  indirect_rule,
  /*  311 */  indirect_rule,
  /*  312 */  indirect_rule,
  /*  313 */  indirect_rule,
  /*  314 */  indirect_rule,
  /*  315 */  indirect_rule,
  /*  316 */  indirect_rule,
  /*  317 */  indirect_rule,
  /*  318 */  indirect_rule,
  /*  319 */  indirect_rule,
  /*  320 */  indirect_rule,
  /*  321 */  indirect_rule,
  /*  322 */  indirect_rule,
  /*  323 */  indirect_rule,
  /*  324 */  indirect_rule,
  /*  325 */  indirect_rule,
  /*  326 */  indirect_rule,
  /*  327 */  indirect_rule,
  /*  328 */  indirect_rule,
  /*  329 */  indirect_rule,
  /*  330 */  indirect_rule,
  /*  331 */  indirect_rule,
  /*  332 */  indirect_rule,
  /*  333 */  indirect_rule,
  /*  334 */  indirect_rule,
  /*  335 */  indirect_rule,
  /*  336 */  indirect_rule,
  /*  337 */  indirect_rule,
  /*  338 */  indirect_rule,
  /*  339 */  indirect_rule,
  /*  340 */  indirect_rule,
  /*  341 */  indirect_rule,
  /*  342 */  indirect_rule,
  /*  343 */  indirect_rule,
  /*  344 */  indirect_rule,
  /*  345 */  iRegIorL2I_rule,
  /*  346 */  iRegIorL2I_rule,
  /*  347 */  _ConvL2I_iRegL__rule,
  /*  348 */  iRegP_rule,
  /*  349 */  _ConvI2L_iRegI__rule,
  /*  350 */  iRegP_rule,
  /*  351 */  iRegL_rule,
  /*  352 */  iRegL_rule,
  /*  353 */  iRegIorL2I_rule,
  /*  354 */  iRegIorL2I_rule,
  /*  355 */  iRegL_rule,
  /*  356 */  iRegL_rule,
  /*  357 */  immI0_rule,
  /*  358 */  immL0_rule,
  /*  359 */  iRegIorL2I_rule,
  /*  360 */  iRegL_rule,
  /*  361 */  iRegL_rule,
  /*  362 */  iRegIorL2I_rule,
  /*  363 */  _RShiftI_iRegIorL2I_immI_31_rule,
  /*  364 */  iRegL_rule,
  /*  365 */  _RShiftL_iRegL_immI_63_rule,
  /*  366 */  iRegIorL2I_rule,
  /*  367 */  iRegL_rule,
  /*  368 */  iRegIorL2I_rule,
  /*  369 */  iRegIorL2I_rule,
  /*  370 */  iRegIorL2I_rule,
  /*  371 */  iRegIorL2I_rule,
  /*  372 */  iRegIorL2I_rule,
  /*  373 */  iRegIorL2I_rule,
  /*  374 */  iRegL_rule,
  /*  375 */  iRegL_rule,
  /*  376 */  iRegL_rule,
  /*  377 */  iRegL_rule,
  /*  378 */  _CastP2X_iRegP__rule,
  /*  379 */  iRegL_rule,
  /*  380 */  iRegL_rule,
  /*  381 */  iRegI_rule,
  /*  382 */  iRegL_rule,
  /*  383 */  fRegF_rule,
  /*  384 */  fRegD_rule,
  /*  385 */  fRegF_rule,
  /*  386 */  fRegD_rule,
  /*  387 */  fRegF_rule,
  /*  388 */  fRegD_rule,
  /*  389 */  fRegF_rule,
  /*  390 */  fRegD_rule,
  /*  391 */  _NegF_fRegF__rule,
  /*  392 */  _NegD_fRegD__rule,
  /*  393 */  fRegF_rule,
  /*  394 */  fRegF_rule,
  /*  395 */  fRegD_rule,
  /*  396 */  fRegD_rule,
  /*  397 */  _NegF_fRegF__rule,
  /*  398 */  _NegF_fRegF__rule,
  /*  399 */  _NegD_fRegD__rule,
  /*  400 */  _NegD_fRegD__rule,
  /*  401 */  fRegF_rule,
  /*  402 */  fRegF_rule,
  /*  403 */  fRegD_rule,
  /*  404 */  fRegD_rule,
  /*  405 */  fRegF_rule,
  /*  406 */  fRegD_rule,
  /*  407 */  _SqrtD__ConvF2D_fRegF___rule,
  /*  408 */  fRegD_rule,
  /*  409 */  iRegI_rule,
  /*  410 */  iRegI_rule,
  /*  411 */  iRegI_rule,
  /*  412 */  iRegI_rule,
  /*  413 */  iRegI_rule,
  /*  414 */  iRegI_rule,
  /*  415 */  iRegL_rule,
  /*  416 */  iRegL_rule,
  /*  417 */  iRegL_rule,
  /*  418 */  iRegL_rule,
  /*  419 */  iRegL_rule,
  /*  420 */  iRegL_rule,
  /*  421 */  iRegIorL2I_rule,
  /*  422 */  iRegL_rule,
  /*  423 */  iRegIorL2I_rule,
  /*  424 */  iRegIorL2I_rule,
  /*  425 */  0,
  /*  426 */  0,
  /*  427 */  0,
  /*  428 */  0,
  /*  429 */  0,
  /*  430 */  0,
  /*  431 */  0,
  /*  432 */  0,
  /*  433 */  0,
  /*  434 */  iRegIorL2I_rule,
  /*  435 */  iRegL_rule,
  /*  436 */  _ConvI2L_iRegIorL2I__rule,
  /*  437 */  fRegF_rule,
  /*  438 */  fRegD_rule,
  /*  439 */  fRegF_rule,
  /*  440 */  iRegIorL2I_rule,
  /*  441 */  fRegF_rule,
  /*  442 */  iRegL_rule,
  /*  443 */  fRegD_rule,
  /*  444 */  iRegIorL2I_rule,
  /*  445 */  fRegD_rule,
  /*  446 */  iRegL_rule,
  /*  447 */  _CastP2X_iRegP__rule,
  /*  448 */  _CastP2X__DecodeN_iRegN___rule,
  /*  449 */  iRegP_rule,
  /*  450 */  iRegN_rule,
  /*  451 */  iRegN_rule,
  /*  452 */  iRegP_rule,
  /*  453 */  iRegN_rule,
  /*  454 */  fRegF_rule,
  /*  455 */  iRegI_rule,
  /*  456 */  fRegD_rule,
  /*  457 */  iRegL_rule,
  /*  458 */  fRegF_rule,
  /*  459 */  fRegD_rule,
  /*  460 */  iRegL_rule,
  /*  461 */  iRegI_rule,
  /*  462 */  iRegIorL2I_rule,
  /*  463 */  iRegI_rule,
  /*  464 */  iRegI_rule,
  /*  465 */  0,
  /*  466 */  cmpOpEqNe_rule,
  /*  467 */  cmpOp_rule,
  /*  468 */  cmpOp_rule,
  /*  469 */  cmpOpU_rule,
  /*  470 */  cmpOpU_rule,
  /*  471 */  cmpOp_rule,
  /*  472 */  cmpOp_rule,
  /*  473 */  cmpOpU_rule,
  /*  474 */  cmpOpU_rule,
  /*  475 */  cmpOpU_rule,
  /*  476 */  cmpOpU_rule,
  /*  477 */  cmpOpU_rule,
  /*  478 */  cmpOpU_rule,
  /*  479 */  cmpOp_rule,
  /*  480 */  cmpOp_rule,
  /*  481 */  cmpOp_rule,
  /*  482 */  cmpOp_rule,
  /*  483 */  cmpOp_rule,
  /*  484 */  cmpOp_rule,
  /*  485 */  cmpOpUEqNeLeGt_rule,
  /*  486 */  cmpOpUEqNeLeGt_rule,
  /*  487 */  cmpOp_rule,
  /*  488 */  cmpOp_rule,
  /*  489 */  cmpOpUEqNeLeGt_rule,
  /*  490 */  cmpOpUEqNeLeGt_rule,
  /*  491 */  cmpOpEqNe_rule,
  /*  492 */  cmpOpEqNe_rule,
  /*  493 */  cmpOpEqNe_rule,
  /*  494 */  cmpOpEqNe_rule,
  /*  495 */  cmpOpEqNe_rule,
  /*  496 */  cmpOpEqNe_rule,
  /*  497 */  cmpOp_rule,
  /*  498 */  cmpOp_rule,
  /*  499 */  cmpOp_rule,
  /*  500 */  cmpOpU_rule,
  /*  501 */  cmpOpU_rule,
  /*  502 */  cmpOp_rule,
  /*  503 */  cmpOp_rule,
  /*  504 */  cmpOpU_rule,
  /*  505 */  cmpOpU_rule,
  /*  506 */  cmpOpU_rule,
  /*  507 */  cmpOpU_rule,
  /*  508 */  cmpOpU_rule,
  /*  509 */  cmpOpU_rule,
  /*  510 */  cmpOp_rule,
  /*  511 */  cmpOp_rule,
  /*  512 */  cmpOp_rule,
  /*  513 */  cmpOp_rule,
  /*  514 */  cmpOp_rule,
  /*  515 */  cmpOp_rule,
  /*  516 */  cmpOpUEqNeLeGt_rule,
  /*  517 */  cmpOpUEqNeLeGt_rule,
  /*  518 */  cmpOpULtGe_rule,
  /*  519 */  cmpOpULtGe_rule,
  /*  520 */  cmpOp_rule,
  /*  521 */  cmpOp_rule,
  /*  522 */  cmpOpUEqNeLeGt_rule,
  /*  523 */  cmpOpUEqNeLeGt_rule,
  /*  524 */  cmpOpULtGe_rule,
  /*  525 */  cmpOpULtGe_rule,
  /*  526 */  cmpOpEqNe_rule,
  /*  527 */  cmpOpEqNe_rule,
  /*  528 */  cmpOpEqNe_rule,
  /*  529 */  cmpOpEqNe_rule,
  /*  530 */  cmpOpEqNe_rule,
  /*  531 */  cmpOpEqNe_rule,
  /*  532 */  _Binary_cmpOp__CmpI_iRegI_iRegI_rule,
  /*  533 */  _Binary_cmpOpU__CmpU_iRegI_iRegI_rule,
  /*  534 */  _Binary_cmpOp__CmpL_iRegL_iRegL_rule,
  /*  535 */  _Binary_cmpOp__CmpL_iRegL_iRegL_rule,
  /*  536 */  _Binary_cmpOpU__CmpUL_iRegL_iRegL_rule,
  /*  537 */  _Binary_cmpOpU__CmpUL_iRegL_iRegL_rule,
  /*  538 */  0,
  /*  539 */  0,
  /*  540 */  0,
  /*  541 */  0,
  /*  542 */  0,
  /*  543 */  iRegP_R14_rule,
  /*  544 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  545 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  546 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  547 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  548 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  549 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  550 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  551 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  552 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  553 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  554 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  555 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  556 */  iRegL_R29_rule,
  /*  557 */  immL_rule,
  /*  558 */  _Binary_iRegP_R11_iRegP_R13_rule,
  /*  559 */  _Binary_iRegP_R11_iRegP_R13_rule,
  /*  560 */  iRegP_R11_rule,
  /*  561 */  iRegP_R11_rule,
  /*  562 */  iRegP_rule,
  /*  563 */  iRegPNoSp_rule,
  /*  564 */  iRegPNoSp_rule,
  /*  565 */  0,
  /*  566 */  0,
  /*  567 */  0,
  /*  568 */  0,
  /*  569 */  vmemA_rule,
  /*  570 */  vmemA_rule,
  /*  571 */  vReg_rule,
  /*  572 */  vReg_rule,
  /*  573 */  vReg_rule,
  /*  574 */  vReg_rule,
  /*  575 */  vReg_rule,
  /*  576 */  vReg_rule,
  /*  577 */  vReg_rule,
  /*  578 */  vReg_rule,
  /*  579 */  vReg_rule,
  /*  580 */  vReg_rule,
  /*  581 */  vReg_rule,
  /*  582 */  vReg_rule,
  /*  583 */  vReg_rule,
  /*  584 */  vReg_rule,
  /*  585 */  vReg_rule,
  /*  586 */  vReg_rule,
  /*  587 */  vReg_rule,
  /*  588 */  vReg_rule,
  /*  589 */  vReg_rule,
  /*  590 */  vReg_rule,
  /*  591 */  vReg_rule,
  /*  592 */  vReg_rule,
  /*  593 */  vReg_rule,
  /*  594 */  vReg_rule,
  /*  595 */  vReg_rule,
  /*  596 */  vReg_rule,
  /*  597 */  vReg_rule,
  /*  598 */  _NegVF_vReg__rule,
  /*  599 */  _NegVF_vReg__rule,
  /*  600 */  _NegVD_vReg__rule,
  /*  601 */  _NegVD_vReg__rule,
  /*  602 */  _NegVF_vReg__rule,
  /*  603 */  _NegVD_vReg__rule,
  /*  604 */  vReg_rule,
  /*  605 */  _MulVB_vReg_vReg_rule,
  /*  606 */  vReg_rule,
  /*  607 */  _MulVS_vReg_vReg_rule,
  /*  608 */  vReg_rule,
  /*  609 */  _MulVI_vReg_vReg_rule,
  /*  610 */  vReg_rule,
  /*  611 */  _MulVL_vReg_vReg_rule,
  /*  612 */  vReg_rule,
  /*  613 */  vReg_rule,
  /*  614 */  vReg_rule,
  /*  615 */  vReg_rule,
  /*  616 */  vReg_rule,
  /*  617 */  vReg_rule,
  /*  618 */  vReg_rule,
  /*  619 */  vReg_rule,
  /*  620 */  vReg_rule,
  /*  621 */  vReg_rule,
  /*  622 */  iRegIorL2I_rule,
  /*  623 */  iRegIorL2I_rule,
  /*  624 */  iRegIorL2I_rule,
  /*  625 */  iRegL_rule,
  /*  626 */  fRegF_rule,
  /*  627 */  fRegD_rule,
  /*  628 */  iRegI_rule,
  /*  629 */  iRegI_rule,
  /*  630 */  iRegIorL2I_rule,
  /*  631 */  iRegL_rule,
  /*  632 */  iRegI_rule,
  /*  633 */  iRegI_rule,
  /*  634 */  iRegIorL2I_rule,
  /*  635 */  iRegL_rule,
  /*  636 */  fRegF_rule,
  /*  637 */  fRegD_rule,
  /*  638 */  fRegF_rule,
  /*  639 */  fRegD_rule,
  /*  640 */  vReg_rule,
  /*  641 */  iRegIorL2I_rule,
  /*  642 */  iRegIorL2I_rule,
  /*  643 */  iRegIorL2I_rule,
  /*  644 */  iRegL_rule,
  /*  645 */  immI5_rule,
  /*  646 */  immI5_rule,
  /*  647 */  immI5_rule,
  /*  648 */  immL5_rule,
  /*  649 */  fRegF_rule,
  /*  650 */  fRegD_rule,
  /*  651 */  vReg_rule,
  /*  652 */  vReg_rule,
  /*  653 */  vReg_rule,
  /*  654 */  vReg_rule,
  /*  655 */  vReg_rule,
  /*  656 */  vReg_rule,
  /*  657 */  vReg_rule,
  /*  658 */  vReg_rule,
  /*  659 */  vReg_rule,
  /*  660 */  vReg_rule,
  /*  661 */  vReg_rule,
  /*  662 */  vReg_rule,
  /*  663 */  vReg_rule,
  /*  664 */  vReg_rule,
  /*  665 */  vReg_rule,
  /*  666 */  vReg_rule,
  /*  667 */  vReg_rule,
  /*  668 */  vReg_rule,
  /*  669 */  vReg_rule,
  /*  670 */  vReg_rule,
  /*  671 */  vReg_rule,
  /*  672 */  vReg_rule,
  /*  673 */  vReg_rule,
  /*  674 */  vReg_rule,
  /*  675 */  vReg_rule,
  /*  676 */  vReg_rule,
  /*  677 */  vReg_rule,
  /*  678 */  vReg_rule,
  /*  679 */  vReg_rule,
  /*  680 */  vReg_rule,
  /*  681 */  _Binary_iRegP_R11_iRegP_R13_rule,
  /*  682 */  _Binary_iRegP_R11_iRegP_R13_rule,
  /*  683 */  iRegP_R11_rule,
  /*  684 */  iRegP_R11_rule,
  /*  685 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  686 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  687 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  688 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  689 */  iRegP_R10_rule,
  /*  690 */  iRegP_R12_rule,
  /*  691 */  iRegP_R12_rule,
  /*  692 */  iRegP_R11_rule,
  /*  693 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  694 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  695 */  iRegL_R29_rule,
  /*  696 */  iRegI_rule,
  /*  697 */  iRegL_rule,
  /*  698 */  iRegI_rule,
  /*  699 */  iRegL_rule,
  /*  700 */  iRegI_rule,
  /*  701 */  iRegL_rule,
  /*  702 */  _CastP2X_iRegP__rule,
  /*  703 */  _LShiftI_iRegIorL2I_immI_24_rule,
  /*  704 */  _LShiftI_iRegIorL2I_immI_16_rule,
  /*  705 */  iRegIorL2I_rule,
  /*  706 */  _ConvI2L_iRegIorL2I__rule,
  /*  707 */  iRegIorL2I_rule,
  /*  708 */  iRegL_rule,
  /*  709 */  iRegIorL2I_rule,
  /*  710 */  iRegIorL2I_rule,
  /*  711 */  iRegP_rule,
  /*  712 */  iRegP_rule,
  /*  713 */  iRegL_rule,
  /*  714 */  _LShiftL_iRegL_immIScale_rule,
  /*  715 */  iRegL_rule,
  /*  716 */  _LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  717 */  iRegI_rule,
  /*  718 */  iRegI_rule,
  /*  719 */  iRegI_rule,
  /*  720 */  _XorI_iRegI_immI_M1_rule,
  /*  721 */  iRegL_rule,
  /*  722 */  _XorL_iRegL_immL_M1_rule,
  /*  723 */  iRegI_rule,
  /*  724 */  _XorI_iRegI_immI_M1_rule,
  /*  725 */  iRegL_rule,
  /*  726 */  _XorL_iRegL_immL_M1_rule,
  /*  727 */  indirect_rule,
  /*  728 */  indirect_rule,
  /*  729 */  indirect_rule,
  /*  730 */  indirect_rule,
  /*  731 */  indirect_rule,
  /*  732 */  indirect_rule,
  /*  733 */  indirect_rule,
  /*  734 */  indirect_rule,
  /*  735 */  indirect_rule,
  /*  736 */  indirect_rule,
  /*  737 */  indirect_rule,
  /*  738 */  indirect_rule,
  /*  739 */  memory_rule,
  /*  740 */  indirect_rule,
  /*  741 */  indirect_rule,
  /*  742 */  indirect_rule,
  /*  743 */  indirect_rule,
  /*  744 */  indirect_rule,
  /*  745 */  indirect_rule,
  /*  746 */  indirect_rule,
  /*  747 */  indirect_rule,
  // last instruction
  0 // no trailing comma
};

const        int   rightOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  0,
  /*   76 */  0,
  /*   77 */  0,
  /*   78 */  0,
  /*   79 */  immIOffset_rule,
  /*   80 */  immLOffset_rule,
  /*   81 */  0,
  /*   82 */  immIOffset_rule,
  /*   83 */  immLOffset_rule,
  /*   84 */  immL_pc_off_rule,
  /*   85 */  0,
  /*   86 */  0,
  /*   87 */  0,
  /*   88 */  0,
  /*   89 */  0,
  /*   90 */  0,
  /*   91 */  0,
  /*   92 */  0,
  /*   93 */  0,
  /*   94 */  0,
  /*   95 */  0,
  /*   96 */  0,
  // last operand
  /*   97 */  0,
  /*   98 */  0,
  /*   99 */  0,
  /*  100 */  0,
  /*  101 */  0,
  /*  102 */  0,
  /*  103 */  0,
  /*  104 */  0,
  // last operand class
  /*  105 */  0,
  /*  106 */  0,
  /*  107 */  0,
  /*  108 */  0,
  /*  109 */  0,
  /*  110 */  0,
  /*  111 */  0,
  /*  112 */  iRegP_rule,
  /*  113 */  iRegL_rule,
  /*  114 */  iRegI_rule,
  /*  115 */  iRegI_R13_rule,
  /*  116 */  iRegN_rule,
  /*  117 */  0,
  /*  118 */  0,
  /*  119 */  immI_31_rule,
  /*  120 */  immI_63_rule,
  /*  121 */  0,
  /*  122 */  fRegF_rule,
  /*  123 */  fRegD_rule,
  /*  124 */  0,
  /*  125 */  0,
  /*  126 */  fRegF_rule,
  /*  127 */  _NegF_fRegF__rule,
  /*  128 */  fRegD_rule,
  /*  129 */  _NegD_fRegD__rule,
  /*  130 */  0,
  /*  131 */  0,
  /*  132 */  0,
  /*  133 */  0,
  /*  134 */  iRegI_rule,
  /*  135 */  iRegI_rule,
  /*  136 */  iRegL_rule,
  /*  137 */  iRegL_rule,
  /*  138 */  iRegP_rule,
  /*  139 */  iRegN_rule,
  /*  140 */  fRegF_rule,
  /*  141 */  fRegD_rule,
  /*  142 */  immI0_rule,
  /*  143 */  immI0_rule,
  /*  144 */  immL0_rule,
  /*  145 */  immL0_rule,
  /*  146 */  immP0_rule,
  /*  147 */  immN0_rule,
  /*  148 */  immP0_rule,
  /*  149 */  _CmpI_iRegI_iRegI_rule,
  /*  150 */  iRegI_rule,
  /*  151 */  _CmpU_iRegI_iRegI_rule,
  /*  152 */  _CmpL_iRegL_iRegL_rule,
  /*  153 */  iRegL_rule,
  /*  154 */  _CmpUL_iRegL_iRegL_rule,
  /*  155 */  iRegP_R10_rule,
  /*  156 */  iRegI_R12_rule,
  /*  157 */  iRegI_R14_rule,
  /*  158 */  immI_le_4_rule,
  /*  159 */  immI_1_rule,
  /*  160 */  iRegP_R13_rule,
  /*  161 */  vReg_rule,
  /*  162 */  0,
  /*  163 */  vReg_rule,
  /*  164 */  _NegVF_vReg__rule,
  /*  165 */  0,
  /*  166 */  vReg_rule,
  /*  167 */  _NegVD_vReg__rule,
  /*  168 */  vReg_rule,
  /*  169 */  vReg_rule,
  /*  170 */  vReg_rule,
  /*  171 */  vReg_rule,
  /*  172 */  0,
  /*  173 */  0,
  /*  174 */  iRegI_R13_rule,
  /*  175 */  immI_24_rule,
  /*  176 */  immI_16_rule,
  /*  177 */  immIScale_rule,
  /*  178 */  immIScale_rule,
  /*  179 */  immI_M1_rule,
  /*  180 */  immL_M1_rule,
  // last internally defined operand
  /*  181 */  0,
  /*  182 */  0,
  /*  183 */  0,
  /*  184 */  0,
  /*  185 */  0,
  /*  186 */  0,
  /*  187 */  0,
  /*  188 */  0,
  /*  189 */  0,
  /*  190 */  0,
  /*  191 */  0,
  /*  192 */  0,
  /*  193 */  0,
  /*  194 */  _Binary_iRegP_iRegP_rule,
  /*  195 */  _Binary_iRegL_iRegL_rule,
  /*  196 */  _Binary_iRegI_iRegI_rule,
  /*  197 */  0,
  /*  198 */  0,
  /*  199 */  0,
  /*  200 */  0,
  /*  201 */  0,
  /*  202 */  0,
  /*  203 */  0,
  /*  204 */  0,
  /*  205 */  0,
  /*  206 */  0,
  /*  207 */  0,
  /*  208 */  0,
  /*  209 */  0,
  /*  210 */  0,
  /*  211 */  0,
  /*  212 */  0,
  /*  213 */  0,
  /*  214 */  0,
  /*  215 */  0,
  /*  216 */  0,
  /*  217 */  0,
  /*  218 */  0,
  /*  219 */  0,
  /*  220 */  0,
  /*  221 */  0,
  /*  222 */  immP0_rule,
  /*  223 */  0,
  /*  224 */  iRegP_rule,
  /*  225 */  iRegP_rule,
  /*  226 */  0,
  /*  227 */  0,
  /*  228 */  0,
  /*  229 */  0,
  /*  230 */  0,
  /*  231 */  0,
  /*  232 */  0,
  /*  233 */  0,
  /*  234 */  0,
  /*  235 */  0,
  /*  236 */  0,
  /*  237 */  0,
  /*  238 */  0,
  /*  239 */  0,
  /*  240 */  0,
  /*  241 */  0,
  /*  242 */  0,
  /*  243 */  0,
  /*  244 */  0,
  /*  245 */  0,
  /*  246 */  0,
  /*  247 */  0,
  /*  248 */  0,
  /*  249 */  0,
  /*  250 */  0,
  /*  251 */  0,
  /*  252 */  0,
  /*  253 */  0,
  /*  254 */  0,
  /*  255 */  0,
  /*  256 */  0,
  /*  257 */  0,
  /*  258 */  immL_32bits_rule,
  /*  259 */  0,
  /*  260 */  0,
  /*  261 */  0,
  /*  262 */  0,
  /*  263 */  0,
  /*  264 */  0,
  /*  265 */  0,
  /*  266 */  0,
  /*  267 */  immI0_rule,
  /*  268 */  immI0_rule,
  /*  269 */  iRegIorL2I_rule,
  /*  270 */  immI0_rule,
  /*  271 */  iRegIorL2I_rule,
  /*  272 */  immI0_rule,
  /*  273 */  iRegIorL2I_rule,
  /*  274 */  immI0_rule,
  /*  275 */  iRegL_rule,
  /*  276 */  immL0_rule,
  /*  277 */  iRegP_rule,
  /*  278 */  immP0_rule,
  /*  279 */  iRegN_rule,
  /*  280 */  immN0_rule,
  /*  281 */  fRegF_rule,
  /*  282 */  fRegD_rule,
  /*  283 */  iRegN_rule,
  /*  284 */  0,
  /*  285 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  286 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  287 */  _Binary_iRegI_iRegI_rule,
  /*  288 */  _Binary_iRegL_iRegL_rule,
  /*  289 */  _Binary_iRegP_iRegP_rule,
  /*  290 */  _Binary_iRegN_iRegN_rule,
  /*  291 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  292 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  293 */  _Binary_iRegI_iRegI_rule,
  /*  294 */  _Binary_iRegL_iRegL_rule,
  /*  295 */  _Binary_iRegP_iRegP_rule,
  /*  296 */  _Binary_iRegN_iRegN_rule,
  /*  297 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  298 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  299 */  _Binary_iRegI_iRegI_rule,
  /*  300 */  _Binary_iRegL_iRegL_rule,
  /*  301 */  _Binary_iRegN_iRegN_rule,
  /*  302 */  _Binary_iRegP_iRegP_rule,
  /*  303 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  304 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  305 */  _Binary_iRegI_iRegI_rule,
  /*  306 */  _Binary_iRegL_iRegL_rule,
  /*  307 */  _Binary_iRegN_iRegN_rule,
  /*  308 */  _Binary_iRegP_iRegP_rule,
  /*  309 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  310 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  311 */  _Binary_iRegI_iRegI_rule,
  /*  312 */  _Binary_iRegL_iRegL_rule,
  /*  313 */  _Binary_iRegN_iRegN_rule,
  /*  314 */  _Binary_iRegP_iRegP_rule,
  /*  315 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  316 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  317 */  _Binary_iRegI_iRegI_rule,
  /*  318 */  _Binary_iRegL_iRegL_rule,
  /*  319 */  _Binary_iRegN_iRegN_rule,
  /*  320 */  _Binary_iRegP_iRegP_rule,
  /*  321 */  iRegI_rule,
  /*  322 */  iRegL_rule,
  /*  323 */  iRegN_rule,
  /*  324 */  iRegP_rule,
  /*  325 */  iRegI_rule,
  /*  326 */  iRegL_rule,
  /*  327 */  iRegN_rule,
  /*  328 */  iRegP_rule,
  /*  329 */  iRegL_rule,
  /*  330 */  iRegL_rule,
  /*  331 */  immLAdd_rule,
  /*  332 */  immLAdd_rule,
  /*  333 */  iRegIorL2I_rule,
  /*  334 */  iRegIorL2I_rule,
  /*  335 */  immIAdd_rule,
  /*  336 */  immIAdd_rule,
  /*  337 */  iRegL_rule,
  /*  338 */  iRegL_rule,
  /*  339 */  immLAdd_rule,
  /*  340 */  immLAdd_rule,
  /*  341 */  iRegIorL2I_rule,
  /*  342 */  iRegIorL2I_rule,
  /*  343 */  immIAdd_rule,
  /*  344 */  immIAdd_rule,
  /*  345 */  iRegIorL2I_rule,
  /*  346 */  immIAdd_rule,
  /*  347 */  immIAdd_rule,
  /*  348 */  iRegL_rule,
  /*  349 */  uimmI6_ge32_rule,
  /*  350 */  immLAdd_rule,
  /*  351 */  iRegL_rule,
  /*  352 */  immLAdd_rule,
  /*  353 */  iRegIorL2I_rule,
  /*  354 */  immISub_rule,
  /*  355 */  iRegL_rule,
  /*  356 */  immLSub_rule,
  /*  357 */  iRegIorL2I_rule,
  /*  358 */  iRegL_rule,
  /*  359 */  iRegIorL2I_rule,
  /*  360 */  iRegL_rule,
  /*  361 */  iRegL_rule,
  /*  362 */  iRegIorL2I_rule,
  /*  363 */  immI_31_rule,
  /*  364 */  iRegL_rule,
  /*  365 */  immI_63_rule,
  /*  366 */  iRegIorL2I_rule,
  /*  367 */  iRegL_rule,
  /*  368 */  iRegIorL2I_rule,
  /*  369 */  immI_rule,
  /*  370 */  iRegIorL2I_rule,
  /*  371 */  immI_rule,
  /*  372 */  iRegIorL2I_rule,
  /*  373 */  immI_rule,
  /*  374 */  iRegIorL2I_rule,
  /*  375 */  immI_rule,
  /*  376 */  iRegIorL2I_rule,
  /*  377 */  immI_rule,
  /*  378 */  immI_rule,
  /*  379 */  iRegIorL2I_rule,
  /*  380 */  immI_rule,
  /*  381 */  immI_M1_rule,
  /*  382 */  immL_M1_rule,
  /*  383 */  fRegF_rule,
  /*  384 */  fRegD_rule,
  /*  385 */  fRegF_rule,
  /*  386 */  fRegD_rule,
  /*  387 */  fRegF_rule,
  /*  388 */  fRegD_rule,
  /*  389 */  _Binary_fRegF_fRegF_rule,
  /*  390 */  _Binary_fRegD_fRegD_rule,
  /*  391 */  _Binary_fRegF_fRegF_rule,
  /*  392 */  _Binary_fRegD_fRegD_rule,
  /*  393 */  _Binary__NegF_fRegF__fRegF_rule,
  /*  394 */  _Binary_fRegF__NegF_fRegF__rule,
  /*  395 */  _Binary__NegD_fRegD__fRegD_rule,
  /*  396 */  _Binary_fRegD__NegD_fRegD__rule,
  /*  397 */  _Binary__NegF_fRegF__fRegF_rule,
  /*  398 */  _Binary_fRegF__NegF_fRegF__rule,
  /*  399 */  _Binary__NegD_fRegD__fRegD_rule,
  /*  400 */  _Binary_fRegD__NegD_fRegD__rule,
  /*  401 */  fRegF_rule,
  /*  402 */  fRegF_rule,
  /*  403 */  fRegD_rule,
  /*  404 */  fRegD_rule,
  /*  405 */  fRegF_rule,
  /*  406 */  fRegD_rule,
  /*  407 */  0,
  /*  408 */  0,
  /*  409 */  iRegI_rule,
  /*  410 */  immIAdd_rule,
  /*  411 */  iRegI_rule,
  /*  412 */  immIAdd_rule,
  /*  413 */  iRegI_rule,
  /*  414 */  immIAdd_rule,
  /*  415 */  iRegL_rule,
  /*  416 */  immLAdd_rule,
  /*  417 */  iRegL_rule,
  /*  418 */  immLAdd_rule,
  /*  419 */  iRegL_rule,
  /*  420 */  immLAdd_rule,
  /*  421 */  0,
  /*  422 */  0,
  /*  423 */  0,
  /*  424 */  0,
  /*  425 */  0,
  /*  426 */  0,
  /*  427 */  0,
  /*  428 */  0,
  /*  429 */  0,
  /*  430 */  0,
  /*  431 */  0,
  /*  432 */  0,
  /*  433 */  0,
  /*  434 */  0,
  /*  435 */  0,
  /*  436 */  immL_32bits_rule,
  /*  437 */  0,
  /*  438 */  0,
  /*  439 */  0,
  /*  440 */  0,
  /*  441 */  0,
  /*  442 */  0,
  /*  443 */  0,
  /*  444 */  0,
  /*  445 */  0,
  /*  446 */  0,
  /*  447 */  0,
  /*  448 */  0,
  /*  449 */  0,
  /*  450 */  0,
  /*  451 */  0,
  /*  452 */  0,
  /*  453 */  0,
  /*  454 */  0,
  /*  455 */  0,
  /*  456 */  0,
  /*  457 */  0,
  /*  458 */  fRegF_rule,
  /*  459 */  fRegD_rule,
  /*  460 */  iRegL_rule,
  /*  461 */  iRegI_rule,
  /*  462 */  immI0_rule,
  /*  463 */  iRegI_rule,
  /*  464 */  iRegI_rule,
  /*  465 */  0,
  /*  466 */  rFlagsReg_rule,
  /*  467 */  _CmpI_iRegI_iRegI_rule,
  /*  468 */  _CmpI_iRegI_iRegI_rule,
  /*  469 */  _CmpU_iRegI_iRegI_rule,
  /*  470 */  _CmpU_iRegI_iRegI_rule,
  /*  471 */  _CmpL_iRegL_iRegL_rule,
  /*  472 */  _CmpL_iRegL_iRegL_rule,
  /*  473 */  _CmpUL_iRegL_iRegL_rule,
  /*  474 */  _CmpUL_iRegL_iRegL_rule,
  /*  475 */  _CmpP_iRegP_iRegP_rule,
  /*  476 */  _CmpP_iRegP_iRegP_rule,
  /*  477 */  _CmpN_iRegN_iRegN_rule,
  /*  478 */  _CmpN_iRegN_iRegN_rule,
  /*  479 */  _CmpF_fRegF_fRegF_rule,
  /*  480 */  _CmpF_fRegF_fRegF_rule,
  /*  481 */  _CmpD_fRegD_fRegD_rule,
  /*  482 */  _CmpD_fRegD_fRegD_rule,
  /*  483 */  _CmpI_iRegI_immI0_rule,
  /*  484 */  _CmpI_iRegI_immI0_rule,
  /*  485 */  _CmpU_iRegI_immI0_rule,
  /*  486 */  _CmpU_iRegI_immI0_rule,
  /*  487 */  _CmpL_iRegL_immL0_rule,
  /*  488 */  _CmpL_iRegL_immL0_rule,
  /*  489 */  _CmpUL_iRegL_immL0_rule,
  /*  490 */  _CmpUL_iRegL_immL0_rule,
  /*  491 */  _CmpP_iRegP_immP0_rule,
  /*  492 */  _CmpP_iRegP_immP0_rule,
  /*  493 */  _CmpN_iRegN_immN0_rule,
  /*  494 */  _CmpN_iRegN_immN0_rule,
  /*  495 */  _CmpP__DecodeN_iRegN__immP0_rule,
  /*  496 */  _CmpP__DecodeN_iRegN__immP0_rule,
  /*  497 */  rFlagsReg_rule,
  /*  498 */  _CmpI_iRegI_iRegI_rule,
  /*  499 */  _CmpI_iRegI_iRegI_rule,
  /*  500 */  _CmpU_iRegI_iRegI_rule,
  /*  501 */  _CmpU_iRegI_iRegI_rule,
  /*  502 */  _CmpL_iRegL_iRegL_rule,
  /*  503 */  _CmpL_iRegL_iRegL_rule,
  /*  504 */  _CmpUL_iRegL_iRegL_rule,
  /*  505 */  _CmpUL_iRegL_iRegL_rule,
  /*  506 */  _CmpP_iRegP_iRegP_rule,
  /*  507 */  _CmpP_iRegP_iRegP_rule,
  /*  508 */  _CmpN_iRegN_iRegN_rule,
  /*  509 */  _CmpN_iRegN_iRegN_rule,
  /*  510 */  _CmpF_fRegF_fRegF_rule,
  /*  511 */  _CmpF_fRegF_fRegF_rule,
  /*  512 */  _CmpD_fRegD_fRegD_rule,
  /*  513 */  _CmpD_fRegD_fRegD_rule,
  /*  514 */  _CmpI_iRegI_immI0_rule,
  /*  515 */  _CmpI_iRegI_immI0_rule,
  /*  516 */  _CmpU_iRegI_immI0_rule,
  /*  517 */  _CmpU_iRegI_immI0_rule,
  /*  518 */  _CmpU_iRegI_immI0_rule,
  /*  519 */  _CmpU_iRegI_immI0_rule,
  /*  520 */  _CmpL_iRegL_immL0_rule,
  /*  521 */  _CmpL_iRegL_immL0_rule,
  /*  522 */  _CmpUL_iRegL_immL0_rule,
  /*  523 */  _CmpUL_iRegL_immL0_rule,
  /*  524 */  _CmpUL_iRegL_immL0_rule,
  /*  525 */  _CmpUL_iRegL_immL0_rule,
  /*  526 */  _CmpP_iRegP_immP0_rule,
  /*  527 */  _CmpP_iRegP_immP0_rule,
  /*  528 */  _CmpN_iRegN_immN0_rule,
  /*  529 */  _CmpN_iRegN_immN0_rule,
  /*  530 */  _CmpP__DecodeN_iRegN__immP0_rule,
  /*  531 */  _CmpP__DecodeN_iRegN__immP0_rule,
  /*  532 */  _Binary_iRegINoSp_iRegI_rule,
  /*  533 */  _Binary_iRegINoSp_iRegI_rule,
  /*  534 */  _Binary_iRegINoSp_iRegI_rule,
  /*  535 */  _Binary_iRegLNoSp_iRegL_rule,
  /*  536 */  _Binary_iRegLNoSp_iRegL_rule,
  /*  537 */  _Binary_iRegINoSp_iRegI_rule,
  /*  538 */  0,
  /*  539 */  0,
  /*  540 */  0,
  /*  541 */  0,
  /*  542 */  0,
  /*  543 */  iRegP_R10_rule,
  /*  544 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  545 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  546 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  547 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  548 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  549 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  550 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  551 */  _Binary_iRegP_R13_immI_le_4_rule,
  /*  552 */  _Binary_iRegP_R13_immI_le_4_rule,
  /*  553 */  _Binary_iRegP_R13_immI_1_rule,
  /*  554 */  iRegI_R13_rule,
  /*  555 */  iRegI_R13_rule,
  /*  556 */  iRegP_R28_rule,
  /*  557 */  iRegP_R28_rule,
  /*  558 */  iRegI_R14_rule,
  /*  559 */  iRegI_R14_rule,
  /*  560 */  iRegP_R12_rule,
  /*  561 */  iRegP_R12_rule,
  /*  562 */  0,
  /*  563 */  inline_cache_RegP_rule,
  /*  564 */  iRegP_R10_rule,
  /*  565 */  0,
  /*  566 */  0,
  /*  567 */  0,
  /*  568 */  0,
  /*  569 */  0,
  /*  570 */  vReg_rule,
  /*  571 */  0,
  /*  572 */  0,
  /*  573 */  0,
  /*  574 */  0,
  /*  575 */  vReg_rule,
  /*  576 */  vReg_rule,
  /*  577 */  vReg_rule,
  /*  578 */  vReg_rule,
  /*  579 */  vReg_rule,
  /*  580 */  vReg_rule,
  /*  581 */  vReg_rule,
  /*  582 */  vReg_rule,
  /*  583 */  vReg_rule,
  /*  584 */  vReg_rule,
  /*  585 */  vReg_rule,
  /*  586 */  vReg_rule,
  /*  587 */  vReg_rule,
  /*  588 */  vReg_rule,
  /*  589 */  vReg_rule,
  /*  590 */  vReg_rule,
  /*  591 */  vReg_rule,
  /*  592 */  _Binary_vReg_vReg_rule,
  /*  593 */  _Binary_vReg_vReg_rule,
  /*  594 */  _Binary__NegVF_vReg__vReg_rule,
  /*  595 */  _Binary_vReg__NegVF_vReg__rule,
  /*  596 */  _Binary__NegVD_vReg__vReg_rule,
  /*  597 */  _Binary_vReg__NegVD_vReg__rule,
  /*  598 */  _Binary__NegVF_vReg__vReg_rule,
  /*  599 */  _Binary_vReg__NegVF_vReg__rule,
  /*  600 */  _Binary__NegVD_vReg__vReg_rule,
  /*  601 */  _Binary_vReg__NegVD_vReg__rule,
  /*  602 */  _Binary_vReg_vReg_rule,
  /*  603 */  _Binary_vReg_vReg_rule,
  /*  604 */  _MulVB_vReg_vReg_rule,
  /*  605 */  vReg_rule,
  /*  606 */  _MulVS_vReg_vReg_rule,
  /*  607 */  vReg_rule,
  /*  608 */  _MulVI_vReg_vReg_rule,
  /*  609 */  vReg_rule,
  /*  610 */  _MulVL_vReg_vReg_rule,
  /*  611 */  vReg_rule,
  /*  612 */  _MulVB_vReg_vReg_rule,
  /*  613 */  _MulVS_vReg_vReg_rule,
  /*  614 */  _MulVI_vReg_vReg_rule,
  /*  615 */  _MulVL_vReg_vReg_rule,
  /*  616 */  vReg_rule,
  /*  617 */  vReg_rule,
  /*  618 */  vReg_rule,
  /*  619 */  vReg_rule,
  /*  620 */  vReg_rule,
  /*  621 */  vReg_rule,
  /*  622 */  vReg_rule,
  /*  623 */  vReg_rule,
  /*  624 */  vReg_rule,
  /*  625 */  vReg_rule,
  /*  626 */  vReg_rule,
  /*  627 */  vReg_rule,
  /*  628 */  vReg_rule,
  /*  629 */  vReg_rule,
  /*  630 */  vReg_rule,
  /*  631 */  vReg_rule,
  /*  632 */  vReg_rule,
  /*  633 */  vReg_rule,
  /*  634 */  vReg_rule,
  /*  635 */  vReg_rule,
  /*  636 */  vReg_rule,
  /*  637 */  vReg_rule,
  /*  638 */  vReg_rule,
  /*  639 */  vReg_rule,
  /*  640 */  immI_rule,
  /*  641 */  0,
  /*  642 */  0,
  /*  643 */  0,
  /*  644 */  0,
  /*  645 */  0,
  /*  646 */  0,
  /*  647 */  0,
  /*  648 */  0,
  /*  649 */  0,
  /*  650 */  0,
  /*  651 */  vReg_rule,
  /*  652 */  vReg_rule,
  /*  653 */  vReg_rule,
  /*  654 */  vReg_rule,
  /*  655 */  vReg_rule,
  /*  656 */  vReg_rule,
  /*  657 */  vReg_rule,
  /*  658 */  vReg_rule,
  /*  659 */  vReg_rule,
  /*  660 */  vReg_rule,
  /*  661 */  vReg_rule,
  /*  662 */  vReg_rule,
  /*  663 */  _RShiftCntV_immI__rule,
  /*  664 */  _RShiftCntV_immI__rule,
  /*  665 */  _RShiftCntV_immI__rule,
  /*  666 */  _RShiftCntV_immI__rule,
  /*  667 */  _RShiftCntV_immI__rule,
  /*  668 */  _RShiftCntV_immI__rule,
  /*  669 */  _RShiftCntV_immI__rule,
  /*  670 */  _RShiftCntV_immI__rule,
  /*  671 */  _LShiftCntV_immI__rule,
  /*  672 */  _LShiftCntV_immI__rule,
  /*  673 */  _LShiftCntV_immI__rule,
  /*  674 */  _LShiftCntV_immI__rule,
  /*  675 */  vReg_rule,
  /*  676 */  vReg_rule,
  /*  677 */  vReg_rule,
  /*  678 */  vReg_rule,
  /*  679 */  vReg_rule,
  /*  680 */  vReg_rule,
  /*  681 */  iRegI_R14_rule,
  /*  682 */  iRegI_R14_rule,
  /*  683 */  iRegP_R12_rule,
  /*  684 */  iRegP_R12_rule,
  /*  685 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  686 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  687 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  688 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  689 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  690 */  _Binary_iRegP_R11_iRegI_R13_rule,
  /*  691 */  _Binary_iRegP_R11_iRegI_R13_rule,
  /*  692 */  iRegI_R12_rule,
  /*  693 */  iRegI_R13_rule,
  /*  694 */  iRegI_R13_rule,
  /*  695 */  iRegP_R28_rule,
  /*  696 */  immI_rule,
  /*  697 */  immI_rule,
  /*  698 */  iRegI_rule,
  /*  699 */  iRegI_rule,
  /*  700 */  iRegI_rule,
  /*  701 */  iRegI_rule,
  /*  702 */  0,
  /*  703 */  immI_24_rule,
  /*  704 */  immI_16_rule,
  /*  705 */  immI_16bits_rule,
  /*  706 */  immL_32bits_rule,
  /*  707 */  0,
  /*  708 */  0,
  /*  709 */  0,
  /*  710 */  0,
  /*  711 */  _LShiftL_iRegL_immIScale_rule,
  /*  712 */  _LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  713 */  _LShiftL_iRegL_immIScale_rule,
  /*  714 */  iRegL_rule,
  /*  715 */  _LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  716 */  iRegL_rule,
  /*  717 */  iRegI_rule,
  /*  718 */  iRegI_rule,
  /*  719 */  _XorI_iRegI_immI_M1_rule,
  /*  720 */  iRegI_rule,
  /*  721 */  _XorL_iRegL_immL_M1_rule,
  /*  722 */  iRegL_rule,
  /*  723 */  _XorI_iRegI_immI_M1_rule,
  /*  724 */  iRegI_rule,
  /*  725 */  _XorL_iRegL_immL_M1_rule,
  /*  726 */  iRegL_rule,
  /*  727 */  _Binary_iRegP_iRegP_rule,
  /*  728 */  _Binary_iRegN_iRegN_rule,
  /*  729 */  _Binary_iRegP_iRegP_rule,
  /*  730 */  _Binary_iRegN_iRegN_rule,
  /*  731 */  _Binary_iRegN_iRegN_rule,
  /*  732 */  _Binary_iRegP_iRegP_rule,
  /*  733 */  _Binary_iRegN_iRegN_rule,
  /*  734 */  _Binary_iRegN_iRegN_rule,
  /*  735 */  _Binary_iRegP_iRegP_rule,
  /*  736 */  _Binary_iRegP_iRegP_rule,
  /*  737 */  _Binary_iRegN_iRegN_rule,
  /*  738 */  _Binary_iRegP_iRegP_rule,
  /*  739 */  0,
  /*  740 */  _Binary_iRegP_iRegP_rule,
  /*  741 */  _Binary_iRegP_iRegP_rule,
  /*  742 */  _Binary_iRegP_iRegP_rule,
  /*  743 */  _Binary_iRegP_iRegP_rule,
  /*  744 */  _Binary_iRegP_iRegP_rule,
  /*  745 */  _Binary_iRegP_iRegP_rule,
  /*  746 */  iRegP_rule,
  /*  747 */  iRegP_rule,
  // last instruction
  0 // no trailing comma
};

const char        *ruleName[] = {
  /*    0 */  "UNIVERSE",
  /*    1 */  "LABEL",
  /*    2 */  "SREGI",
  /*    3 */  "SREGP",
  /*    4 */  "SREGF",
  /*    5 */  "SREGD",
  /*    6 */  "SREGL",
  /*    7 */  "METHOD",
  /*    8 */  "IMMI",
  /*    9 */  "IMMI0",
  /*   10 */  "IMMI_1",
  /*   11 */  "IMMI_M1",
  /*   12 */  "UIMMI6_GE32",
  /*   13 */  "IMMI_LE_4",
  /*   14 */  "IMMI_16",
  /*   15 */  "IMMI_24",
  /*   16 */  "IMMI_31",
  /*   17 */  "IMMI_63",
  /*   18 */  "IMMIADD",
  /*   19 */  "IMMISUB",
  /*   20 */  "IMMI5",
  /*   21 */  "IMML5",
  /*   22 */  "IMML",
  /*   23 */  "IMML0",
  /*   24 */  "IMMP",
  /*   25 */  "IMMP0",
  /*   26 */  "IMMP_1",
  /*   27 */  "IMMBYTEMAPBASE",
  /*   28 */  "IMMI_16BITS",
  /*   29 */  "IMML_32BITS",
  /*   30 */  "IMML_M1",
  /*   31 */  "IMML_PC_OFF",
  /*   32 */  "IMMLADD",
  /*   33 */  "IMMLSUB",
  /*   34 */  "IMMN",
  /*   35 */  "IMMN0",
  /*   36 */  "IMMNKLASS",
  /*   37 */  "IMMD",
  /*   38 */  "IMMD0",
  /*   39 */  "IMMF",
  /*   40 */  "IMMF0",
  /*   41 */  "IMMIOFFSET",
  /*   42 */  "IMMLOFFSET",
  /*   43 */  "IMMISCALE",
  /*   44 */  "IREGI",
  /*   45 */  "IREGINOSP",
  /*   46 */  "IREGI_R10",
  /*   47 */  "IREGI_R12",
  /*   48 */  "IREGI_R13",
  /*   49 */  "IREGI_R14",
  /*   50 */  "IREGL",
  /*   51 */  "IREGLNOSP",
  /*   52 */  "IREGL_R28",
  /*   53 */  "IREGL_R29",
  /*   54 */  "IREGL_R30",
  /*   55 */  "IREGP",
  /*   56 */  "IREGPNOSP",
  /*   57 */  "IREGP_R10",
  /*   58 */  "IREGP_R11",
  /*   59 */  "IREGP_R12",
  /*   60 */  "IREGP_R13",
  /*   61 */  "IREGP_R14",
  /*   62 */  "IREGP_R15",
  /*   63 */  "IREGP_R16",
  /*   64 */  "IREGP_R28",
  /*   65 */  "IREGN",
  /*   66 */  "IREGNNOSP",
  /*   67 */  "IREGIHEAPBASE",
  /*   68 */  "IREGL_R10",
  /*   69 */  "FREGF",
  /*   70 */  "FREGD",
  /*   71 */  "VREG",
  /*   72 */  "VREG_V1",
  /*   73 */  "VREG_V2",
  /*   74 */  "VREG_V3",
  /*   75 */  "VREG_V4",
  /*   76 */  "VREG_V5",
  /*   77 */  "JAVATHREAD_REGP",
  /*   78 */  "INDIRECT",
  /*   79 */  "INDOFFI",
  /*   80 */  "INDOFFL",
  /*   81 */  "INDIRECTN",
  /*   82 */  "INDOFFIN",
  /*   83 */  "INDOFFLN",
  /*   84 */  "THREAD_ANCHOR_PC",
  /*   85 */  "STACKSLOTI",
  /*   86 */  "STACKSLOTF",
  /*   87 */  "STACKSLOTD",
  /*   88 */  "STACKSLOTL",
  /*   89 */  "IREGL2I",
  /*   90 */  "CMPOP",
  /*   91 */  "CMPOPU",
  /*   92 */  "CMPOPEQNE",
  /*   93 */  "CMPOPULTGE",
  /*   94 */  "CMPOPUEQNELEGT",
  /*   95 */  "RFLAGSREG",
  /*   96 */  "INLINE_CACHE_REGP",
  // last operand
  /*   97 */  "MEMORY",
  /*   98 */  "IREGIORL2I",
  /*   99 */  "IREGIORL",
  /*  100 */  "IREGNORP",
  /*  101 */  "IREGILNP",
  /*  102 */  "IREGILNPNOSP",
  /*  103 */  "IMMIORL",
  /*  104 */  "VMEMA",
  // last operand class
  /*  105 */  "_DecodeN_iRegN_",
  /*  106 */  "_LoadB_memory_",
  /*  107 */  "_LoadUB_memory_",
  /*  108 */  "_LoadS_memory_",
  /*  109 */  "_LoadUS_memory_",
  /*  110 */  "_LoadI_memory_",
  /*  111 */  "_ConvI2L__LoadI_memory__",
  /*  112 */  "_Binary_iRegP_iRegP",
  /*  113 */  "_Binary_iRegL_iRegL",
  /*  114 */  "_Binary_iRegI_iRegI",
  /*  115 */  "_Binary_iRegI_R12_iRegI_R13",
  /*  116 */  "_Binary_iRegN_iRegN",
  /*  117 */  "_ConvL2I_iRegL_",
  /*  118 */  "_ConvI2L_iRegI_",
  /*  119 */  "_RShiftI_iRegIorL2I_immI_31",
  /*  120 */  "_RShiftL_iRegL_immI_63",
  /*  121 */  "_CastP2X_iRegP_",
  /*  122 */  "_Binary_fRegF_fRegF",
  /*  123 */  "_Binary_fRegD_fRegD",
  /*  124 */  "_NegF_fRegF_",
  /*  125 */  "_NegD_fRegD_",
  /*  126 */  "_Binary__NegF_fRegF__fRegF",
  /*  127 */  "_Binary_fRegF__NegF_fRegF_",
  /*  128 */  "_Binary__NegD_fRegD__fRegD",
  /*  129 */  "_Binary_fRegD__NegD_fRegD_",
  /*  130 */  "_ConvF2D_fRegF_",
  /*  131 */  "_SqrtD__ConvF2D_fRegF__",
  /*  132 */  "_ConvI2L_iRegIorL2I_",
  /*  133 */  "_CastP2X__DecodeN_iRegN__",
  /*  134 */  "_CmpI_iRegI_iRegI",
  /*  135 */  "_CmpU_iRegI_iRegI",
  /*  136 */  "_CmpL_iRegL_iRegL",
  /*  137 */  "_CmpUL_iRegL_iRegL",
  /*  138 */  "_CmpP_iRegP_iRegP",
  /*  139 */  "_CmpN_iRegN_iRegN",
  /*  140 */  "_CmpF_fRegF_fRegF",
  /*  141 */  "_CmpD_fRegD_fRegD",
  /*  142 */  "_CmpI_iRegI_immI0",
  /*  143 */  "_CmpU_iRegI_immI0",
  /*  144 */  "_CmpL_iRegL_immL0",
  /*  145 */  "_CmpUL_iRegL_immL0",
  /*  146 */  "_CmpP_iRegP_immP0",
  /*  147 */  "_CmpN_iRegN_immN0",
  /*  148 */  "_CmpP__DecodeN_iRegN__immP0",
  /*  149 */  "_Binary_cmpOp__CmpI_iRegI_iRegI",
  /*  150 */  "_Binary_iRegINoSp_iRegI",
  /*  151 */  "_Binary_cmpOpU__CmpU_iRegI_iRegI",
  /*  152 */  "_Binary_cmpOp__CmpL_iRegL_iRegL",
  /*  153 */  "_Binary_iRegLNoSp_iRegL",
  /*  154 */  "_Binary_cmpOpU__CmpUL_iRegL_iRegL",
  /*  155 */  "_PartialSubtypeCheck_iRegP_R14_iRegP_R10",
  /*  156 */  "_Binary_iRegP_R11_iRegI_R12",
  /*  157 */  "_Binary_iRegP_R13_iRegI_R14",
  /*  158 */  "_Binary_iRegP_R13_immI_le_4",
  /*  159 */  "_Binary_iRegP_R13_immI_1",
  /*  160 */  "_Binary_iRegP_R11_iRegP_R13",
  /*  161 */  "_Binary_vReg_vReg",
  /*  162 */  "_NegVF_vReg_",
  /*  163 */  "_Binary__NegVF_vReg__vReg",
  /*  164 */  "_Binary_vReg__NegVF_vReg_",
  /*  165 */  "_NegVD_vReg_",
  /*  166 */  "_Binary__NegVD_vReg__vReg",
  /*  167 */  "_Binary_vReg__NegVD_vReg_",
  /*  168 */  "_MulVB_vReg_vReg",
  /*  169 */  "_MulVS_vReg_vReg",
  /*  170 */  "_MulVI_vReg_vReg",
  /*  171 */  "_MulVL_vReg_vReg",
  /*  172 */  "_RShiftCntV_immI_",
  /*  173 */  "_LShiftCntV_immI_",
  /*  174 */  "_Binary_iRegP_R11_iRegI_R13",
  /*  175 */  "_LShiftI_iRegIorL2I_immI_24",
  /*  176 */  "_LShiftI_iRegIorL2I_immI_16",
  /*  177 */  "_LShiftL_iRegL_immIScale",
  /*  178 */  "_LShiftL__ConvI2L_iRegI__immIScale",
  /*  179 */  "_XorI_iRegI_immI_M1",
  /*  180 */  "_XorL_iRegL_immL_M1",
  // last internally defined operand
  /*  181 */  "loadConI",
  /*  182 */  "loadConL",
  /*  183 */  "loadConP",
  /*  184 */  "loadConP0",
  /*  185 */  "loadConP1",
  /*  186 */  "loadByteMapBase",
  /*  187 */  "loadConN",
  /*  188 */  "loadConN0",
  /*  189 */  "loadConNKlass",
  /*  190 */  "loadConF",
  /*  191 */  "loadConF0",
  /*  192 */  "loadConD",
  /*  193 */  "loadConD0",
  /*  194 */  "storePConditional",
  /*  195 */  "storeLConditional",
  /*  196 */  "storeIConditional",
  /*  197 */  "negF_reg_reg",
  /*  198 */  "negD_reg_reg",
  /*  199 */  "absI_reg",
  /*  200 */  "absL_reg",
  /*  201 */  "absF_reg",
  /*  202 */  "absD_reg",
  /*  203 */  "castX2P",
  /*  204 */  "castP2X",
  /*  205 */  "castPP",
  /*  206 */  "castLL",
  /*  207 */  "castII",
  /*  208 */  "checkCastPP",
  /*  209 */  "castFF",
  /*  210 */  "castDD",
  /*  211 */  "castVV",
  /*  212 */  "convI2Bool",
  /*  213 */  "convP2Bool",
  /*  214 */  "MoveF2I_stack_reg",
  /*  215 */  "MoveI2F_stack_reg",
  /*  216 */  "MoveD2L_stack_reg",
  /*  217 */  "MoveL2D_stack_reg",
  /*  218 */  "MoveF2I_reg_reg",
  /*  219 */  "MoveI2F_reg_reg",
  /*  220 */  "MoveD2L_reg_reg",
  /*  221 */  "MoveL2D_reg_reg",
  /*  222 */  "partialSubtypeCheckVsZero",
  /*  223 */  "tlsLoadP",
  /*  224 */  "cmpFastLock",
  /*  225 */  "cmpFastUnlock",
  /*  226 */  "vabsF",
  /*  227 */  "vabsD",
  /*  228 */  "vnegF",
  /*  229 */  "vnegD",
  /*  230 */  "vshiftcntB",
  /*  231 */  "vshiftcntB_0",
  /*  232 */  "vshiftcntS",
  /*  233 */  "vshiftcntS_0",
  /*  234 */  "vshiftcntI",
  /*  235 */  "vshiftcntI_0",
  /*  236 */  "vshiftcntL",
  /*  237 */  "vshiftcntL_0",
  /*  238 */  "vsqrtF",
  /*  239 */  "vsqrtD",
  /*  240 */  "countLeadingZerosI_b",
  /*  241 */  "countLeadingZerosL_b",
  /*  242 */  "countTrailingZerosI_b",
  /*  243 */  "countTrailingZerosL_b",
  /*  244 */  "popCountI_b",
  /*  245 */  "popCountL_b",
  /*  246 */  "absI_reg_b",
  /*  247 */  "absL_reg_b",
  /*  248 */  "loadB",
  /*  249 */  "loadB2L",
  /*  250 */  "loadUB",
  /*  251 */  "loadUB2L",
  /*  252 */  "loadS",
  /*  253 */  "loadS2L",
  /*  254 */  "loadUS",
  /*  255 */  "loadUS2L",
  /*  256 */  "loadI",
  /*  257 */  "loadI2L",
  /*  258 */  "loadUI2L",
  /*  259 */  "loadL",
  /*  260 */  "loadRange",
  /*  261 */  "loadP",
  /*  262 */  "loadN",
  /*  263 */  "loadKlass",
  /*  264 */  "loadNKlass",
  /*  265 */  "loadF",
  /*  266 */  "loadD",
  /*  267 */  "storeimmCM0",
  /*  268 */  "storeimmCM0_ordered",
  /*  269 */  "storeB",
  /*  270 */  "storeimmB0",
  /*  271 */  "storeC",
  /*  272 */  "storeimmC0",
  /*  273 */  "storeI",
  /*  274 */  "storeimmI0",
  /*  275 */  "storeL",
  /*  276 */  "storeimmL0",
  /*  277 */  "storeP",
  /*  278 */  "storeimmP0",
  /*  279 */  "storeN",
  /*  280 */  "storeImmN0",
  /*  281 */  "storeF",
  /*  282 */  "storeD",
  /*  283 */  "storeNKlass",
  /*  284 */  "loadPLocked",
  /*  285 */  "compareAndSwapB",
  /*  286 */  "compareAndSwapS",
  /*  287 */  "compareAndSwapI",
  /*  288 */  "compareAndSwapL",
  /*  289 */  "compareAndSwapP",
  /*  290 */  "compareAndSwapN",
  /*  291 */  "compareAndSwapBAcq",
  /*  292 */  "compareAndSwapSAcq",
  /*  293 */  "compareAndSwapIAcq",
  /*  294 */  "compareAndSwapLAcq",
  /*  295 */  "compareAndSwapPAcq",
  /*  296 */  "compareAndSwapNAcq",
  /*  297 */  "compareAndExchangeB",
  /*  298 */  "compareAndExchangeS",
  /*  299 */  "compareAndExchangeI",
  /*  300 */  "compareAndExchangeL",
  /*  301 */  "compareAndExchangeN",
  /*  302 */  "compareAndExchangeP",
  /*  303 */  "compareAndExchangeBAcq",
  /*  304 */  "compareAndExchangeSAcq",
  /*  305 */  "compareAndExchangeIAcq",
  /*  306 */  "compareAndExchangeLAcq",
  /*  307 */  "compareAndExchangeNAcq",
  /*  308 */  "compareAndExchangePAcq",
  /*  309 */  "weakCompareAndSwapB",
  /*  310 */  "weakCompareAndSwapS",
  /*  311 */  "weakCompareAndSwapI",
  /*  312 */  "weakCompareAndSwapL",
  /*  313 */  "weakCompareAndSwapN",
  /*  314 */  "weakCompareAndSwapP",
  /*  315 */  "weakCompareAndSwapBAcq",
  /*  316 */  "weakCompareAndSwapSAcq",
  /*  317 */  "weakCompareAndSwapIAcq",
  /*  318 */  "weakCompareAndSwapLAcq",
  /*  319 */  "weakCompareAndSwapNAcq",
  /*  320 */  "weakCompareAndSwapPAcq",
  /*  321 */  "get_and_setI",
  /*  322 */  "get_and_setL",
  /*  323 */  "get_and_setN",
  /*  324 */  "get_and_setP",
  /*  325 */  "get_and_setIAcq",
  /*  326 */  "get_and_setLAcq",
  /*  327 */  "get_and_setNAcq",
  /*  328 */  "get_and_setPAcq",
  /*  329 */  "get_and_addL",
  /*  330 */  "get_and_addL_no_res",
  /*  331 */  "get_and_addLi",
  /*  332 */  "get_and_addLi_no_res",
  /*  333 */  "get_and_addI",
  /*  334 */  "get_and_addI_no_res",
  /*  335 */  "get_and_addIi",
  /*  336 */  "get_and_addIi_no_res",
  /*  337 */  "get_and_addLAcq",
  /*  338 */  "get_and_addL_no_resAcq",
  /*  339 */  "get_and_addLiAcq",
  /*  340 */  "get_and_addLi_no_resAcq",
  /*  341 */  "get_and_addIAcq",
  /*  342 */  "get_and_addI_no_resAcq",
  /*  343 */  "get_and_addIiAcq",
  /*  344 */  "get_and_addIi_no_resAcq",
  /*  345 */  "addI_reg_reg",
  /*  346 */  "addI_reg_imm",
  /*  347 */  "addI_reg_imm_l2i",
  /*  348 */  "addP_reg_reg",
  /*  349 */  "lShiftL_regI_immGE32",
  /*  350 */  "addP_reg_imm",
  /*  351 */  "addL_reg_reg",
  /*  352 */  "addL_reg_imm",
  /*  353 */  "subI_reg_reg",
  /*  354 */  "subI_reg_imm",
  /*  355 */  "subL_reg_reg",
  /*  356 */  "subL_reg_imm",
  /*  357 */  "negI_reg",
  /*  358 */  "negL_reg",
  /*  359 */  "mulI",
  /*  360 */  "mulL",
  /*  361 */  "mulHiL_rReg",
  /*  362 */  "divI",
  /*  363 */  "signExtract",
  /*  364 */  "divL",
  /*  365 */  "signExtractL",
  /*  366 */  "modI",
  /*  367 */  "modL",
  /*  368 */  "lShiftI_reg_reg",
  /*  369 */  "lShiftI_reg_imm",
  /*  370 */  "urShiftI_reg_reg",
  /*  371 */  "urShiftI_reg_imm",
  /*  372 */  "rShiftI_reg_reg",
  /*  373 */  "rShiftI_reg_imm",
  /*  374 */  "lShiftL_reg_reg",
  /*  375 */  "lShiftL_reg_imm",
  /*  376 */  "urShiftL_reg_reg",
  /*  377 */  "urShiftL_reg_imm",
  /*  378 */  "urShiftP_reg_imm",
  /*  379 */  "rShiftL_reg_reg",
  /*  380 */  "rShiftL_reg_imm",
  /*  381 */  "regI_not_reg",
  /*  382 */  "regL_not_reg",
  /*  383 */  "addF_reg_reg",
  /*  384 */  "addD_reg_reg",
  /*  385 */  "subF_reg_reg",
  /*  386 */  "subD_reg_reg",
  /*  387 */  "mulF_reg_reg",
  /*  388 */  "mulD_reg_reg",
  /*  389 */  "maddF_reg_reg",
  /*  390 */  "maddD_reg_reg",
  /*  391 */  "msubF_reg_reg",
  /*  392 */  "msubD_reg_reg",
  /*  393 */  "nmsubF_reg_reg",
  /*  394 */  "nmsubF_reg_reg_0",
  /*  395 */  "nmsubD_reg_reg",
  /*  396 */  "nmsubD_reg_reg_0",
  /*  397 */  "nmaddF_reg_reg",
  /*  398 */  "nmaddF_reg_reg_0",
  /*  399 */  "nmaddD_reg_reg",
  /*  400 */  "nmaddD_reg_reg_0",
  /*  401 */  "maxF_reg_reg",
  /*  402 */  "minF_reg_reg",
  /*  403 */  "maxD_reg_reg",
  /*  404 */  "minD_reg_reg",
  /*  405 */  "divF_reg_reg",
  /*  406 */  "divD_reg_reg",
  /*  407 */  "sqrtF_reg",
  /*  408 */  "sqrtD_reg",
  /*  409 */  "andI_reg_reg",
  /*  410 */  "andI_reg_imm",
  /*  411 */  "orI_reg_reg",
  /*  412 */  "orI_reg_imm",
  /*  413 */  "xorI_reg_reg",
  /*  414 */  "xorI_reg_imm",
  /*  415 */  "andL_reg_reg",
  /*  416 */  "andL_reg_imm",
  /*  417 */  "orL_reg_reg",
  /*  418 */  "orL_reg_imm",
  /*  419 */  "xorL_reg_reg",
  /*  420 */  "xorL_reg_imm",
  /*  421 */  "bytes_reverse_int",
  /*  422 */  "bytes_reverse_long",
  /*  423 */  "bytes_reverse_unsigned_short",
  /*  424 */  "bytes_reverse_short",
  /*  425 */  "load_fence",
  /*  426 */  "membar_acquire",
  /*  427 */  "membar_acquire_lock",
  /*  428 */  "store_fence",
  /*  429 */  "membar_release",
  /*  430 */  "membar_storestore",
  /*  431 */  "membar_storestore_0",
  /*  432 */  "membar_release_lock",
  /*  433 */  "membar_volatile",
  /*  434 */  "convI2L_reg_reg",
  /*  435 */  "convL2I_reg",
  /*  436 */  "convI2UL_reg_reg",
  /*  437 */  "convF2D_reg",
  /*  438 */  "convD2F_reg",
  /*  439 */  "convF2I_reg_reg",
  /*  440 */  "convI2F_reg_reg",
  /*  441 */  "convF2L_reg_reg",
  /*  442 */  "convL2F_reg_reg",
  /*  443 */  "convD2I_reg_reg",
  /*  444 */  "convI2D_reg_reg",
  /*  445 */  "convD2L_reg_reg",
  /*  446 */  "convL2D_reg_reg",
  /*  447 */  "convP2I",
  /*  448 */  "convN2I",
  /*  449 */  "encodeHeapOop",
  /*  450 */  "decodeHeapOop",
  /*  451 */  "decodeHeapOop_not_null",
  /*  452 */  "encodeKlass_not_null",
  /*  453 */  "decodeKlass_not_null",
  /*  454 */  "MoveF2I_reg_stack",
  /*  455 */  "MoveI2F_reg_stack",
  /*  456 */  "MoveD2L_reg_stack",
  /*  457 */  "MoveL2D_reg_stack",
  /*  458 */  "cmpF3_reg_reg",
  /*  459 */  "cmpD3_reg_reg",
  /*  460 */  "cmpL3_reg_reg",
  /*  461 */  "cmpLTMask_reg_reg",
  /*  462 */  "cmpLTMask_reg_zero",
  /*  463 */  "minI_rReg",
  /*  464 */  "maxI_rReg",
  /*  465 */  "branch",
  /*  466 */  "cmpFlag_branch",
  /*  467 */  "cmpI_branch",
  /*  468 */  "cmpI_loop",
  /*  469 */  "cmpU_branch",
  /*  470 */  "cmpU_loop",
  /*  471 */  "cmpL_branch",
  /*  472 */  "cmpL_loop",
  /*  473 */  "cmpUL_branch",
  /*  474 */  "cmpUL_loop",
  /*  475 */  "cmpP_branch",
  /*  476 */  "cmpP_loop",
  /*  477 */  "cmpN_branch",
  /*  478 */  "cmpN_loop",
  /*  479 */  "cmpF_branch",
  /*  480 */  "cmpF_loop",
  /*  481 */  "cmpD_branch",
  /*  482 */  "cmpD_loop",
  /*  483 */  "cmpI_reg_imm0_branch",
  /*  484 */  "cmpI_reg_imm0_loop",
  /*  485 */  "cmpUEqNeLeGt_reg_imm0_branch",
  /*  486 */  "cmpUEqNeLeGt_reg_imm0_loop",
  /*  487 */  "cmpL_reg_imm0_branch",
  /*  488 */  "cmpL_reg_imm0_loop",
  /*  489 */  "cmpULEqNeLeGt_reg_imm0_branch",
  /*  490 */  "cmpULEqNeLeGt_reg_imm0_loop",
  /*  491 */  "cmpP_imm0_branch",
  /*  492 */  "cmpP_imm0_loop",
  /*  493 */  "cmpN_imm0_branch",
  /*  494 */  "cmpN_imm0_loop",
  /*  495 */  "cmpP_narrowOop_imm0_branch",
  /*  496 */  "cmpP_narrowOop_imm0_loop",
  /*  497 */  "far_cmpFlag_branch",
  /*  498 */  "far_cmpI_branch",
  /*  499 */  "far_cmpI_loop",
  /*  500 */  "far_cmpU_branch",
  /*  501 */  "far_cmpU_loop",
  /*  502 */  "far_cmpL_branch",
  /*  503 */  "far_cmpLloop",
  /*  504 */  "far_cmpUL_branch",
  /*  505 */  "far_cmpUL_loop",
  /*  506 */  "far_cmpP_branch",
  /*  507 */  "far_cmpP_loop",
  /*  508 */  "far_cmpN_branch",
  /*  509 */  "far_cmpN_loop",
  /*  510 */  "far_cmpF_branch",
  /*  511 */  "far_cmpF_loop",
  /*  512 */  "far_cmpD_branch",
  /*  513 */  "far_cmpD_loop",
  /*  514 */  "far_cmpI_reg_imm0_branch",
  /*  515 */  "far_cmpI_reg_imm0_loop",
  /*  516 */  "far_cmpUEqNeLeGt_imm0_branch",
  /*  517 */  "far_cmpUEqNeLeGt_reg_imm0_loop",
  /*  518 */  "far_cmpULtGe_reg_imm0_branch",
  /*  519 */  "far_cmpULtGe_reg_imm0_loop",
  /*  520 */  "far_cmpL_reg_imm0_branch",
  /*  521 */  "far_cmpL_reg_imm0_loop",
  /*  522 */  "far_cmpULEqNeLeGt_reg_imm0_branch",
  /*  523 */  "far_cmpULEqNeLeGt_reg_imm0_loop",
  /*  524 */  "far_cmpULLtGe_reg_imm0_branch",
  /*  525 */  "far_cmpULLtGe_reg_imm0_loop",
  /*  526 */  "far_cmpP_imm0_branch",
  /*  527 */  "far_cmpP_imm0_loop",
  /*  528 */  "far_cmpN_imm0_branch",
  /*  529 */  "far_cmpN_imm0_loop",
  /*  530 */  "far_cmpP_narrowOop_imm0_branch",
  /*  531 */  "far_cmpP_narrowOop_imm0_loop",
  /*  532 */  "cmovI_cmpI",
  /*  533 */  "cmovI_cmpU",
  /*  534 */  "cmovI_cmpL",
  /*  535 */  "cmovL_cmpL",
  /*  536 */  "cmovL_cmpUL",
  /*  537 */  "cmovI_cmpUL",
  /*  538 */  "CallStaticJavaDirect",
  /*  539 */  "CallDynamicJavaDirect",
  /*  540 */  "CallRuntimeDirect",
  /*  541 */  "CallLeafDirect",
  /*  542 */  "CallLeafNoFPDirect",
  /*  543 */  "partialSubtypeCheck",
  /*  544 */  "string_compareU",
  /*  545 */  "string_compareL",
  /*  546 */  "string_compareUL",
  /*  547 */  "string_compareLU",
  /*  548 */  "string_indexofUU",
  /*  549 */  "string_indexofLL",
  /*  550 */  "string_indexofUL",
  /*  551 */  "string_indexof_conUU",
  /*  552 */  "string_indexof_conLL",
  /*  553 */  "string_indexof_conUL",
  /*  554 */  "stringU_indexof_char",
  /*  555 */  "stringL_indexof_char",
  /*  556 */  "clearArray_reg_reg",
  /*  557 */  "clearArray_imm_reg",
  /*  558 */  "string_equalsL",
  /*  559 */  "string_equalsU",
  /*  560 */  "array_equalsB",
  /*  561 */  "array_equalsC",
  /*  562 */  "safePoint",
  /*  563 */  "TailCalljmpInd",
  /*  564 */  "TailjmpInd",
  /*  565 */  "CreateException",
  /*  566 */  "RethrowException",
  /*  567 */  "Ret",
  /*  568 */  "ShouldNotReachHere",
  /*  569 */  "loadV",
  /*  570 */  "storeV",
  /*  571 */  "vabsB",
  /*  572 */  "vabsS",
  /*  573 */  "vabsI",
  /*  574 */  "vabsL",
  /*  575 */  "vaddB",
  /*  576 */  "vaddS",
  /*  577 */  "vaddI",
  /*  578 */  "vaddL",
  /*  579 */  "vaddF",
  /*  580 */  "vaddD",
  /*  581 */  "vand",
  /*  582 */  "vor",
  /*  583 */  "vxor",
  /*  584 */  "vdivF",
  /*  585 */  "vdivD",
  /*  586 */  "vmax",
  /*  587 */  "vmin",
  /*  588 */  "vmaxF",
  /*  589 */  "vmaxD",
  /*  590 */  "vminF",
  /*  591 */  "vminD",
  /*  592 */  "vfmlaF",
  /*  593 */  "vfmlaD",
  /*  594 */  "vfmlsF",
  /*  595 */  "vfmlsF_0",
  /*  596 */  "vfmlsD",
  /*  597 */  "vfmlsD_0",
  /*  598 */  "vfnmlaF",
  /*  599 */  "vfnmlaF_0",
  /*  600 */  "vfnmlaD",
  /*  601 */  "vfnmlaD_0",
  /*  602 */  "vfnmlsF",
  /*  603 */  "vfnmlsD",
  /*  604 */  "vmlaB",
  /*  605 */  "vmlaB_0",
  /*  606 */  "vmlaS",
  /*  607 */  "vmlaS_0",
  /*  608 */  "vmlaI",
  /*  609 */  "vmlaI_0",
  /*  610 */  "vmlaL",
  /*  611 */  "vmlaL_0",
  /*  612 */  "vmlsB",
  /*  613 */  "vmlsS",
  /*  614 */  "vmlsI",
  /*  615 */  "vmlsL",
  /*  616 */  "vmulB",
  /*  617 */  "vmulS",
  /*  618 */  "vmulI",
  /*  619 */  "vmulL",
  /*  620 */  "vmulF",
  /*  621 */  "vmulD",
  /*  622 */  "reduce_addB",
  /*  623 */  "reduce_addS",
  /*  624 */  "reduce_addI",
  /*  625 */  "reduce_addL",
  /*  626 */  "reduce_addF",
  /*  627 */  "reduce_addD",
  /*  628 */  "vreduce_maxB",
  /*  629 */  "vreduce_maxS",
  /*  630 */  "vreduce_maxI",
  /*  631 */  "vreduce_maxL",
  /*  632 */  "vreduce_minB",
  /*  633 */  "vreduce_minS",
  /*  634 */  "vreduce_minI",
  /*  635 */  "vreduce_minL",
  /*  636 */  "vreduce_maxF",
  /*  637 */  "vreduce_maxD",
  /*  638 */  "vreduce_minF",
  /*  639 */  "vreduce_minD",
  /*  640 */  "vroundD",
  /*  641 */  "replicateB",
  /*  642 */  "replicateS",
  /*  643 */  "replicateI",
  /*  644 */  "replicateL",
  /*  645 */  "replicateB_imm5",
  /*  646 */  "replicateS_imm5",
  /*  647 */  "replicateI_imm5",
  /*  648 */  "replicateL_imm5",
  /*  649 */  "replicateF",
  /*  650 */  "replicateD",
  /*  651 */  "vasrB",
  /*  652 */  "vasrS",
  /*  653 */  "vasrI",
  /*  654 */  "vasrL",
  /*  655 */  "vlslB",
  /*  656 */  "vlslS",
  /*  657 */  "vlslI",
  /*  658 */  "vlslL",
  /*  659 */  "vlsrB",
  /*  660 */  "vlsrS",
  /*  661 */  "vlsrI",
  /*  662 */  "vlsrL",
  /*  663 */  "vasrB_imm",
  /*  664 */  "vasrS_imm",
  /*  665 */  "vasrI_imm",
  /*  666 */  "vasrL_imm",
  /*  667 */  "vlsrB_imm",
  /*  668 */  "vlsrS_imm",
  /*  669 */  "vlsrI_imm",
  /*  670 */  "vlsrL_imm",
  /*  671 */  "vlslB_imm",
  /*  672 */  "vlslS_imm",
  /*  673 */  "vlslI_imm",
  /*  674 */  "vlslL_imm",
  /*  675 */  "vsubB",
  /*  676 */  "vsubS",
  /*  677 */  "vsubI",
  /*  678 */  "vsubL",
  /*  679 */  "vsubF",
  /*  680 */  "vsubD",
  /*  681 */  "vstring_equalsL",
  /*  682 */  "vstring_equalsU",
  /*  683 */  "varray_equalsB",
  /*  684 */  "varray_equalsC",
  /*  685 */  "vstring_compareU",
  /*  686 */  "vstring_compareL",
  /*  687 */  "vstring_compareUL",
  /*  688 */  "vstring_compareLU",
  /*  689 */  "vstring_inflate",
  /*  690 */  "vencode_iso_array",
  /*  691 */  "vstring_compress",
  /*  692 */  "vcount_positives",
  /*  693 */  "vstringU_indexof_char",
  /*  694 */  "vstringL_indexof_char",
  /*  695 */  "vclearArray_reg_reg",
  /*  696 */  "rorI_imm_b",
  /*  697 */  "rorL_imm_b",
  /*  698 */  "rorI_reg_b",
  /*  699 */  "rorL_reg_b",
  /*  700 */  "rolI_reg_b",
  /*  701 */  "rolL_reg_b",
  /*  702 */  "convP2I_b",
  /*  703 */  "convB2I_reg_reg_b",
  /*  704 */  "convI2S_reg_reg_b",
  /*  705 */  "convS2UI_reg_reg_b",
  /*  706 */  "convI2UL_reg_reg_b",
  /*  707 */  "bytes_reverse_int_b",
  /*  708 */  "bytes_reverse_long_b",
  /*  709 */  "bytes_reverse_unsigned_short_b",
  /*  710 */  "bytes_reverse_short_b",
  /*  711 */  "shaddP_reg_reg_b",
  /*  712 */  "shaddP_reg_reg_ext_b",
  /*  713 */  "shaddL_reg_reg_b",
  /*  714 */  "shaddL_reg_reg_b_0",
  /*  715 */  "shaddL_reg_reg_ext_b",
  /*  716 */  "shaddL_reg_reg_ext_b_0",
  /*  717 */  "minI_reg_b",
  /*  718 */  "maxI_reg_b",
  /*  719 */  "andnI_reg_reg_b",
  /*  720 */  "andnI_reg_reg_b_0",
  /*  721 */  "andnL_reg_reg_b",
  /*  722 */  "andnL_reg_reg_b_0",
  /*  723 */  "ornI_reg_reg_b",
  /*  724 */  "ornI_reg_reg_b_0",
  /*  725 */  "ornL_reg_reg_b",
  /*  726 */  "ornL_reg_reg_b_0",
  /*  727 */  "compareAndSwapP_shenandoah",
  /*  728 */  "compareAndSwapN_shenandoah",
  /*  729 */  "compareAndSwapPAcq_shenandoah",
  /*  730 */  "compareAndSwapNAcq_shenandoah",
  /*  731 */  "compareAndExchangeN_shenandoah",
  /*  732 */  "compareAndExchangeP_shenandoah",
  /*  733 */  "weakCompareAndSwapN_shenandoah",
  /*  734 */  "compareAndExchangeNAcq_shenandoah",
  /*  735 */  "compareAndExchangePAcq_shenandoah",
  /*  736 */  "weakCompareAndSwapP_shenandoah",
  /*  737 */  "weakCompareAndSwapNAcq_shenandoah",
  /*  738 */  "weakCompareAndSwapPAcq_shenandoah",
  /*  739 */  "zLoadP",
  /*  740 */  "zCompareAndSwapP",
  /*  741 */  "zCompareAndSwapP_0",
  /*  742 */  "zCompareAndSwapPAcq",
  /*  743 */  "zCompareAndSwapPAcq_0",
  /*  744 */  "zCompareAndExchangeP",
  /*  745 */  "zCompareAndExchangePAcq",
  /*  746 */  "zGetAndSetP",
  /*  747 */  "zGetAndSetPAcq",
  // last instruction
  "invalid rule name" // no trailing comma
};

const        bool  swallowed[] = {
  /*    0 */  false,
  /*    1 */  false,
  /*    2 */  false,
  /*    3 */  false,
  /*    4 */  false,
  /*    5 */  false,
  /*    6 */  false,
  /*    7 */  false,
  /*    8 */  true,
  /*    9 */  true,
  /*   10 */  true,
  /*   11 */  true,
  /*   12 */  true,
  /*   13 */  true,
  /*   14 */  true,
  /*   15 */  true,
  /*   16 */  true,
  /*   17 */  true,
  /*   18 */  true,
  /*   19 */  true,
  /*   20 */  true,
  /*   21 */  true,
  /*   22 */  true,
  /*   23 */  true,
  /*   24 */  true,
  /*   25 */  true,
  /*   26 */  true,
  /*   27 */  true,
  /*   28 */  true,
  /*   29 */  true,
  /*   30 */  true,
  /*   31 */  true,
  /*   32 */  true,
  /*   33 */  true,
  /*   34 */  true,
  /*   35 */  true,
  /*   36 */  true,
  /*   37 */  true,
  /*   38 */  true,
  /*   39 */  true,
  /*   40 */  true,
  /*   41 */  true,
  /*   42 */  true,
  /*   43 */  true,
  /*   44 */  false,
  /*   45 */  false,
  /*   46 */  false,
  /*   47 */  false,
  /*   48 */  false,
  /*   49 */  false,
  /*   50 */  false,
  /*   51 */  false,
  /*   52 */  false,
  /*   53 */  false,
  /*   54 */  false,
  /*   55 */  false,
  /*   56 */  false,
  /*   57 */  false,
  /*   58 */  false,
  /*   59 */  false,
  /*   60 */  false,
  /*   61 */  false,
  /*   62 */  false,
  /*   63 */  false,
  /*   64 */  false,
  /*   65 */  false,
  /*   66 */  false,
  /*   67 */  false,
  /*   68 */  false,
  /*   69 */  false,
  /*   70 */  false,
  /*   71 */  false,
  /*   72 */  false,
  /*   73 */  false,
  /*   74 */  false,
  /*   75 */  false,
  /*   76 */  false,
  /*   77 */  false,
  /*   78 */  false,
  /*   79 */  false,
  /*   80 */  false,
  /*   81 */  false,
  /*   82 */  false,
  /*   83 */  false,
  /*   84 */  false,
  /*   85 */  false,
  /*   86 */  false,
  /*   87 */  false,
  /*   88 */  false,
  /*   89 */  false,
  /*   90 */  true,
  /*   91 */  true,
  /*   92 */  true,
  /*   93 */  true,
  /*   94 */  true,
  /*   95 */  false,
  /*   96 */  false,
  // last operand
  /*   97 */  false,
  /*   98 */  false,
  /*   99 */  false,
  /*  100 */  false,
  /*  101 */  false,
  /*  102 */  false,
  /*  103 */  false,
  /*  104 */  false,
  // last operand class
  /*  105 */  false,
  /*  106 */  false,
  /*  107 */  false,
  /*  108 */  false,
  /*  109 */  false,
  /*  110 */  false,
  /*  111 */  false,
  /*  112 */  false,
  /*  113 */  false,
  /*  114 */  false,
  /*  115 */  false,
  /*  116 */  false,
  /*  117 */  false,
  /*  118 */  false,
  /*  119 */  false,
  /*  120 */  false,
  /*  121 */  false,
  /*  122 */  false,
  /*  123 */  false,
  /*  124 */  false,
  /*  125 */  false,
  /*  126 */  false,
  /*  127 */  false,
  /*  128 */  false,
  /*  129 */  false,
  /*  130 */  false,
  /*  131 */  false,
  /*  132 */  false,
  /*  133 */  false,
  /*  134 */  false,
  /*  135 */  false,
  /*  136 */  false,
  /*  137 */  false,
  /*  138 */  false,
  /*  139 */  false,
  /*  140 */  false,
  /*  141 */  false,
  /*  142 */  false,
  /*  143 */  false,
  /*  144 */  false,
  /*  145 */  false,
  /*  146 */  false,
  /*  147 */  false,
  /*  148 */  false,
  /*  149 */  false,
  /*  150 */  false,
  /*  151 */  false,
  /*  152 */  false,
  /*  153 */  false,
  /*  154 */  false,
  /*  155 */  false,
  /*  156 */  false,
  /*  157 */  false,
  /*  158 */  false,
  /*  159 */  false,
  /*  160 */  false,
  /*  161 */  false,
  /*  162 */  false,
  /*  163 */  false,
  /*  164 */  false,
  /*  165 */  false,
  /*  166 */  false,
  /*  167 */  false,
  /*  168 */  false,
  /*  169 */  false,
  /*  170 */  false,
  /*  171 */  false,
  /*  172 */  false,
  /*  173 */  false,
  /*  174 */  false,
  /*  175 */  false,
  /*  176 */  false,
  /*  177 */  false,
  /*  178 */  false,
  /*  179 */  false,
  /*  180 */  false,
  // last internally defined operand
  /*  181 */  false,
  /*  182 */  false,
  /*  183 */  false,
  /*  184 */  false,
  /*  185 */  false,
  /*  186 */  false,
  /*  187 */  false,
  /*  188 */  false,
  /*  189 */  false,
  /*  190 */  false,
  /*  191 */  false,
  /*  192 */  false,
  /*  193 */  false,
  /*  194 */  false,
  /*  195 */  false,
  /*  196 */  false,
  /*  197 */  false,
  /*  198 */  false,
  /*  199 */  false,
  /*  200 */  false,
  /*  201 */  false,
  /*  202 */  false,
  /*  203 */  false,
  /*  204 */  false,
  /*  205 */  false,
  /*  206 */  false,
  /*  207 */  false,
  /*  208 */  false,
  /*  209 */  false,
  /*  210 */  false,
  /*  211 */  false,
  /*  212 */  false,
  /*  213 */  false,
  /*  214 */  false,
  /*  215 */  false,
  /*  216 */  false,
  /*  217 */  false,
  /*  218 */  false,
  /*  219 */  false,
  /*  220 */  false,
  /*  221 */  false,
  /*  222 */  false,
  /*  223 */  false,
  /*  224 */  false,
  /*  225 */  false,
  /*  226 */  false,
  /*  227 */  false,
  /*  228 */  false,
  /*  229 */  false,
  /*  230 */  false,
  /*  231 */  false,
  /*  232 */  false,
  /*  233 */  false,
  /*  234 */  false,
  /*  235 */  false,
  /*  236 */  false,
  /*  237 */  false,
  /*  238 */  false,
  /*  239 */  false,
  /*  240 */  false,
  /*  241 */  false,
  /*  242 */  false,
  /*  243 */  false,
  /*  244 */  false,
  /*  245 */  false,
  /*  246 */  false,
  /*  247 */  false,
  /*  248 */  false,
  /*  249 */  false,
  /*  250 */  false,
  /*  251 */  false,
  /*  252 */  false,
  /*  253 */  false,
  /*  254 */  false,
  /*  255 */  false,
  /*  256 */  false,
  /*  257 */  false,
  /*  258 */  false,
  /*  259 */  false,
  /*  260 */  false,
  /*  261 */  false,
  /*  262 */  false,
  /*  263 */  false,
  /*  264 */  false,
  /*  265 */  false,
  /*  266 */  false,
  /*  267 */  false,
  /*  268 */  false,
  /*  269 */  false,
  /*  270 */  false,
  /*  271 */  false,
  /*  272 */  false,
  /*  273 */  false,
  /*  274 */  false,
  /*  275 */  false,
  /*  276 */  false,
  /*  277 */  false,
  /*  278 */  false,
  /*  279 */  false,
  /*  280 */  false,
  /*  281 */  false,
  /*  282 */  false,
  /*  283 */  false,
  /*  284 */  false,
  /*  285 */  false,
  /*  286 */  false,
  /*  287 */  false,
  /*  288 */  false,
  /*  289 */  false,
  /*  290 */  false,
  /*  291 */  false,
  /*  292 */  false,
  /*  293 */  false,
  /*  294 */  false,
  /*  295 */  false,
  /*  296 */  false,
  /*  297 */  false,
  /*  298 */  false,
  /*  299 */  false,
  /*  300 */  false,
  /*  301 */  false,
  /*  302 */  false,
  /*  303 */  false,
  /*  304 */  false,
  /*  305 */  false,
  /*  306 */  false,
  /*  307 */  false,
  /*  308 */  false,
  /*  309 */  false,
  /*  310 */  false,
  /*  311 */  false,
  /*  312 */  false,
  /*  313 */  false,
  /*  314 */  false,
  /*  315 */  false,
  /*  316 */  false,
  /*  317 */  false,
  /*  318 */  false,
  /*  319 */  false,
  /*  320 */  false,
  /*  321 */  false,
  /*  322 */  false,
  /*  323 */  false,
  /*  324 */  false,
  /*  325 */  false,
  /*  326 */  false,
  /*  327 */  false,
  /*  328 */  false,
  /*  329 */  false,
  /*  330 */  false,
  /*  331 */  false,
  /*  332 */  false,
  /*  333 */  false,
  /*  334 */  false,
  /*  335 */  false,
  /*  336 */  false,
  /*  337 */  false,
  /*  338 */  false,
  /*  339 */  false,
  /*  340 */  false,
  /*  341 */  false,
  /*  342 */  false,
  /*  343 */  false,
  /*  344 */  false,
  /*  345 */  false,
  /*  346 */  false,
  /*  347 */  false,
  /*  348 */  false,
  /*  349 */  false,
  /*  350 */  false,
  /*  351 */  false,
  /*  352 */  false,
  /*  353 */  false,
  /*  354 */  false,
  /*  355 */  false,
  /*  356 */  false,
  /*  357 */  false,
  /*  358 */  false,
  /*  359 */  false,
  /*  360 */  false,
  /*  361 */  false,
  /*  362 */  false,
  /*  363 */  false,
  /*  364 */  false,
  /*  365 */  false,
  /*  366 */  false,
  /*  367 */  false,
  /*  368 */  false,
  /*  369 */  false,
  /*  370 */  false,
  /*  371 */  false,
  /*  372 */  false,
  /*  373 */  false,
  /*  374 */  false,
  /*  375 */  false,
  /*  376 */  false,
  /*  377 */  false,
  /*  378 */  false,
  /*  379 */  false,
  /*  380 */  false,
  /*  381 */  false,
  /*  382 */  false,
  /*  383 */  false,
  /*  384 */  false,
  /*  385 */  false,
  /*  386 */  false,
  /*  387 */  false,
  /*  388 */  false,
  /*  389 */  false,
  /*  390 */  false,
  /*  391 */  false,
  /*  392 */  false,
  /*  393 */  false,
  /*  394 */  false,
  /*  395 */  false,
  /*  396 */  false,
  /*  397 */  false,
  /*  398 */  false,
  /*  399 */  false,
  /*  400 */  false,
  /*  401 */  false,
  /*  402 */  false,
  /*  403 */  false,
  /*  404 */  false,
  /*  405 */  false,
  /*  406 */  false,
  /*  407 */  false,
  /*  408 */  false,
  /*  409 */  false,
  /*  410 */  false,
  /*  411 */  false,
  /*  412 */  false,
  /*  413 */  false,
  /*  414 */  false,
  /*  415 */  false,
  /*  416 */  false,
  /*  417 */  false,
  /*  418 */  false,
  /*  419 */  false,
  /*  420 */  false,
  /*  421 */  false,
  /*  422 */  false,
  /*  423 */  false,
  /*  424 */  false,
  /*  425 */  false,
  /*  426 */  false,
  /*  427 */  false,
  /*  428 */  false,
  /*  429 */  false,
  /*  430 */  false,
  /*  431 */  false,
  /*  432 */  false,
  /*  433 */  false,
  /*  434 */  false,
  /*  435 */  false,
  /*  436 */  false,
  /*  437 */  false,
  /*  438 */  false,
  /*  439 */  false,
  /*  440 */  false,
  /*  441 */  false,
  /*  442 */  false,
  /*  443 */  false,
  /*  444 */  false,
  /*  445 */  false,
  /*  446 */  false,
  /*  447 */  false,
  /*  448 */  false,
  /*  449 */  false,
  /*  450 */  false,
  /*  451 */  false,
  /*  452 */  false,
  /*  453 */  false,
  /*  454 */  false,
  /*  455 */  false,
  /*  456 */  false,
  /*  457 */  false,
  /*  458 */  false,
  /*  459 */  false,
  /*  460 */  false,
  /*  461 */  false,
  /*  462 */  false,
  /*  463 */  false,
  /*  464 */  false,
  /*  465 */  false,
  /*  466 */  false,
  /*  467 */  false,
  /*  468 */  false,
  /*  469 */  false,
  /*  470 */  false,
  /*  471 */  false,
  /*  472 */  false,
  /*  473 */  false,
  /*  474 */  false,
  /*  475 */  false,
  /*  476 */  false,
  /*  477 */  false,
  /*  478 */  false,
  /*  479 */  false,
  /*  480 */  false,
  /*  481 */  false,
  /*  482 */  false,
  /*  483 */  false,
  /*  484 */  false,
  /*  485 */  false,
  /*  486 */  false,
  /*  487 */  false,
  /*  488 */  false,
  /*  489 */  false,
  /*  490 */  false,
  /*  491 */  false,
  /*  492 */  false,
  /*  493 */  false,
  /*  494 */  false,
  /*  495 */  false,
  /*  496 */  false,
  /*  497 */  false,
  /*  498 */  false,
  /*  499 */  false,
  /*  500 */  false,
  /*  501 */  false,
  /*  502 */  false,
  /*  503 */  false,
  /*  504 */  false,
  /*  505 */  false,
  /*  506 */  false,
  /*  507 */  false,
  /*  508 */  false,
  /*  509 */  false,
  /*  510 */  false,
  /*  511 */  false,
  /*  512 */  false,
  /*  513 */  false,
  /*  514 */  false,
  /*  515 */  false,
  /*  516 */  false,
  /*  517 */  false,
  /*  518 */  false,
  /*  519 */  false,
  /*  520 */  false,
  /*  521 */  false,
  /*  522 */  false,
  /*  523 */  false,
  /*  524 */  false,
  /*  525 */  false,
  /*  526 */  false,
  /*  527 */  false,
  /*  528 */  false,
  /*  529 */  false,
  /*  530 */  false,
  /*  531 */  false,
  /*  532 */  false,
  /*  533 */  false,
  /*  534 */  false,
  /*  535 */  false,
  /*  536 */  false,
  /*  537 */  false,
  /*  538 */  false,
  /*  539 */  false,
  /*  540 */  false,
  /*  541 */  false,
  /*  542 */  false,
  /*  543 */  false,
  /*  544 */  false,
  /*  545 */  false,
  /*  546 */  false,
  /*  547 */  false,
  /*  548 */  false,
  /*  549 */  false,
  /*  550 */  false,
  /*  551 */  false,
  /*  552 */  false,
  /*  553 */  false,
  /*  554 */  false,
  /*  555 */  false,
  /*  556 */  false,
  /*  557 */  false,
  /*  558 */  false,
  /*  559 */  false,
  /*  560 */  false,
  /*  561 */  false,
  /*  562 */  false,
  /*  563 */  false,
  /*  564 */  false,
  /*  565 */  false,
  /*  566 */  false,
  /*  567 */  false,
  /*  568 */  false,
  /*  569 */  false,
  /*  570 */  false,
  /*  571 */  false,
  /*  572 */  false,
  /*  573 */  false,
  /*  574 */  false,
  /*  575 */  false,
  /*  576 */  false,
  /*  577 */  false,
  /*  578 */  false,
  /*  579 */  false,
  /*  580 */  false,
  /*  581 */  false,
  /*  582 */  false,
  /*  583 */  false,
  /*  584 */  false,
  /*  585 */  false,
  /*  586 */  false,
  /*  587 */  false,
  /*  588 */  false,
  /*  589 */  false,
  /*  590 */  false,
  /*  591 */  false,
  /*  592 */  false,
  /*  593 */  false,
  /*  594 */  false,
  /*  595 */  false,
  /*  596 */  false,
  /*  597 */  false,
  /*  598 */  false,
  /*  599 */  false,
  /*  600 */  false,
  /*  601 */  false,
  /*  602 */  false,
  /*  603 */  false,
  /*  604 */  false,
  /*  605 */  false,
  /*  606 */  false,
  /*  607 */  false,
  /*  608 */  false,
  /*  609 */  false,
  /*  610 */  false,
  /*  611 */  false,
  /*  612 */  false,
  /*  613 */  false,
  /*  614 */  false,
  /*  615 */  false,
  /*  616 */  false,
  /*  617 */  false,
  /*  618 */  false,
  /*  619 */  false,
  /*  620 */  false,
  /*  621 */  false,
  /*  622 */  false,
  /*  623 */  false,
  /*  624 */  false,
  /*  625 */  false,
  /*  626 */  false,
  /*  627 */  false,
  /*  628 */  false,
  /*  629 */  false,
  /*  630 */  false,
  /*  631 */  false,
  /*  632 */  false,
  /*  633 */  false,
  /*  634 */  false,
  /*  635 */  false,
  /*  636 */  false,
  /*  637 */  false,
  /*  638 */  false,
  /*  639 */  false,
  /*  640 */  false,
  /*  641 */  false,
  /*  642 */  false,
  /*  643 */  false,
  /*  644 */  false,
  /*  645 */  false,
  /*  646 */  false,
  /*  647 */  false,
  /*  648 */  false,
  /*  649 */  false,
  /*  650 */  false,
  /*  651 */  false,
  /*  652 */  false,
  /*  653 */  false,
  /*  654 */  false,
  /*  655 */  false,
  /*  656 */  false,
  /*  657 */  false,
  /*  658 */  false,
  /*  659 */  false,
  /*  660 */  false,
  /*  661 */  false,
  /*  662 */  false,
  /*  663 */  false,
  /*  664 */  false,
  /*  665 */  false,
  /*  666 */  false,
  /*  667 */  false,
  /*  668 */  false,
  /*  669 */  false,
  /*  670 */  false,
  /*  671 */  false,
  /*  672 */  false,
  /*  673 */  false,
  /*  674 */  false,
  /*  675 */  false,
  /*  676 */  false,
  /*  677 */  false,
  /*  678 */  false,
  /*  679 */  false,
  /*  680 */  false,
  /*  681 */  false,
  /*  682 */  false,
  /*  683 */  false,
  /*  684 */  false,
  /*  685 */  false,
  /*  686 */  false,
  /*  687 */  false,
  /*  688 */  false,
  /*  689 */  false,
  /*  690 */  false,
  /*  691 */  false,
  /*  692 */  false,
  /*  693 */  false,
  /*  694 */  false,
  /*  695 */  false,
  /*  696 */  false,
  /*  697 */  false,
  /*  698 */  false,
  /*  699 */  false,
  /*  700 */  false,
  /*  701 */  false,
  /*  702 */  false,
  /*  703 */  false,
  /*  704 */  false,
  /*  705 */  false,
  /*  706 */  false,
  /*  707 */  false,
  /*  708 */  false,
  /*  709 */  false,
  /*  710 */  false,
  /*  711 */  false,
  /*  712 */  false,
  /*  713 */  false,
  /*  714 */  false,
  /*  715 */  false,
  /*  716 */  false,
  /*  717 */  false,
  /*  718 */  false,
  /*  719 */  false,
  /*  720 */  false,
  /*  721 */  false,
  /*  722 */  false,
  /*  723 */  false,
  /*  724 */  false,
  /*  725 */  false,
  /*  726 */  false,
  /*  727 */  false,
  /*  728 */  false,
  /*  729 */  false,
  /*  730 */  false,
  /*  731 */  false,
  /*  732 */  false,
  /*  733 */  false,
  /*  734 */  false,
  /*  735 */  false,
  /*  736 */  false,
  /*  737 */  false,
  /*  738 */  false,
  /*  739 */  false,
  /*  740 */  false,
  /*  741 */  false,
  /*  742 */  false,
  /*  743 */  false,
  /*  744 */  false,
  /*  745 */  false,
  /*  746 */  false,
  /*  747 */  false,
  // last instruction
  false // no trailing comma
};

// Mapping from machine-independent opcode to boolean
const        char must_clone[] = {
  0, // Node: 0
  0, // Set: 1
  0, // RegN: 2
  0, // RegI: 3
  0, // RegP: 4
  0, // RegF: 5
  0, // RegD: 6
  0, // RegL: 7
  0, // VecA: 8
  0, // VecS: 9
  0, // VecD: 10
  0, // VecX: 11
  0, // VecY: 12
  0, // VecZ: 13
  0, // RegVectMask: 14
  0, // RegFlags: 15
  0, // _last_machine_leaf: 16
  0, // AbsD: 17
  0, // AbsF: 18
  0, // AbsI: 19
  0, // AbsL: 20
  0, // AddD: 21
  0, // AddF: 22
  0, // AddI: 23
  0, // AddL: 24
  0, // AddP: 25
  0, // Allocate: 26
  0, // AllocateArray: 27
  0, // AndI: 28
  0, // AndL: 29
  0, // ArrayCopy: 30
  0, // AryEq: 31
  0, // AtanD: 32
  1, // Binary: 33
  0, // Blackhole: 34
  1, // Bool: 35
  0, // BoxLock: 36
  0, // ReverseBytesI: 37
  0, // ReverseBytesL: 38
  0, // ReverseBytesUS: 39
  0, // ReverseBytesS: 40
  0, // ReverseBytesV: 41
  0, // CProj: 42
  0, // CacheWB: 43
  0, // CacheWBPreSync: 44
  0, // CacheWBPostSync: 45
  0, // CallDynamicJava: 46
  0, // CallJava: 47
  0, // CallLeaf: 48
  0, // CallLeafNoFP: 49
  0, // CallLeafVector: 50
  0, // CallRuntime: 51
  0, // CallStaticJava: 52
  0, // CastDD: 53
  0, // CastFF: 54
  0, // CastII: 55
  0, // CastLL: 56
  0, // CastVV: 57
  0, // CastX2P: 58
  0, // CastP2X: 59
  0, // CastPP: 60
  0, // Catch: 61
  0, // CatchProj: 62
  0, // CheckCastPP: 63
  0, // ClearArray: 64
  0, // CompressBits: 65
  0, // ExpandBits: 66
  0, // ConstraintCast: 67
  0, // CMoveD: 68
  0, // CMoveVD: 69
  0, // CMoveF: 70
  0, // CMoveVF: 71
  0, // CMoveI: 72
  0, // CMoveL: 73
  0, // CMoveP: 74
  0, // CMoveN: 75
  1, // CmpN: 76
  1, // CmpD: 77
  0, // CmpD3: 78
  1, // CmpF: 79
  0, // CmpF3: 80
  1, // CmpI: 81
  1, // CmpL: 82
  0, // CmpL3: 83
  0, // CmpLTMask: 84
  1, // CmpP: 85
  1, // CmpU: 86
  1, // CmpUL: 87
  0, // CompareAndSwapB: 88
  0, // CompareAndSwapS: 89
  0, // CompareAndSwapI: 90
  0, // CompareAndSwapL: 91
  0, // CompareAndSwapP: 92
  0, // CompareAndSwapN: 93
  0, // WeakCompareAndSwapB: 94
  0, // WeakCompareAndSwapS: 95
  0, // WeakCompareAndSwapI: 96
  0, // WeakCompareAndSwapL: 97
  0, // WeakCompareAndSwapP: 98
  0, // WeakCompareAndSwapN: 99
  0, // CompareAndExchangeB: 100
  0, // CompareAndExchangeS: 101
  0, // CompareAndExchangeI: 102
  0, // CompareAndExchangeL: 103
  0, // CompareAndExchangeP: 104
  0, // CompareAndExchangeN: 105
  0, // GetAndAddB: 106
  0, // GetAndAddS: 107
  0, // GetAndAddI: 108
  0, // GetAndAddL: 109
  0, // GetAndSetB: 110
  0, // GetAndSetS: 111
  0, // GetAndSetI: 112
  0, // GetAndSetL: 113
  0, // GetAndSetP: 114
  0, // GetAndSetN: 115
  0, // Con: 116
  0, // ConN: 117
  0, // ConNKlass: 118
  0, // ConD: 119
  0, // ConF: 120
  0, // ConI: 121
  0, // ConL: 122
  0, // ConP: 123
  0, // Conv2B: 124
  0, // ConvD2F: 125
  0, // ConvD2I: 126
  0, // ConvD2L: 127
  0, // ConvF2D: 128
  0, // ConvF2I: 129
  0, // ConvF2L: 130
  0, // ConvI2D: 131
  0, // ConvI2F: 132
  0, // ConvI2L: 133
  0, // ConvL2D: 134
  0, // ConvL2F: 135
  0, // ConvL2I: 136
  0, // CountedLoop: 137
  0, // CountedLoopEnd: 138
  0, // OuterStripMinedLoop: 139
  0, // OuterStripMinedLoopEnd: 140
  0, // LongCountedLoop: 141
  0, // LongCountedLoopEnd: 142
  0, // CountLeadingZerosI: 143
  0, // CountLeadingZerosL: 144
  0, // CountLeadingZerosV: 145
  0, // CountTrailingZerosI: 146
  0, // CountTrailingZerosL: 147
  0, // CountTrailingZerosV: 148
  0, // CreateEx: 149
  0, // DecodeN: 150
  0, // DecodeNKlass: 151
  0, // DivD: 152
  0, // DivF: 153
  0, // DivI: 154
  0, // DivL: 155
  0, // UDivI: 156
  0, // UDivL: 157
  0, // DivMod: 158
  0, // DivModI: 159
  0, // DivModL: 160
  0, // UDivModI: 161
  0, // UDivModL: 162
  0, // EncodeISOArray: 163
  0, // EncodeP: 164
  0, // EncodePKlass: 165
  1, // FastLock: 166
  1, // FastUnlock: 167
  0, // FmaD: 168
  0, // FmaF: 169
  0, // Goto: 170
  0, // Halt: 171
  0, // CountPositives: 172
  0, // If: 173
  0, // RangeCheck: 174
  0, // IfFalse: 175
  0, // IfTrue: 176
  0, // Initialize: 177
  0, // JProj: 178
  0, // Jump: 179
  0, // JumpProj: 180
  0, // LShiftI: 181
  0, // LShiftL: 182
  0, // LoadB: 183
  0, // LoadUB: 184
  0, // LoadUS: 185
  0, // LoadD: 186
  0, // LoadD_unaligned: 187
  0, // LoadF: 188
  0, // LoadI: 189
  0, // LoadKlass: 190
  0, // LoadNKlass: 191
  0, // LoadL: 192
  0, // LoadL_unaligned: 193
  0, // LoadPLocked: 194
  0, // LoadP: 195
  0, // LoadN: 196
  0, // LoadRange: 197
  0, // LoadS: 198
  0, // Lock: 199
  0, // Loop: 200
  0, // LoopLimit: 201
  0, // Mach: 202
  0, // MachNullCheck: 203
  0, // MachProj: 204
  0, // MulAddS2I: 205
  0, // MaxI: 206
  0, // MaxL: 207
  0, // MaxD: 208
  0, // MaxF: 209
  0, // MemBarAcquire: 210
  0, // LoadFence: 211
  0, // MemBarAcquireLock: 212
  0, // MemBarCPUOrder: 213
  0, // MemBarRelease: 214
  0, // StoreFence: 215
  0, // StoreStoreFence: 216
  0, // MemBarReleaseLock: 217
  0, // MemBarVolatile: 218
  0, // MemBarStoreStore: 219
  0, // MergeMem: 220
  0, // MinI: 221
  0, // MinL: 222
  0, // MinF: 223
  0, // MinD: 224
  0, // ModD: 225
  0, // ModF: 226
  0, // ModI: 227
  0, // ModL: 228
  0, // UModI: 229
  0, // UModL: 230
  0, // MoveI2F: 231
  0, // MoveF2I: 232
  0, // MoveL2D: 233
  0, // MoveD2L: 234
  0, // IsInfiniteF: 235
  0, // IsInfiniteD: 236
  0, // MulD: 237
  0, // MulF: 238
  0, // MulHiL: 239
  0, // UMulHiL: 240
  0, // MulI: 241
  0, // MulL: 242
  0, // Multi: 243
  0, // NegI: 244
  0, // NegL: 245
  0, // NegD: 246
  0, // NegF: 247
  0, // NeverBranch: 248
  0, // OnSpinWait: 249
  0, // Opaque1: 250
  0, // OpaqueLoopInit: 251
  0, // OpaqueLoopStride: 252
  0, // Opaque2: 253
  0, // Opaque3: 254
  0, // Opaque4: 255
  0, // ProfileBoolean: 256
  0, // OrI: 257
  0, // OrL: 258
  1, // OverflowAddI: 259
  1, // OverflowSubI: 260
  1, // OverflowMulI: 261
  1, // OverflowAddL: 262
  1, // OverflowSubL: 263
  1, // OverflowMulL: 264
  0, // PCTable: 265
  0, // Parm: 266
  0, // PartialSubtypeCheck: 267
  0, // SubTypeCheck: 268
  0, // Phi: 269
  0, // PopCountI: 270
  0, // PopCountL: 271
  0, // PopCountVI: 272
  0, // PopCountVL: 273
  0, // PopulateIndex: 274
  0, // PrefetchAllocation: 275
  0, // Proj: 276
  0, // RShiftI: 277
  0, // RShiftL: 278
  0, // Region: 279
  0, // Rethrow: 280
  0, // Return: 281
  0, // ReverseI: 282
  0, // ReverseL: 283
  0, // ReverseV: 284
  0, // Root: 285
  0, // RoundDouble: 286
  0, // RoundDoubleMode: 287
  0, // RoundDoubleModeV: 288
  0, // RoundFloat: 289
  0, // RotateLeft: 290
  0, // RotateLeftV: 291
  0, // RotateRight: 292
  0, // RotateRightV: 293
  0, // SafePoint: 294
  0, // SafePointScalarObject: 295
  0, // ShenandoahCompareAndExchangeP: 296
  0, // ShenandoahCompareAndExchangeN: 297
  0, // ShenandoahCompareAndSwapN: 298
  0, // ShenandoahCompareAndSwapP: 299
  0, // ShenandoahWeakCompareAndSwapN: 300
  0, // ShenandoahWeakCompareAndSwapP: 301
  0, // ShenandoahIUBarrier: 302
  0, // ShenandoahLoadReferenceBarrier: 303
  0, // SCMemProj: 304
  0, // CopySignD: 305
  0, // CopySignF: 306
  0, // SignumD: 307
  0, // SignumF: 308
  0, // SignumVF: 309
  0, // SignumVD: 310
  0, // SqrtD: 311
  0, // SqrtF: 312
  0, // RoundF: 313
  0, // RoundD: 314
  0, // Start: 315
  0, // StartOSR: 316
  0, // StoreB: 317
  0, // StoreC: 318
  0, // StoreCM: 319
  0, // StorePConditional: 320
  0, // StoreIConditional: 321
  0, // StoreLConditional: 322
  0, // StoreD: 323
  0, // StoreF: 324
  0, // StoreI: 325
  0, // StoreL: 326
  0, // StoreP: 327
  0, // StoreN: 328
  0, // StoreNKlass: 329
  0, // StrComp: 330
  0, // StrCompressedCopy: 331
  0, // StrEquals: 332
  0, // StrIndexOf: 333
  0, // StrIndexOfChar: 334
  0, // StrInflatedCopy: 335
  0, // SubD: 336
  0, // SubF: 337
  0, // SubI: 338
  0, // SubL: 339
  0, // TailCall: 340
  0, // TailJump: 341
  0, // MacroLogicV: 342
  0, // ThreadLocal: 343
  0, // Unlock: 344
  0, // URShiftB: 345
  0, // URShiftS: 346
  0, // URShiftI: 347
  0, // URShiftL: 348
  0, // XorI: 349
  0, // XorL: 350
  0, // Vector: 351
  0, // AddVB: 352
  0, // AddVS: 353
  0, // AddVI: 354
  0, // AddReductionVI: 355
  0, // AddVL: 356
  0, // AddReductionVL: 357
  0, // AddVF: 358
  0, // AddReductionVF: 359
  0, // AddVD: 360
  0, // AddReductionVD: 361
  0, // SubVB: 362
  0, // SubVS: 363
  0, // SubVI: 364
  0, // SubVL: 365
  0, // SubVF: 366
  0, // SubVD: 367
  0, // MulVB: 368
  0, // MulVS: 369
  0, // MulVI: 370
  0, // MulReductionVI: 371
  0, // MulVL: 372
  0, // MulReductionVL: 373
  0, // MulVF: 374
  0, // MulReductionVF: 375
  0, // MulVD: 376
  0, // MulReductionVD: 377
  0, // MulAddVS2VI: 378
  0, // FmaVD: 379
  0, // FmaVF: 380
  0, // DivVF: 381
  0, // DivVD: 382
  0, // AbsVB: 383
  0, // AbsVS: 384
  0, // AbsVI: 385
  0, // AbsVL: 386
  0, // AbsVF: 387
  0, // AbsVD: 388
  0, // NegVI: 389
  0, // NegVL: 390
  0, // NegVF: 391
  0, // NegVD: 392
  0, // SqrtVD: 393
  0, // SqrtVF: 394
  0, // LShiftCntV: 395
  0, // RShiftCntV: 396
  0, // LShiftVB: 397
  0, // LShiftVS: 398
  0, // LShiftVI: 399
  0, // LShiftVL: 400
  0, // RShiftVB: 401
  0, // RShiftVS: 402
  0, // RShiftVI: 403
  0, // RShiftVL: 404
  0, // URShiftVB: 405
  0, // URShiftVS: 406
  0, // URShiftVI: 407
  0, // URShiftVL: 408
  0, // AndV: 409
  0, // AndReductionV: 410
  0, // OrV: 411
  0, // OrReductionV: 412
  0, // XorV: 413
  0, // XorReductionV: 414
  0, // MinV: 415
  0, // MaxV: 416
  0, // MinReductionV: 417
  0, // MaxReductionV: 418
  0, // CompressV: 419
  0, // CompressM: 420
  0, // ExpandV: 421
  0, // LoadVector: 422
  0, // LoadVectorGather: 423
  0, // LoadVectorGatherMasked: 424
  0, // StoreVector: 425
  0, // StoreVectorScatter: 426
  0, // StoreVectorScatterMasked: 427
  0, // LoadVectorMasked: 428
  0, // StoreVectorMasked: 429
  0, // VectorCmpMasked: 430
  0, // VectorMaskGen: 431
  0, // VectorMaskOp: 432
  0, // VectorMaskTrueCount: 433
  0, // VectorMaskFirstTrue: 434
  0, // VectorMaskLastTrue: 435
  0, // VectorMaskToLong: 436
  0, // VectorLongToMask: 437
  0, // Pack: 438
  0, // PackB: 439
  0, // PackS: 440
  0, // PackI: 441
  0, // PackL: 442
  0, // PackF: 443
  0, // PackD: 444
  0, // Pack2L: 445
  0, // Pack2D: 446
  0, // ReplicateB: 447
  0, // ReplicateS: 448
  0, // ReplicateI: 449
  0, // ReplicateL: 450
  0, // ReplicateF: 451
  0, // ReplicateD: 452
  0, // RoundVF: 453
  0, // RoundVD: 454
  0, // Extract: 455
  0, // ExtractB: 456
  0, // ExtractUB: 457
  0, // ExtractC: 458
  0, // ExtractS: 459
  0, // ExtractI: 460
  0, // ExtractL: 461
  0, // ExtractF: 462
  0, // ExtractD: 463
  0, // Digit: 464
  0, // LowerCase: 465
  0, // UpperCase: 466
  0, // Whitespace: 467
  0, // VectorBox: 468
  0, // VectorBoxAllocate: 469
  0, // VectorUnbox: 470
  0, // VectorMaskWrapper: 471
  0, // VectorMaskCmp: 472
  0, // VectorMaskCast: 473
  0, // VectorTest: 474
  0, // VectorBlend: 475
  0, // VectorRearrange: 476
  0, // VectorLoadMask: 477
  0, // VectorLoadShuffle: 478
  0, // VectorLoadConst: 479
  0, // VectorStoreMask: 480
  0, // VectorReinterpret: 481
  0, // VectorCast: 482
  0, // VectorCastB2X: 483
  0, // VectorCastS2X: 484
  0, // VectorCastI2X: 485
  0, // VectorCastL2X: 486
  0, // VectorCastF2X: 487
  0, // VectorCastD2X: 488
  0, // VectorUCastB2X: 489
  0, // VectorUCastS2X: 490
  0, // VectorUCastI2X: 491
  0, // VectorInsert: 492
  0, // MaskAll: 493
  0, // AndVMask: 494
  0, // OrVMask: 495
  0 // no trailing comma // XorVMask: 496
};
//  The following instructions can cisc-spill



// An array of character pointers to machine register names.
const char *Matcher::regName[REG_COUNT] = {
  "R7",
  "R7_H",
  "R28",
  "R28_H",
  "R29",
  "R29_H",
  "R30",
  "R30_H",
  "R31",
  "R31_H",
  "R10",
  "R10_H",
  "R11",
  "R11_H",
  "R12",
  "R12_H",
  "R13",
  "R13_H",
  "R14",
  "R14_H",
  "R15",
  "R15_H",
  "R16",
  "R16_H",
  "R17",
  "R17_H",
  "R9",
  "R9_H",
  "R18",
  "R18_H",
  "R19",
  "R19_H",
  "R20",
  "R20_H",
  "R21",
  "R21_H",
  "R22",
  "R22_H",
  "R24",
  "R24_H",
  "R25",
  "R25_H",
  "R26",
  "R26_H",
  "R23",
  "R23_H",
  "R27",
  "R27_H",
  "R4",
  "R4_H",
  "R8",
  "R8_H",
  "R0",
  "R0_H",
  "R1",
  "R1_H",
  "R2",
  "R2_H",
  "R3",
  "R3_H",
  "F0",
  "F0_H",
  "F1",
  "F1_H",
  "F2",
  "F2_H",
  "F3",
  "F3_H",
  "F4",
  "F4_H",
  "F5",
  "F5_H",
  "F6",
  "F6_H",
  "F7",
  "F7_H",
  "F28",
  "F28_H",
  "F29",
  "F29_H",
  "F30",
  "F30_H",
  "F31",
  "F31_H",
  "F10",
  "F10_H",
  "F11",
  "F11_H",
  "F12",
  "F12_H",
  "F13",
  "F13_H",
  "F14",
  "F14_H",
  "F15",
  "F15_H",
  "F16",
  "F16_H",
  "F17",
  "F17_H",
  "F8",
  "F8_H",
  "F9",
  "F9_H",
  "F18",
  "F18_H",
  "F19",
  "F19_H",
  "F20",
  "F20_H",
  "F21",
  "F21_H",
  "F22",
  "F22_H",
  "F23",
  "F23_H",
  "F24",
  "F24_H",
  "F25",
  "F25_H",
  "F26",
  "F26_H",
  "F27",
  "F27_H",
  "V0",
  "V0_H",
  "V0_J",
  "V0_K",
  "V1",
  "V1_H",
  "V1_J",
  "V1_K",
  "V2",
  "V2_H",
  "V2_J",
  "V2_K",
  "V3",
  "V3_H",
  "V3_J",
  "V3_K",
  "V4",
  "V4_H",
  "V4_J",
  "V4_K",
  "V5",
  "V5_H",
  "V5_J",
  "V5_K",
  "V6",
  "V6_H",
  "V6_J",
  "V6_K",
  "V7",
  "V7_H",
  "V7_J",
  "V7_K",
  "V8",
  "V8_H",
  "V8_J",
  "V8_K",
  "V9",
  "V9_H",
  "V9_J",
  "V9_K",
  "V10",
  "V10_H",
  "V10_J",
  "V10_K",
  "V11",
  "V11_H",
  "V11_J",
  "V11_K",
  "V12",
  "V12_H",
  "V12_J",
  "V12_K",
  "V13",
  "V13_H",
  "V13_J",
  "V13_K",
  "V14",
  "V14_H",
  "V14_J",
  "V14_K",
  "V15",
  "V15_H",
  "V15_J",
  "V15_K",
  "V16",
  "V16_H",
  "V16_J",
  "V16_K",
  "V17",
  "V17_H",
  "V17_J",
  "V17_K",
  "V18",
  "V18_H",
  "V18_J",
  "V18_K",
  "V19",
  "V19_H",
  "V19_J",
  "V19_K",
  "V20",
  "V20_H",
  "V20_J",
  "V20_K",
  "V21",
  "V21_H",
  "V21_J",
  "V21_K",
  "V22",
  "V22_H",
  "V22_J",
  "V22_K",
  "V23",
  "V23_H",
  "V23_J",
  "V23_K",
  "V24",
  "V24_H",
  "V24_J",
  "V24_K",
  "V25",
  "V25_H",
  "V25_J",
  "V25_K",
  "V26",
  "V26_H",
  "V26_J",
  "V26_K",
  "V27",
  "V27_H",
  "V27_J",
  "V27_K",
  "V28",
  "V28_H",
  "V28_J",
  "V28_K",
  "V29",
  "V29_H",
  "V29_J",
  "V29_K",
  "V30",
  "V30_H",
  "V30_J",
  "V30_K",
  "V31",
  "V31_H",
  "V31_J",
  "V31_K",
  "RFLAGS" // no trailing comma
};

// An array of character pointers to machine register names.
const VMReg OptoReg::opto2vm[REG_COUNT] = {
	x7->as_VMReg()         ,
	x7->as_VMReg()->next() ,
	x28->as_VMReg()        ,
	x28->as_VMReg()->next(),
	x29->as_VMReg()        ,
	x29->as_VMReg()->next(),
	x30->as_VMReg()        ,
	x30->as_VMReg()->next(),
	x31->as_VMReg()        ,
	x31->as_VMReg()->next(),
	x10->as_VMReg()        ,
	x10->as_VMReg()->next(),
	x11->as_VMReg()        ,
	x11->as_VMReg()->next(),
	x12->as_VMReg()        ,
	x12->as_VMReg()->next(),
	x13->as_VMReg()        ,
	x13->as_VMReg()->next(),
	x14->as_VMReg()        ,
	x14->as_VMReg()->next(),
	x15->as_VMReg()        ,
	x15->as_VMReg()->next(),
	x16->as_VMReg()        ,
	x16->as_VMReg()->next(),
	x17->as_VMReg()        ,
	x17->as_VMReg()->next(),
	x9->as_VMReg()         ,
	x9->as_VMReg()->next() ,
	x18->as_VMReg()        ,
	x18->as_VMReg()->next(),
	x19->as_VMReg()        ,
	x19->as_VMReg()->next(),
	x20->as_VMReg()        ,
	x20->as_VMReg()->next(),
	x21->as_VMReg()        ,
	x21->as_VMReg()->next(),
	x22->as_VMReg()        ,
	x22->as_VMReg()->next(),
	x24->as_VMReg()        ,
	x24->as_VMReg()->next(),
	x25->as_VMReg()        ,
	x25->as_VMReg()->next(),
	x26->as_VMReg()        ,
	x26->as_VMReg()->next(),
	x23->as_VMReg()        ,
	x23->as_VMReg()->next(),
	x27->as_VMReg()        ,
	x27->as_VMReg()->next(),
	x4->as_VMReg()         ,
	x4->as_VMReg()->next() ,
	x8->as_VMReg()         ,
	x8->as_VMReg()->next() ,
	x0->as_VMReg()         ,
	x0->as_VMReg()->next() ,
	x1->as_VMReg()         ,
	x1->as_VMReg()->next() ,
	x2->as_VMReg()         ,
	x2->as_VMReg()->next() ,
	x3->as_VMReg()         ,
	x3->as_VMReg()->next() ,
	f0->as_VMReg()          ,
	f0->as_VMReg()->next()  ,
	f1->as_VMReg()          ,
	f1->as_VMReg()->next()  ,
	f2->as_VMReg()          ,
	f2->as_VMReg()->next()  ,
	f3->as_VMReg()          ,
	f3->as_VMReg()->next()  ,
	f4->as_VMReg()          ,
	f4->as_VMReg()->next()  ,
	f5->as_VMReg()          ,
	f5->as_VMReg()->next()  ,
	f6->as_VMReg()          ,
	f6->as_VMReg()->next()  ,
	f7->as_VMReg()          ,
	f7->as_VMReg()->next()  ,
	f28->as_VMReg()         ,
	f28->as_VMReg()->next() ,
	f29->as_VMReg()         ,
	f29->as_VMReg()->next() ,
	f30->as_VMReg()         ,
	f30->as_VMReg()->next() ,
	f31->as_VMReg()         ,
	f31->as_VMReg()->next() ,
	f10->as_VMReg()         ,
	f10->as_VMReg()->next() ,
	f11->as_VMReg()         ,
	f11->as_VMReg()->next() ,
	f12->as_VMReg()         ,
	f12->as_VMReg()->next() ,
	f13->as_VMReg()         ,
	f13->as_VMReg()->next() ,
	f14->as_VMReg()         ,
	f14->as_VMReg()->next() ,
	f15->as_VMReg()         ,
	f15->as_VMReg()->next() ,
	f16->as_VMReg()         ,
	f16->as_VMReg()->next() ,
	f17->as_VMReg()         ,
	f17->as_VMReg()->next() ,
	f8->as_VMReg()          ,
	f8->as_VMReg()->next()  ,
	f9->as_VMReg()          ,
	f9->as_VMReg()->next()  ,
	f18->as_VMReg()         ,
	f18->as_VMReg()->next() ,
	f19->as_VMReg()         ,
	f19->as_VMReg()->next() ,
	f20->as_VMReg()         ,
	f20->as_VMReg()->next() ,
	f21->as_VMReg()         ,
	f21->as_VMReg()->next() ,
	f22->as_VMReg()         ,
	f22->as_VMReg()->next() ,
	f23->as_VMReg()         ,
	f23->as_VMReg()->next() ,
	f24->as_VMReg()         ,
	f24->as_VMReg()->next() ,
	f25->as_VMReg()         ,
	f25->as_VMReg()->next() ,
	f26->as_VMReg()         ,
	f26->as_VMReg()->next() ,
	f27->as_VMReg()         ,
	f27->as_VMReg()->next() ,
	v0->as_VMReg()           ,
	v0->as_VMReg()->next()   ,
	v0->as_VMReg()->next(2)  ,
	v0->as_VMReg()->next(3)  ,
	v1->as_VMReg() 	        ,
	v1->as_VMReg()->next()   ,
	v1->as_VMReg()->next(2)  ,
	v1->as_VMReg()->next(3)  ,
	v2->as_VMReg()           ,
	v2->as_VMReg()->next()   ,
	v2->as_VMReg()->next(2)  ,
	v2->as_VMReg()->next(3)  ,
	v3->as_VMReg()           ,
	v3->as_VMReg()->next()   ,
	v3->as_VMReg()->next(2)  ,
	v3->as_VMReg()->next(3)  ,
	v4->as_VMReg()           ,
	v4->as_VMReg()->next()   ,
	v4->as_VMReg()->next(2)  ,
	v4->as_VMReg()->next(3)  ,
	v5->as_VMReg() 	        ,
	v5->as_VMReg()->next()   ,
	v5->as_VMReg()->next(2)  ,
	v5->as_VMReg()->next(3)  ,
	v6->as_VMReg()           ,
	v6->as_VMReg()->next()   ,
	v6->as_VMReg()->next(2)  ,
	v6->as_VMReg()->next(3)  ,
	v7->as_VMReg() 	        ,
	v7->as_VMReg()->next()   ,
	v7->as_VMReg()->next(2)  ,
	v7->as_VMReg()->next(3)  ,
	v8->as_VMReg()           ,
	v8->as_VMReg()->next()   ,
	v8->as_VMReg()->next(2)  ,
	v8->as_VMReg()->next(3)  ,
	v9->as_VMReg()           ,
	v9->as_VMReg()->next()   ,
	v9->as_VMReg()->next(2)  ,
	v9->as_VMReg()->next(3)  ,
	v10->as_VMReg()          ,
	v10->as_VMReg()->next()  ,
	v10->as_VMReg()->next(2) ,
	v10->as_VMReg()->next(3) ,
	v11->as_VMReg()          ,
	v11->as_VMReg()->next()  ,
	v11->as_VMReg()->next(2) ,
	v11->as_VMReg()->next(3) ,
	v12->as_VMReg()          ,
	v12->as_VMReg()->next()  ,
	v12->as_VMReg()->next(2) ,
	v12->as_VMReg()->next(3) ,
	v13->as_VMReg()          ,
	v13->as_VMReg()->next()  ,
	v13->as_VMReg()->next(2) ,
	v13->as_VMReg()->next(3) ,
	v14->as_VMReg()          ,
	v14->as_VMReg()->next()  ,
	v14->as_VMReg()->next(2) ,
	v14->as_VMReg()->next(3) ,
	v15->as_VMReg()          ,
	v15->as_VMReg()->next()  ,
	v15->as_VMReg()->next(2) ,
	v15->as_VMReg()->next(3) ,
	v16->as_VMReg()          ,
	v16->as_VMReg()->next()  ,
	v16->as_VMReg()->next(2) ,
	v16->as_VMReg()->next(3) ,
	v17->as_VMReg()          ,
	v17->as_VMReg()->next()  ,
	v17->as_VMReg()->next(2) ,
	v17->as_VMReg()->next(3) ,
	v18->as_VMReg()          ,
	v18->as_VMReg()->next()  ,
	v18->as_VMReg()->next(2) ,
	v18->as_VMReg()->next(3) ,
	v19->as_VMReg()          ,
	v19->as_VMReg()->next()  ,
	v19->as_VMReg()->next(2) ,
	v19->as_VMReg()->next(3) ,
	v20->as_VMReg()          ,
	v20->as_VMReg()->next()  ,
	v20->as_VMReg()->next(2) ,
	v20->as_VMReg()->next(3) ,
	v21->as_VMReg()          ,
	v21->as_VMReg()->next()  ,
	v21->as_VMReg()->next(2) ,
	v21->as_VMReg()->next(3) ,
	v22->as_VMReg()          ,
	v22->as_VMReg()->next()  ,
	v22->as_VMReg()->next(2) ,
	v22->as_VMReg()->next(3) ,
	v23->as_VMReg()          ,
	v23->as_VMReg()->next()  ,
	v23->as_VMReg()->next(2) ,
	v23->as_VMReg()->next(3) ,
	v24->as_VMReg()          ,
	v24->as_VMReg()->next()  ,
	v24->as_VMReg()->next(2) ,
	v24->as_VMReg()->next(3) ,
	v25->as_VMReg()          ,
	v25->as_VMReg()->next()  ,
	v25->as_VMReg()->next(2) ,
	v25->as_VMReg()->next(3) ,
	v26->as_VMReg()          ,
	v26->as_VMReg()->next()  ,
	v26->as_VMReg()->next(2) ,
	v26->as_VMReg()->next(3) ,
	v27->as_VMReg()          ,
	v27->as_VMReg()->next()  ,
	v27->as_VMReg()->next(2) ,
	v27->as_VMReg()->next(3) ,
	v28->as_VMReg()          ,
	v28->as_VMReg()->next()  ,
	v28->as_VMReg()->next(2) ,
	v28->as_VMReg()->next(3) ,
	v29->as_VMReg()          ,
	v29->as_VMReg()->next()  ,
	v29->as_VMReg()->next(2) ,
	v29->as_VMReg()->next(3) ,
	v30->as_VMReg()          ,
	v30->as_VMReg()->next()  ,
	v30->as_VMReg()->next(2) ,
	v30->as_VMReg()->next(3) ,
	v31->as_VMReg()          ,
	v31->as_VMReg()->next()  ,
	v31->as_VMReg()->next(2) ,
	v31->as_VMReg()->next(3) ,
	x6->as_VMReg()         // no trailing comma
	};

 OptoReg::Name OptoReg::vm2opto[ConcreteRegisterImpl::number_of_registers];

// An array of the machine register encode values
const unsigned char Matcher::_regEncode[REG_COUNT] = {
  (unsigned char)'\x7',  // R7
  (unsigned char)'\x7',  // R7_H
  (unsigned char)'\x1C',  // R28
  (unsigned char)'\x1C',  // R28_H
  (unsigned char)'\x1D',  // R29
  (unsigned char)'\x1D',  // R29_H
  (unsigned char)'\x1E',  // R30
  (unsigned char)'\x1E',  // R30_H
  (unsigned char)'\x1F',  // R31
  (unsigned char)'\x1F',  // R31_H
  (unsigned char)'\xA',  // R10
  (unsigned char)'\xA',  // R10_H
  (unsigned char)'\xB',  // R11
  (unsigned char)'\xB',  // R11_H
  (unsigned char)'\xC',  // R12
  (unsigned char)'\xC',  // R12_H
  (unsigned char)'\xD',  // R13
  (unsigned char)'\xD',  // R13_H
  (unsigned char)'\xE',  // R14
  (unsigned char)'\xE',  // R14_H
  (unsigned char)'\xF',  // R15
  (unsigned char)'\xF',  // R15_H
  (unsigned char)'\x10',  // R16
  (unsigned char)'\x10',  // R16_H
  (unsigned char)'\x11',  // R17
  (unsigned char)'\x11',  // R17_H
  (unsigned char)'\x9',  // R9
  (unsigned char)'\x9',  // R9_H
  (unsigned char)'\x12',  // R18
  (unsigned char)'\x12',  // R18_H
  (unsigned char)'\x13',  // R19
  (unsigned char)'\x13',  // R19_H
  (unsigned char)'\x14',  // R20
  (unsigned char)'\x14',  // R20_H
  (unsigned char)'\x15',  // R21
  (unsigned char)'\x15',  // R21_H
  (unsigned char)'\x16',  // R22
  (unsigned char)'\x16',  // R22_H
  (unsigned char)'\x18',  // R24
  (unsigned char)'\x18',  // R24_H
  (unsigned char)'\x19',  // R25
  (unsigned char)'\x19',  // R25_H
  (unsigned char)'\x1A',  // R26
  (unsigned char)'\x1A',  // R26_H
  (unsigned char)'\x17',  // R23
  (unsigned char)'\x17',  // R23_H
  (unsigned char)'\x1B',  // R27
  (unsigned char)'\x1B',  // R27_H
  (unsigned char)'\x4',  // R4
  (unsigned char)'\x4',  // R4_H
  (unsigned char)'\x8',  // R8
  (unsigned char)'\x8',  // R8_H
  (unsigned char)'\x0',  // R0
  (unsigned char)'\x0',  // R0_H
  (unsigned char)'\x1',  // R1
  (unsigned char)'\x1',  // R1_H
  (unsigned char)'\x2',  // R2
  (unsigned char)'\x2',  // R2_H
  (unsigned char)'\x3',  // R3
  (unsigned char)'\x3',  // R3_H
  (unsigned char)'\x0',  // F0
  (unsigned char)'\x0',  // F0_H
  (unsigned char)'\x1',  // F1
  (unsigned char)'\x1',  // F1_H
  (unsigned char)'\x2',  // F2
  (unsigned char)'\x2',  // F2_H
  (unsigned char)'\x3',  // F3
  (unsigned char)'\x3',  // F3_H
  (unsigned char)'\x4',  // F4
  (unsigned char)'\x4',  // F4_H
  (unsigned char)'\x5',  // F5
  (unsigned char)'\x5',  // F5_H
  (unsigned char)'\x6',  // F6
  (unsigned char)'\x6',  // F6_H
  (unsigned char)'\x7',  // F7
  (unsigned char)'\x7',  // F7_H
  (unsigned char)'\x1C',  // F28
  (unsigned char)'\x1C',  // F28_H
  (unsigned char)'\x1D',  // F29
  (unsigned char)'\x1D',  // F29_H
  (unsigned char)'\x1E',  // F30
  (unsigned char)'\x1E',  // F30_H
  (unsigned char)'\x1F',  // F31
  (unsigned char)'\x1F',  // F31_H
  (unsigned char)'\xA',  // F10
  (unsigned char)'\xA',  // F10_H
  (unsigned char)'\xB',  // F11
  (unsigned char)'\xB',  // F11_H
  (unsigned char)'\xC',  // F12
  (unsigned char)'\xC',  // F12_H
  (unsigned char)'\xD',  // F13
  (unsigned char)'\xD',  // F13_H
  (unsigned char)'\xE',  // F14
  (unsigned char)'\xE',  // F14_H
  (unsigned char)'\xF',  // F15
  (unsigned char)'\xF',  // F15_H
  (unsigned char)'\x10',  // F16
  (unsigned char)'\x10',  // F16_H
  (unsigned char)'\x11',  // F17
  (unsigned char)'\x11',  // F17_H
  (unsigned char)'\x8',  // F8
  (unsigned char)'\x8',  // F8_H
  (unsigned char)'\x9',  // F9
  (unsigned char)'\x9',  // F9_H
  (unsigned char)'\x12',  // F18
  (unsigned char)'\x12',  // F18_H
  (unsigned char)'\x13',  // F19
  (unsigned char)'\x13',  // F19_H
  (unsigned char)'\x14',  // F20
  (unsigned char)'\x14',  // F20_H
  (unsigned char)'\x15',  // F21
  (unsigned char)'\x15',  // F21_H
  (unsigned char)'\x16',  // F22
  (unsigned char)'\x16',  // F22_H
  (unsigned char)'\x17',  // F23
  (unsigned char)'\x17',  // F23_H
  (unsigned char)'\x18',  // F24
  (unsigned char)'\x18',  // F24_H
  (unsigned char)'\x19',  // F25
  (unsigned char)'\x19',  // F25_H
  (unsigned char)'\x1A',  // F26
  (unsigned char)'\x1A',  // F26_H
  (unsigned char)'\x1B',  // F27
  (unsigned char)'\x1B',  // F27_H
  (unsigned char)'\x0',  // V0
  (unsigned char)'\x0',  // V0_H
  (unsigned char)'\x0',  // V0_J
  (unsigned char)'\x0',  // V0_K
  (unsigned char)'\x1',  // V1
  (unsigned char)'\x1',  // V1_H
  (unsigned char)'\x1',  // V1_J
  (unsigned char)'\x1',  // V1_K
  (unsigned char)'\x2',  // V2
  (unsigned char)'\x2',  // V2_H
  (unsigned char)'\x2',  // V2_J
  (unsigned char)'\x2',  // V2_K
  (unsigned char)'\x3',  // V3
  (unsigned char)'\x3',  // V3_H
  (unsigned char)'\x3',  // V3_J
  (unsigned char)'\x3',  // V3_K
  (unsigned char)'\x4',  // V4
  (unsigned char)'\x4',  // V4_H
  (unsigned char)'\x4',  // V4_J
  (unsigned char)'\x4',  // V4_K
  (unsigned char)'\x5',  // V5
  (unsigned char)'\x5',  // V5_H
  (unsigned char)'\x5',  // V5_J
  (unsigned char)'\x5',  // V5_K
  (unsigned char)'\x6',  // V6
  (unsigned char)'\x6',  // V6_H
  (unsigned char)'\x6',  // V6_J
  (unsigned char)'\x6',  // V6_K
  (unsigned char)'\x7',  // V7
  (unsigned char)'\x7',  // V7_H
  (unsigned char)'\x7',  // V7_J
  (unsigned char)'\x7',  // V7_K
  (unsigned char)'\x8',  // V8
  (unsigned char)'\x8',  // V8_H
  (unsigned char)'\x8',  // V8_J
  (unsigned char)'\x8',  // V8_K
  (unsigned char)'\x9',  // V9
  (unsigned char)'\x9',  // V9_H
  (unsigned char)'\x9',  // V9_J
  (unsigned char)'\x9',  // V9_K
  (unsigned char)'\xA',  // V10
  (unsigned char)'\xA',  // V10_H
  (unsigned char)'\xA',  // V10_J
  (unsigned char)'\xA',  // V10_K
  (unsigned char)'\xB',  // V11
  (unsigned char)'\xB',  // V11_H
  (unsigned char)'\xB',  // V11_J
  (unsigned char)'\xB',  // V11_K
  (unsigned char)'\xC',  // V12
  (unsigned char)'\xC',  // V12_H
  (unsigned char)'\xC',  // V12_J
  (unsigned char)'\xC',  // V12_K
  (unsigned char)'\xD',  // V13
  (unsigned char)'\xD',  // V13_H
  (unsigned char)'\xD',  // V13_J
  (unsigned char)'\xD',  // V13_K
  (unsigned char)'\xE',  // V14
  (unsigned char)'\xE',  // V14_H
  (unsigned char)'\xE',  // V14_J
  (unsigned char)'\xE',  // V14_K
  (unsigned char)'\xF',  // V15
  (unsigned char)'\xF',  // V15_H
  (unsigned char)'\xF',  // V15_J
  (unsigned char)'\xF',  // V15_K
  (unsigned char)'\x10',  // V16
  (unsigned char)'\x10',  // V16_H
  (unsigned char)'\x10',  // V16_J
  (unsigned char)'\x10',  // V16_K
  (unsigned char)'\x11',  // V17
  (unsigned char)'\x11',  // V17_H
  (unsigned char)'\x11',  // V17_J
  (unsigned char)'\x11',  // V17_K
  (unsigned char)'\x12',  // V18
  (unsigned char)'\x12',  // V18_H
  (unsigned char)'\x12',  // V18_J
  (unsigned char)'\x12',  // V18_K
  (unsigned char)'\x13',  // V19
  (unsigned char)'\x13',  // V19_H
  (unsigned char)'\x13',  // V19_J
  (unsigned char)'\x13',  // V19_K
  (unsigned char)'\x14',  // V20
  (unsigned char)'\x14',  // V20_H
  (unsigned char)'\x14',  // V20_J
  (unsigned char)'\x14',  // V20_K
  (unsigned char)'\x15',  // V21
  (unsigned char)'\x15',  // V21_H
  (unsigned char)'\x15',  // V21_J
  (unsigned char)'\x15',  // V21_K
  (unsigned char)'\x16',  // V22
  (unsigned char)'\x16',  // V22_H
  (unsigned char)'\x16',  // V22_J
  (unsigned char)'\x16',  // V22_K
  (unsigned char)'\x17',  // V23
  (unsigned char)'\x17',  // V23_H
  (unsigned char)'\x17',  // V23_J
  (unsigned char)'\x17',  // V23_K
  (unsigned char)'\x18',  // V24
  (unsigned char)'\x18',  // V24_H
  (unsigned char)'\x18',  // V24_J
  (unsigned char)'\x18',  // V24_K
  (unsigned char)'\x19',  // V25
  (unsigned char)'\x19',  // V25_H
  (unsigned char)'\x19',  // V25_J
  (unsigned char)'\x19',  // V25_K
  (unsigned char)'\x1A',  // V26
  (unsigned char)'\x1A',  // V26_H
  (unsigned char)'\x1A',  // V26_J
  (unsigned char)'\x1A',  // V26_K
  (unsigned char)'\x1B',  // V27
  (unsigned char)'\x1B',  // V27_H
  (unsigned char)'\x1B',  // V27_J
  (unsigned char)'\x1B',  // V27_K
  (unsigned char)'\x1C',  // V28
  (unsigned char)'\x1C',  // V28_H
  (unsigned char)'\x1C',  // V28_J
  (unsigned char)'\x1C',  // V28_K
  (unsigned char)'\x1D',  // V29
  (unsigned char)'\x1D',  // V29_H
  (unsigned char)'\x1D',  // V29_J
  (unsigned char)'\x1D',  // V29_K
  (unsigned char)'\x1E',  // V30
  (unsigned char)'\x1E',  // V30_H
  (unsigned char)'\x1E',  // V30_J
  (unsigned char)'\x1E',  // V30_K
  (unsigned char)'\x1F',  // V31
  (unsigned char)'\x1F',  // V31_H
  (unsigned char)'\x1F',  // V31_J
  (unsigned char)'\x1F',  // V31_K
  (unsigned char)'\x6' // no trailing comma  // RFLAGS
};


//------------------Define classes derived from MachOper---------------------
MachOper  *labelOper::clone() const {
  return  new labelOper(_label, _block_num);
}
uint labelOper::opcode() const { return LABEL; }

const RegMask *sRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

MachOper  *methodOper::clone() const {
  return  new methodOper(_method);
}
uint methodOper::opcode() const { return METHOD; }

const RegMask *iRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *iRegINoSpOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NO_SPECIAL_REG32_mask();
}

const RegMask *iRegI_R10Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_R10_REG_mask();
}

const RegMask *iRegI_R12Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_R12_REG_mask();
}

const RegMask *iRegI_R13Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_R13_REG_mask();
}

const RegMask *iRegI_R14Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_R14_REG_mask();
}

const RegMask *iRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask();
}

const RegMask *iRegLNoSpOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NO_SPECIAL_REG_mask();
}

const RegMask *iRegL_R28Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R28_REG_mask();
}

const RegMask *iRegL_R29Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R29_REG_mask();
}

const RegMask *iRegL_R30Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R30_REG_mask();
}

const RegMask *iRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *iRegPNoSpOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NO_SPECIAL_PTR_REG_mask();
}

const RegMask *iRegP_R10Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R10_REG_mask();
}

const RegMask *iRegP_R11Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R11_REG_mask();
}

const RegMask *iRegP_R12Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R12_REG_mask();
}

const RegMask *iRegP_R13Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R13_REG_mask();
}

const RegMask *iRegP_R14Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R14_REG_mask();
}

const RegMask *iRegP_R15Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R15_REG_mask();
}

const RegMask *iRegP_R16Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R16_REG_mask();
}

const RegMask *iRegP_R28Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R28_REG_mask();
}

const RegMask *iRegNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *iRegNNoSpOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NO_SPECIAL_REG32_mask();
}

const RegMask *iRegIHeapbaseOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &HEAPBASE_REG_mask();
}

const RegMask *iRegL_R10Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R10_REG_mask();
}

const RegMask *fRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_REG_mask();
}

const RegMask *fRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DOUBLE_REG_mask();
}

const RegMask *vRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORA_REG_mask();
}

const RegMask *vReg_V1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V1_REG_mask();
}

const RegMask *vReg_V2Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V2_REG_mask();
}

const RegMask *vReg_V3Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V3_REG_mask();
}

const RegMask *vReg_V4Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V4_REG_mask();
}

const RegMask *vReg_V5Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V5_REG_mask();
}

const RegMask *javaThread_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &JAVA_THREAD_REG_mask();
}

const RegMask *indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indirectNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *indOffINOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *indOffLNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *thread_anchor_pcOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *stackSlotIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *iRegL2IOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask();
}

const RegMask *rFlagsRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &REG_FLAGS_mask();
}

const RegMask *inline_cache_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &METHOD_REG_mask();
}

//------------------Define members for classes derived from MachNode----------
// Build short branch version of this instruction
MachNode *far_cmpI_branchNode::short_branch_version() {
  cmpI_branchNode *node = new cmpI_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpI_loopNode::short_branch_version() {
  cmpI_loopNode *node = new cmpI_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpU_branchNode::short_branch_version() {
  cmpU_branchNode *node = new cmpU_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpU_loopNode::short_branch_version() {
  cmpU_loopNode *node = new cmpU_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpL_branchNode::short_branch_version() {
  cmpL_branchNode *node = new cmpL_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpLloopNode::short_branch_version() {
  cmpL_loopNode *node = new cmpL_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpUL_branchNode::short_branch_version() {
  cmpUL_branchNode *node = new cmpUL_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpUL_loopNode::short_branch_version() {
  cmpUL_loopNode *node = new cmpUL_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpP_branchNode::short_branch_version() {
  cmpP_branchNode *node = new cmpP_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpP_loopNode::short_branch_version() {
  cmpP_loopNode *node = new cmpP_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpN_branchNode::short_branch_version() {
  cmpN_branchNode *node = new cmpN_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpN_loopNode::short_branch_version() {
  cmpN_loopNode *node = new cmpN_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpF_branchNode::short_branch_version() {
  cmpF_branchNode *node = new cmpF_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpF_loopNode::short_branch_version() {
  cmpF_loopNode *node = new cmpF_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpD_branchNode::short_branch_version() {
  cmpD_branchNode *node = new cmpD_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpD_loopNode::short_branch_version() {
  cmpD_loopNode *node = new cmpD_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpI_reg_imm0_branchNode::short_branch_version() {
  cmpI_reg_imm0_branchNode *node = new cmpI_reg_imm0_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpI_reg_imm0_loopNode::short_branch_version() {
  cmpI_reg_imm0_loopNode *node = new cmpI_reg_imm0_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpUEqNeLeGt_imm0_branchNode::short_branch_version() {
  cmpUEqNeLeGt_reg_imm0_branchNode *node = new cmpUEqNeLeGt_reg_imm0_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpUEqNeLeGt_reg_imm0_loopNode::short_branch_version() {
  cmpUEqNeLeGt_reg_imm0_loopNode *node = new cmpUEqNeLeGt_reg_imm0_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpL_reg_imm0_branchNode::short_branch_version() {
  cmpL_reg_imm0_branchNode *node = new cmpL_reg_imm0_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpL_reg_imm0_loopNode::short_branch_version() {
  cmpL_reg_imm0_loopNode *node = new cmpL_reg_imm0_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpULEqNeLeGt_reg_imm0_branchNode::short_branch_version() {
  cmpULEqNeLeGt_reg_imm0_branchNode *node = new cmpULEqNeLeGt_reg_imm0_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpULEqNeLeGt_reg_imm0_loopNode::short_branch_version() {
  cmpULEqNeLeGt_reg_imm0_loopNode *node = new cmpULEqNeLeGt_reg_imm0_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpP_imm0_branchNode::short_branch_version() {
  cmpP_imm0_branchNode *node = new cmpP_imm0_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpP_imm0_loopNode::short_branch_version() {
  cmpP_imm0_loopNode *node = new cmpP_imm0_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpN_imm0_branchNode::short_branch_version() {
  cmpN_imm0_branchNode *node = new cmpN_imm0_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpN_imm0_loopNode::short_branch_version() {
  cmpN_imm0_loopNode *node = new cmpN_imm0_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpP_narrowOop_imm0_branchNode::short_branch_version() {
  cmpP_narrowOop_imm0_branchNode *node = new cmpP_narrowOop_imm0_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpP_narrowOop_imm0_loopNode::short_branch_version() {
  cmpP_narrowOop_imm0_loopNode *node = new cmpP_narrowOop_imm0_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}


// Copy _idx, inputs and operands to new node
void MachNode::fill_new_machnode(MachNode* node) const {
  // New node must use same node index
  node->set_idx( _idx );
  // Copy machine-independent inputs
  for( uint j = 0; j < req(); j++ ) {
    node->add_req(in(j));
  }
  // Copy my operands, except for cisc position
  int nopnds = num_opnds();
  assert( node->num_opnds() == (uint)nopnds, "Must have same number of operands");
  MachOper **to = node->_opnds;
  for( int i = 0; i < nopnds; i++ ) {
    if( i != cisc_operand() ) 
      to[i] = _opnds[i]->clone();
  }
}

void branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpFlag_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpFlag_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpI_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpI_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpI_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpI_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpU_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpU_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpU_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpU_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUL_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUL_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUL_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUL_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpP_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpP_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpP_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpP_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpN_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpN_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpN_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpN_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpF_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpF_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpF_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpF_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpD_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpD_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpD_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpD_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpI_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpI_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpI_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpI_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUEqNeLeGt_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUEqNeLeGt_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUEqNeLeGt_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUEqNeLeGt_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpULEqNeLeGt_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpULEqNeLeGt_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpULEqNeLeGt_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpULEqNeLeGt_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpP_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpP_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpP_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpP_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpN_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpN_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpN_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpN_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpP_narrowOop_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpP_narrowOop_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpP_narrowOop_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpP_narrowOop_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpFlag_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpFlag_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpI_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpI_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpI_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpI_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpU_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpU_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpU_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpU_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpL_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpL_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpLloopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpLloopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpUL_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpUL_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpUL_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpUL_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpP_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpP_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpP_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpP_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpN_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpN_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpN_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpN_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpF_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpF_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpF_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpF_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpD_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpD_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpD_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpD_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpI_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpI_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpI_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpI_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpUEqNeLeGt_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpUEqNeLeGt_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpUEqNeLeGt_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpUEqNeLeGt_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpULtGe_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpULtGe_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpULtGe_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpULtGe_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpL_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpL_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpL_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpL_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpULEqNeLeGt_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpULEqNeLeGt_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpULEqNeLeGt_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpULEqNeLeGt_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpULLtGe_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpULLtGe_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpULLtGe_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpULLtGe_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpP_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpP_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpP_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpP_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpN_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpN_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpN_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpN_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpP_narrowOop_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpP_narrowOop_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpP_narrowOop_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpP_narrowOop_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void CallStaticJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallDynamicJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallRuntimeDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafNoFPDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

int loadBNode::reloc() const {
  return 1;
}

int loadB2LNode::reloc() const {
  return 1;
}

int loadUBNode::reloc() const {
  return 1;
}

int loadUB2LNode::reloc() const {
  return 1;
}

int loadSNode::reloc() const {
  return 1;
}

int loadS2LNode::reloc() const {
  return 1;
}

int loadUSNode::reloc() const {
  return 1;
}

int loadUS2LNode::reloc() const {
  return 1;
}

int loadINode::reloc() const {
  return 1;
}

int loadI2LNode::reloc() const {
  return 1;
}

int loadUI2LNode::reloc() const {
  return 1;
}

int loadLNode::reloc() const {
  return 1;
}

int loadRangeNode::reloc() const {
  return 1;
}

int loadPNode::reloc() const {
  return 1;
}

int loadNNode::reloc() const {
  return 1;
}

int loadKlassNode::reloc() const {
  return 1;
}

int loadNKlassNode::reloc() const {
  return 1;
}

int loadFNode::reloc() const {
  return 1;
}

int loadDNode::reloc() const {
  return 1;
}

int loadConPNode::reloc() const {
  return 1;
}

int loadConP0Node::reloc() const {
  return 1;
}

int loadConP1Node::reloc() const {
  return 1;
}

int loadByteMapBaseNode::reloc() const {
  return 1;
}

int loadConFNode::reloc() const {
  return 2;
}

int loadConF0Node::reloc() const {
  return 2;
}

int loadConDNode::reloc() const {
  return 2;
}

int loadConD0Node::reloc() const {
  return 2;
}

int storeimmCM0Node::reloc() const {
  return 2;
}

int storeimmCM0_orderedNode::reloc() const {
  return 2;
}

int storeBNode::reloc() const {
  return 3;
}

int storeimmB0Node::reloc() const {
  return 2;
}

int storeCNode::reloc() const {
  return 3;
}

int storeimmC0Node::reloc() const {
  return 2;
}

int storeINode::reloc() const {
  return 3;
}

int storeimmI0Node::reloc() const {
  return 2;
}

int storeLNode::reloc() const {
  return 2;
}

int storeimmL0Node::reloc() const {
  return 2;
}

int storePNode::reloc() const {
  return 2;
}

int storeimmP0Node::reloc() const {
  return 3;
}

int storeNNode::reloc() const {
  return 2;
}

int storeImmN0Node::reloc() const {
  return 2;
}

int storeFNode::reloc() const {
  return 2;
}

int storeDNode::reloc() const {
  return 2;
}

int storeNKlassNode::reloc() const {
  return 2;
}

int storePConditionalNode::reloc() const {
  return 1;
}

int get_and_addINode::reloc() const {
  return 1;
}

int get_and_addI_no_resNode::reloc() const {
  return 1;
}

int get_and_addIAcqNode::reloc() const {
  return 1;
}

int get_and_addI_no_resAcqNode::reloc() const {
  return 1;
}

int addI_reg_regNode::reloc() const {
  return 2;
}

int addI_reg_immNode::reloc() const {
  return 1;
}

int subI_reg_regNode::reloc() const {
  return 2;
}

int subI_reg_immNode::reloc() const {
  return 1;
}

int negI_regNode::reloc() const {
  return 1;
}

int mulINode::reloc() const {
  return 2;
}

int divINode::reloc() const {
  return 2;
}

int signExtractNode::reloc() const {
  return 1;
}

int modINode::reloc() const {
  return 2;
}

int lShiftI_reg_regNode::reloc() const {
  return 2;
}

int lShiftI_reg_immNode::reloc() const {
  return 1;
}

int urShiftI_reg_regNode::reloc() const {
  return 2;
}

int urShiftI_reg_immNode::reloc() const {
  return 1;
}

int rShiftI_reg_regNode::reloc() const {
  return 2;
}

int rShiftI_reg_immNode::reloc() const {
  return 1;
}

int lShiftL_reg_regNode::reloc() const {
  return 1;
}

int urShiftL_reg_regNode::reloc() const {
  return 1;
}

int rShiftL_reg_regNode::reloc() const {
  return 1;
}

int absI_regNode::reloc() const {
  return 1;
}

int bytes_reverse_intNode::reloc() const {
  return 1;
}

int bytes_reverse_unsigned_shortNode::reloc() const {
  return 1;
}

int bytes_reverse_shortNode::reloc() const {
  return 1;
}

int convI2L_reg_regNode::reloc() const {
  return 1;
}

int convI2UL_reg_regNode::reloc() const {
  return 1;
}

int convI2F_reg_regNode::reloc() const {
  return 1;
}

int convI2D_reg_regNode::reloc() const {
  return 1;
}

int cmpLTMask_reg_zeroNode::reloc() const {
  return 1;
}

int cmpP_imm0_branchNode::reloc() const {
  return 1;
}

int cmpP_imm0_loopNode::reloc() const {
  return 1;
}

int cmpP_narrowOop_imm0_branchNode::reloc() const {
  return 1;
}

int cmpP_narrowOop_imm0_loopNode::reloc() const {
  return 1;
}

int far_cmpP_imm0_branchNode::reloc() const {
  return 1;
}

int far_cmpP_imm0_loopNode::reloc() const {
  return 1;
}

int far_cmpP_narrowOop_imm0_branchNode::reloc() const {
  return 1;
}

int far_cmpP_narrowOop_imm0_loopNode::reloc() const {
  return 1;
}

int CallStaticJavaDirectNode::reloc() const {
  return 1;
}

int CallDynamicJavaDirectNode::reloc() const {
  return 1;
}

int CallRuntimeDirectNode::reloc() const {
  return 1;
}

int CallLeafDirectNode::reloc() const {
  return 1;
}

int CallLeafNoFPDirectNode::reloc() const {
  return 1;
}

int partialSubtypeCheckVsZeroNode::reloc() const {
  return 1;
}

int safePointNode::reloc() const {
  return 1;
}

int TailCalljmpIndNode::reloc() const {
  return 1;
}

int TailjmpIndNode::reloc() const {
  return 1;
}

int RethrowExceptionNode::reloc() const {
  return 1;
}

int RetNode::reloc() const {
  return 1;
}

int loadVNode::reloc() const {
  return 1;
}

int storeVNode::reloc() const {
  return 2;
}

int reduce_addBNode::reloc() const {
  return 1;
}

int reduce_addSNode::reloc() const {
  return 1;
}

int reduce_addINode::reloc() const {
  return 1;
}

int vreduce_maxINode::reloc() const {
  return 1;
}

int vreduce_minINode::reloc() const {
  return 1;
}

int replicateBNode::reloc() const {
  return 1;
}

int replicateSNode::reloc() const {
  return 1;
}

int replicateINode::reloc() const {
  return 1;
}

int vshiftcntBNode::reloc() const {
  return 1;
}

int vshiftcntB_0Node::reloc() const {
  return 1;
}

int vshiftcntSNode::reloc() const {
  return 1;
}

int vshiftcntS_0Node::reloc() const {
  return 1;
}

int vshiftcntINode::reloc() const {
  return 1;
}

int vshiftcntI_0Node::reloc() const {
  return 1;
}

int vshiftcntLNode::reloc() const {
  return 1;
}

int vshiftcntL_0Node::reloc() const {
  return 1;
}

int convB2I_reg_reg_bNode::reloc() const {
  return 1;
}

int convI2S_reg_reg_bNode::reloc() const {
  return 1;
}

int convS2UI_reg_reg_bNode::reloc() const {
  return 1;
}

int convI2UL_reg_reg_bNode::reloc() const {
  return 1;
}

int bytes_reverse_int_bNode::reloc() const {
  return 1;
}

int bytes_reverse_unsigned_short_bNode::reloc() const {
  return 1;
}

int bytes_reverse_short_bNode::reloc() const {
  return 1;
}

int countLeadingZerosI_bNode::reloc() const {
  return 1;
}

int countTrailingZerosI_bNode::reloc() const {
  return 1;
}

int popCountI_bNode::reloc() const {
  return 1;
}

int zLoadPNode::reloc() const {
  return 1;
}


void loadBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4404 "src/hotspot/cpu/riscv/riscv.ad"

    __ lb(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 8855 "ad_riscv.cpp"
  }
}

void loadB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4419 "src/hotspot/cpu/riscv/riscv.ad"

    __ lb(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 8871 "ad_riscv.cpp"
  }
}

void loadUBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4434 "src/hotspot/cpu/riscv/riscv.ad"

    __ lbu(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 8887 "ad_riscv.cpp"
  }
}

void loadUB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4449 "src/hotspot/cpu/riscv/riscv.ad"

    __ lbu(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 8903 "ad_riscv.cpp"
  }
}

void loadSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4464 "src/hotspot/cpu/riscv/riscv.ad"

    __ lh(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 8919 "ad_riscv.cpp"
  }
}

void loadS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4479 "src/hotspot/cpu/riscv/riscv.ad"

    __ lh(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 8935 "ad_riscv.cpp"
  }
}

void loadUSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4494 "src/hotspot/cpu/riscv/riscv.ad"

    __ lhu(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 8951 "ad_riscv.cpp"
  }
}

void loadUS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4509 "src/hotspot/cpu/riscv/riscv.ad"

    __ lhu(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 8967 "ad_riscv.cpp"
  }
}

void loadINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4524 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ lw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 8984 "ad_riscv.cpp"
  }
}

void loadI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4540 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ lw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9001 "ad_riscv.cpp"
  }
}

void loadUI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4556 "src/hotspot/cpu/riscv/riscv.ad"

    __ lwu(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9018 "ad_riscv.cpp"
  }
}

void loadLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4571 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ ld(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9035 "ad_riscv.cpp"
  }
}

void loadRangeNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4587 "src/hotspot/cpu/riscv/riscv.ad"

    __ lwu(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9051 "ad_riscv.cpp"
  }
}

void loadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4603 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ ld(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9068 "ad_riscv.cpp"
  }
}

void loadNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4619 "src/hotspot/cpu/riscv/riscv.ad"

    __ lwu(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9084 "ad_riscv.cpp"
  }
}

void loadKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4634 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ ld(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9101 "ad_riscv.cpp"
  }
}

void loadNKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4650 "src/hotspot/cpu/riscv/riscv.ad"

    __ lwu(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9117 "ad_riscv.cpp"
  }
}

void loadFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4665 "src/hotspot/cpu/riscv/riscv.ad"

    __ flw(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9133 "ad_riscv.cpp"
  }
}

void loadDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4680 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ fld(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9150 "ad_riscv.cpp"
  }
}

void loadConINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2075 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Assembler::CompressibleRegion cr(&_masm);
    int64_t con = (int64_t)opnd_array(1)->constant();
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ li(dst_reg, con);
  
#line 9169 "ad_riscv.cpp"
  }
}

void loadConLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2075 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Assembler::CompressibleRegion cr(&_masm);
    int64_t con = (int64_t)opnd_array(1)->constantL();
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ li(dst_reg, con);
  
#line 9188 "ad_riscv.cpp"
  }
}

void loadConPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 2083 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    address con = (address)opnd_array(1)->constant();
    if (con == NULL || con == (address)1) {
      ShouldNotReachHere();
    } else {
      relocInfo::relocType rtype = opnd_array(1)->constant_reloc();
      if (rtype == relocInfo::oop_type) {
        __ movoop(dst_reg, (jobject)con, /*immediate*/true);
      } else if (rtype == relocInfo::metadata_type) {
        __ mov_metadata(dst_reg, (Metadata*)con);
      } else {
        assert(rtype == relocInfo::none, "unexpected reloc type");
        __ li(dst_reg, opnd_array(1)->constant());
      }
    }
  
#line 9218 "ad_riscv.cpp"
  }
}

void loadConP0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 2127 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ mv(dst_reg, zr);
  
#line 9235 "ad_riscv.cpp"
  }
}

void loadConP1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 2102 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Assembler::CompressibleRegion cr(&_masm);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ li(dst_reg, 1);
  
#line 9253 "ad_riscv.cpp"
  }
}

void loadByteMapBaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 2109 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    __ load_byte_map_base(opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 9269 "ad_riscv.cpp"
  }
}

void loadConNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 2114 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    address con = (address)opnd_array(1)->constant();
    if (con == NULL) {
      ShouldNotReachHere();
    } else {
      relocInfo::relocType rtype = opnd_array(1)->constant_reloc();
      assert(rtype == relocInfo::oop_type, "unexpected reloc type");
      __ set_narrow_oop(dst_reg, (jobject)con);
    }
  
#line 9293 "ad_riscv.cpp"
  }
}

void loadConN0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 2127 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ mv(dst_reg, zr);
  
#line 9310 "ad_riscv.cpp"
  }
}

void loadConNKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 2133 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    address con = (address)opnd_array(1)->constant();
    if (con == NULL) {
      ShouldNotReachHere();
    } else {
      relocInfo::relocType rtype = opnd_array(1)->constant_reloc();
      assert(rtype == relocInfo::metadata_type, "unexpected reloc type");
      __ set_narrow_klass(dst_reg, (Klass *)con);
    }
  
#line 9334 "ad_riscv.cpp"
  }
}

void loadConFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4813 "src/hotspot/cpu/riscv/riscv.ad"

    __ flw(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 9350 "ad_riscv.cpp"
  }
}

void loadConFNode::eval_constant(Compile* C) {
  {

#line 4814 "src/hotspot/cpu/riscv/riscv.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 9359 "ad_riscv.cpp"
  }
}
void loadConF0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4827 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmv_w_x(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), zr);
  
#line 9374 "ad_riscv.cpp"
  }
}

void loadConDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4843 "src/hotspot/cpu/riscv/riscv.ad"

    __ fld(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 9390 "ad_riscv.cpp"
  }
}

void loadConDNode::eval_constant(Compile* C) {
  {

#line 4844 "src/hotspot/cpu/riscv/riscv.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 9399 "ad_riscv.cpp"
  }
}
void loadConD0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4857 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmv_d_x(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), zr);
  
#line 9414 "ad_riscv.cpp"
  }
}

void storeimmCM0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4874 "src/hotspot/cpu/riscv/riscv.ad"

    __ sb(zr, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9431 "ad_riscv.cpp"
  }
}

void storeimmCM0_orderedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4891 "src/hotspot/cpu/riscv/riscv.ad"

    __ membar(MacroAssembler::LoadStore | MacroAssembler::StoreStore);
    __ sb(zr, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9449 "ad_riscv.cpp"
  }
}

void storeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4907 "src/hotspot/cpu/riscv/riscv.ad"

    __ sb(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9466 "ad_riscv.cpp"
  }
}

void storeimmB0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4921 "src/hotspot/cpu/riscv/riscv.ad"

    __ sb(zr, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9483 "ad_riscv.cpp"
  }
}

void storeCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4936 "src/hotspot/cpu/riscv/riscv.ad"

    __ sh(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9500 "ad_riscv.cpp"
  }
}

void storeimmC0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4950 "src/hotspot/cpu/riscv/riscv.ad"

    __ sh(zr, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9517 "ad_riscv.cpp"
  }
}

void storeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4965 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ sw(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9535 "ad_riscv.cpp"
  }
}

void storeimmI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4980 "src/hotspot/cpu/riscv/riscv.ad"

    __ sw(zr, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9552 "ad_riscv.cpp"
  }
}

void storeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4995 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ sd(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9570 "ad_riscv.cpp"
  }
}

void storeimmL0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5011 "src/hotspot/cpu/riscv/riscv.ad"

    __ sd(zr, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9587 "ad_riscv.cpp"
  }
}

void storePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5026 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ sd(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9605 "ad_riscv.cpp"
  }
}

void storeimmP0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5042 "src/hotspot/cpu/riscv/riscv.ad"

    __ sd(zr, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9622 "ad_riscv.cpp"
  }
}

void storeNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5057 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ sw(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9640 "ad_riscv.cpp"
  }
}

void storeImmN0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5072 "src/hotspot/cpu/riscv/riscv.ad"

    __ sw(as_Register(R27_enc), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9657 "ad_riscv.cpp"
  }
}

void storeFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5087 "src/hotspot/cpu/riscv/riscv.ad"

    __ fsw(as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9674 "ad_riscv.cpp"
  }
}

void storeDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5102 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ fsd(as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9692 "ad_riscv.cpp"
  }
}

void storeNKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5118 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ sw(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 9710 "ad_riscv.cpp"
  }
}

void loadPLockedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5161 "src/hotspot/cpu/riscv/riscv.ad"

    __ la(t0, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
    __ lr_d(opnd_array(0)->as_Register(ra_,this)/* dst */, t0, Assembler::aq);
  
#line 9727 "ad_riscv.cpp"
  }
}

void storePConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// heap_top_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5182 "src/hotspot/cpu/riscv/riscv.ad"

    __ la(t0, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
    __ sc_d(opnd_array(0)->as_Register(ra_,this)/* cr */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, t0, Assembler::rl);
  
#line 9746 "ad_riscv.cpp"
  }
}

void storeLConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5201 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* cr */);
    __ xorr(opnd_array(0)->as_Register(ra_,this)/* cr */,opnd_array(0)->as_Register(ra_,this)/* cr */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */);
  
#line 9766 "ad_riscv.cpp"
  }
}

void storeIConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5223 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int32,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* cr */);
    __ xorr(opnd_array(0)->as_Register(ra_,this)/* cr */,opnd_array(0)->as_Register(ra_,this)/* cr */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */);
  
#line 9786 "ad_riscv.cpp"
  }
}

void compareAndSwapBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5248 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int8,
                            Assembler::relaxed /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                            true /* result as bool */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
  
#line 9810 "ad_riscv.cpp"
  }
}

void compareAndSwapSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5271 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int16,
                            Assembler::relaxed /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                            true /* result as bool */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
  
#line 9834 "ad_riscv.cpp"
  }
}

void compareAndSwapINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2146 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int32,
               /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */,
               /*result as bool*/ true);
  
#line 9854 "ad_riscv.cpp"
  }
}

void compareAndSwapLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2160 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */,
               /*result as bool*/ true);
  
#line 9874 "ad_riscv.cpp"
  }
}

void compareAndSwapPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2160 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */,
               /*result as bool*/ true);
  
#line 9894 "ad_riscv.cpp"
  }
}

void compareAndSwapNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2153 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::uint32,
               /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */,
               /*result as bool*/ true);
  
#line 9914 "ad_riscv.cpp"
  }
}

void compareAndSwapBAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5363 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int8,
                            Assembler::aq /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                            true /* result as bool */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
  
#line 9938 "ad_riscv.cpp"
  }
}

void compareAndSwapSAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5388 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int16,
                            Assembler::aq /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                            true /* result as bool */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
  
#line 9962 "ad_riscv.cpp"
  }
}

void compareAndSwapIAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2167 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int32,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */,
               /*result as bool*/ true);
  
#line 9982 "ad_riscv.cpp"
  }
}

void compareAndSwapLAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2181 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */,
               /*result as bool*/ true);
  
#line 10002 "ad_riscv.cpp"
  }
}

void compareAndSwapPAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2181 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */,
               /*result as bool*/ true);
  
#line 10022 "ad_riscv.cpp"
  }
}

void compareAndSwapNAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2174 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::uint32,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */,
               /*result as bool*/ true);
  
#line 10042 "ad_riscv.cpp"
  }
}

void compareAndExchangeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5488 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int8,
                            /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                            /*result_as_bool*/ false, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
  
#line 10066 "ad_riscv.cpp"
  }
}

void compareAndExchangeSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5510 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int16,
                            /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                            /*result_as_bool*/ false, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
  
#line 10090 "ad_riscv.cpp"
  }
}

void compareAndExchangeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5531 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int32,
               /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 10110 "ad_riscv.cpp"
  }
}

void compareAndExchangeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5551 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 10130 "ad_riscv.cpp"
  }
}

void compareAndExchangeNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5571 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::uint32,
               /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 10150 "ad_riscv.cpp"
  }
}

void compareAndExchangePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5592 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 10170 "ad_riscv.cpp"
  }
}

void compareAndExchangeBAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5615 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int8,
                            /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                            /*result_as_bool*/ false, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
  
#line 10194 "ad_riscv.cpp"
  }
}

void compareAndExchangeSAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5639 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int16,
                            /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                            /*result_as_bool*/ false, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
  
#line 10218 "ad_riscv.cpp"
  }
}

void compareAndExchangeIAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5662 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int32,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 10238 "ad_riscv.cpp"
  }
}

void compareAndExchangeLAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5684 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 10258 "ad_riscv.cpp"
  }
}

void compareAndExchangeNAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5706 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::uint32,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 10278 "ad_riscv.cpp"
  }
}

void compareAndExchangePAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5728 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 10298 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5750 "src/hotspot/cpu/riscv/riscv.ad"

    __ weak_cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int8,
                                 /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                                 opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
    __ xori(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */, 1);
  
#line 10323 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5774 "src/hotspot/cpu/riscv/riscv.ad"

    __ weak_cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int16,
                                 /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                                 opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
    __ xori(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */, 1);
  
#line 10348 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5795 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_weak(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int32,
                    /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */);
    __ xori(opnd_array(0)->as_Register(ra_,this)/* res */, opnd_array(0)->as_Register(ra_,this)/* res */, 1);
  
#line 10368 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5815 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_weak(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
                    /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */);
    __ xori(opnd_array(0)->as_Register(ra_,this)/* res */, opnd_array(0)->as_Register(ra_,this)/* res */, 1);
  
#line 10388 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5835 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_weak(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::uint32,
                    /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */);
    __ xori(opnd_array(0)->as_Register(ra_,this)/* res */, opnd_array(0)->as_Register(ra_,this)/* res */, 1);
  
#line 10408 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5856 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_weak(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
                    /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */);
    __ xori(opnd_array(0)->as_Register(ra_,this)/* res */, opnd_array(0)->as_Register(ra_,this)/* res */, 1);
  
#line 10428 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapBAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5881 "src/hotspot/cpu/riscv/riscv.ad"

    __ weak_cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int8,
                                 /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                                 opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
    __ xori(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */, 1);
  
#line 10453 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapSAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5907 "src/hotspot/cpu/riscv/riscv.ad"

    __ weak_cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int16,
                                 /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                                 opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
    __ xori(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */, 1);
  
#line 10478 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapIAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5930 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_weak(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int32,
                    /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */);
    __ xori(opnd_array(0)->as_Register(ra_,this)/* res */, opnd_array(0)->as_Register(ra_,this)/* res */, 1);
  
#line 10498 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapLAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5952 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_weak(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
                    /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */);
    __ xori(opnd_array(0)->as_Register(ra_,this)/* res */, opnd_array(0)->as_Register(ra_,this)/* res */, 1);
  
#line 10518 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapNAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5974 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_weak(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::uint32,
                    /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */);
    __ xori(opnd_array(0)->as_Register(ra_,this)/* res */, opnd_array(0)->as_Register(ra_,this)/* res */, 1);
  
#line 10538 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapPAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5996 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_weak(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
                    /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */);
    __ xori(opnd_array(0)->as_Register(ra_,this)/* res */, opnd_array(0)->as_Register(ra_,this)/* res */, 1);
  
#line 10558 "ad_riscv.cpp"
  }
}

void get_and_setINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6013 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_xchgw(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10575 "ad_riscv.cpp"
  }
}

void get_and_setLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6028 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_xchg(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10592 "ad_riscv.cpp"
  }
}

void get_and_setNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6043 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_xchgwu(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10609 "ad_riscv.cpp"
  }
}

void get_and_setPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6059 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_xchg(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10626 "ad_riscv.cpp"
  }
}

void get_and_setIAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6076 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_xchgalw(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10643 "ad_riscv.cpp"
  }
}

void get_and_setLAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6093 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_xchgal(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10660 "ad_riscv.cpp"
  }
}

void get_and_setNAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6110 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_xchgalwu(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10677 "ad_riscv.cpp"
  }
}

void get_and_setPAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6127 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_xchgal(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10694 "ad_riscv.cpp"
  }
}

void get_and_addLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6142 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_add(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10711 "ad_riscv.cpp"
  }
}

void get_and_addL_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6159 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_add(noreg, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10728 "ad_riscv.cpp"
  }
}

void get_and_addLiNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6174 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_add(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->constantL(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10745 "ad_riscv.cpp"
  }
}

void get_and_addLi_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6191 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_add(noreg, opnd_array(2)->constantL(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10762 "ad_riscv.cpp"
  }
}

void get_and_addINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6206 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addw(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10779 "ad_riscv.cpp"
  }
}

void get_and_addI_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6223 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addw(noreg, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10796 "ad_riscv.cpp"
  }
}

void get_and_addIiNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6238 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addw(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->constant(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10813 "ad_riscv.cpp"
  }
}

void get_and_addIi_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6255 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addw(noreg, opnd_array(2)->constant(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10830 "ad_riscv.cpp"
  }
}

void get_and_addLAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6272 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addal(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10847 "ad_riscv.cpp"
  }
}

void get_and_addL_no_resAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6288 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addal(noreg, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10864 "ad_riscv.cpp"
  }
}

void get_and_addLiAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6305 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addal(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->constantL(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10881 "ad_riscv.cpp"
  }
}

void get_and_addLi_no_resAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6322 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addal(noreg, opnd_array(2)->constantL(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10898 "ad_riscv.cpp"
  }
}

void get_and_addIAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6339 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addalw(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10915 "ad_riscv.cpp"
  }
}

void get_and_addI_no_resAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6356 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addalw(noreg, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10932 "ad_riscv.cpp"
  }
}

void get_and_addIiAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6373 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addalw(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->constant(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10949 "ad_riscv.cpp"
  }
}

void get_and_addIi_no_resAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6390 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addalw(noreg, opnd_array(2)->constant(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 10966 "ad_riscv.cpp"
  }
}

void addI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6414 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 10986 "ad_riscv.cpp"
  }
}

void addI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6430 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    int32_t con = (int32_t)opnd_array(2)->constant();
    __ addiw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             opnd_array(2)->constant());
  
#line 11007 "ad_riscv.cpp"
  }
}

void addI_reg_imm_l2iNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6447 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ addiw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             opnd_array(2)->constant());
  
#line 11027 "ad_riscv.cpp"
  }
}

void addP_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6464 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11047 "ad_riscv.cpp"
  }
}

void lShiftL_regI_immGE32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scale
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6480 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ slli(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), opnd_array(2)->constant()& 63);
  
#line 11065 "ad_riscv.cpp"
  }
}

void addP_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6496 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    // src2 is imm, so actually call the addi
    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           opnd_array(2)->constantL());
  
#line 11086 "ad_riscv.cpp"
  }
}

void addL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6513 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11106 "ad_riscv.cpp"
  }
}

void addL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6529 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    // src2 is imm, so actually call the addi
    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           opnd_array(2)->constantL());
  
#line 11127 "ad_riscv.cpp"
  }
}

void subI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6547 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11147 "ad_riscv.cpp"
  }
}

void subI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6564 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    // src2 is imm, so actually call the addiw
    __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            opnd_array(2)->constant());
  
#line 11168 "ad_riscv.cpp"
  }
}

void subL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6581 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11188 "ad_riscv.cpp"
  }
}

void subL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6597 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    // src2 is imm, so actually call the addi
    __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           opnd_array(2)->constantL());
  
#line 11209 "ad_riscv.cpp"
  }
}

void negI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6615 "src/hotspot/cpu/riscv/riscv.ad"

    // actually call the subw
    __ negw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 11228 "ad_riscv.cpp"
  }
}

void negL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6631 "src/hotspot/cpu/riscv/riscv.ad"

    // actually call the sub
    __ neg(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 11247 "ad_riscv.cpp"
  }
}

void mulINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6648 "src/hotspot/cpu/riscv/riscv.ad"

    // riscv64 mulw will sign-extension to high 32 bits in dst reg
    __ mulw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11267 "ad_riscv.cpp"
  }
}

void mulLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6665 "src/hotspot/cpu/riscv/riscv.ad"

    __ mul(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11286 "ad_riscv.cpp"
  }
}

void mulHiL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6680 "src/hotspot/cpu/riscv/riscv.ad"

    __ mulh(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11305 "ad_riscv.cpp"
  }
}

void divINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2483 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src1_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register src2_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ corrected_idivl(dst_reg, src1_reg, src2_reg, false);
  
#line 11325 "ad_riscv.cpp"
  }
}

void signExtractNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// div2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6705 "src/hotspot/cpu/riscv/riscv.ad"

    __ srliw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), 31);
  
#line 11343 "ad_riscv.cpp"
  }
}

void divLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2491 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src1_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register src2_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ corrected_idivq(dst_reg, src1_reg, src2_reg, false);
  
#line 11363 "ad_riscv.cpp"
  }
}

void signExtractLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// div2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6727 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ srli(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), 63);
  
#line 11382 "ad_riscv.cpp"
  }
}

void modINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2499 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src1_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register src2_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ corrected_idivl(dst_reg, src1_reg, src2_reg, true);
  
#line 11402 "ad_riscv.cpp"
  }
}

void modLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2507 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src1_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register src2_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ corrected_idivq(dst_reg, src1_reg, src2_reg, true);
  
#line 11422 "ad_riscv.cpp"
  }
}

void lShiftI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6765 "src/hotspot/cpu/riscv/riscv.ad"

    __ sllw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11441 "ad_riscv.cpp"
  }
}

void lShiftI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6780 "src/hotspot/cpu/riscv/riscv.ad"

    // the shift amount is encoded in the lower
    // 5 bits of the I-immediate field for RV32I
    __ slliw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             (unsigned) opnd_array(2)->constant()& 0x1f);
  
#line 11462 "ad_riscv.cpp"
  }
}

void urShiftI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6798 "src/hotspot/cpu/riscv/riscv.ad"

    __ srlw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11481 "ad_riscv.cpp"
  }
}

void urShiftI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6813 "src/hotspot/cpu/riscv/riscv.ad"

    // the shift amount is encoded in the lower
    // 6 bits of the I-immediate field for RV64I
    __ srliw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             (unsigned) opnd_array(2)->constant()& 0x1f);
  
#line 11502 "ad_riscv.cpp"
  }
}

void rShiftI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6831 "src/hotspot/cpu/riscv/riscv.ad"

    // riscv will sign-ext dst high 32 bits
    __ sraw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11522 "ad_riscv.cpp"
  }
}

void rShiftI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6847 "src/hotspot/cpu/riscv/riscv.ad"

    // riscv will sign-ext dst high 32 bits
    __ sraiw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             (unsigned) opnd_array(2)->constant()& 0x1f);
  
#line 11542 "ad_riscv.cpp"
  }
}

void lShiftL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6867 "src/hotspot/cpu/riscv/riscv.ad"

    __ sll(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11561 "ad_riscv.cpp"
  }
}

void lShiftL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6883 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    // the shift amount is encoded in the lower
    // 6 bits of the I-immediate field for RV64I
    __ slli(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (unsigned) opnd_array(2)->constant()& 0x3f);
  
#line 11583 "ad_riscv.cpp"
  }
}

void urShiftL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6903 "src/hotspot/cpu/riscv/riscv.ad"

    __ srl(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11602 "ad_riscv.cpp"
  }
}

void urShiftL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6919 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    // the shift amount is encoded in the lower
    // 6 bits of the I-immediate field for RV64I
    __ srli(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (unsigned) opnd_array(2)->constant()& 0x3f);
  
#line 11624 "ad_riscv.cpp"
  }
}

void urShiftP_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6938 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    // the shift amount is encoded in the lower
    // 6 bits of the I-immediate field for RV64I
    __ srli(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (unsigned) opnd_array(2)->constant()& 0x3f);
  
#line 11646 "ad_riscv.cpp"
  }
}

void rShiftL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6958 "src/hotspot/cpu/riscv/riscv.ad"

    __ sra(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11665 "ad_riscv.cpp"
  }
}

void rShiftL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6974 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    // the shift amount is encoded in the lower
    // 6 bits of the I-immediate field for RV64I
    __ srai(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (unsigned) opnd_array(2)->constant()& 0x3f);
  
#line 11687 "ad_riscv.cpp"
  }
}

void regI_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6991 "src/hotspot/cpu/riscv/riscv.ad"

    __ xori(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), -1);
  
#line 11704 "ad_riscv.cpp"
  }
}

void regL_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7003 "src/hotspot/cpu/riscv/riscv.ad"

    __ xori(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), -1);
  
#line 11721 "ad_riscv.cpp"
  }
}

void addF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7020 "src/hotspot/cpu/riscv/riscv.ad"

    __ fadd_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11740 "ad_riscv.cpp"
  }
}

void addD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7035 "src/hotspot/cpu/riscv/riscv.ad"

    __ fadd_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11759 "ad_riscv.cpp"
  }
}

void subF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7050 "src/hotspot/cpu/riscv/riscv.ad"

    __ fsub_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11778 "ad_riscv.cpp"
  }
}

void subD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7065 "src/hotspot/cpu/riscv/riscv.ad"

    __ fsub_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11797 "ad_riscv.cpp"
  }
}

void mulF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7080 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmul_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11816 "ad_riscv.cpp"
  }
}

void mulD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7095 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmul_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 11835 "ad_riscv.cpp"
  }
}

void maddF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7112 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmadd_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
               as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 11856 "ad_riscv.cpp"
  }
}

void maddD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7130 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmadd_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
               as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 11877 "ad_riscv.cpp"
  }
}

void msubF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7148 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmsub_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
               as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 11898 "ad_riscv.cpp"
  }
}

void msubD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7166 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmsub_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
               as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 11919 "ad_riscv.cpp"
  }
}

void nmsubF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7185 "src/hotspot/cpu/riscv/riscv.ad"

    __ fnmsub_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
                as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
                as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 11940 "ad_riscv.cpp"
  }
}

void nmsubF_reg_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7185 "src/hotspot/cpu/riscv/riscv.ad"

    __ fnmsub_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
                as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
                as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 11961 "ad_riscv.cpp"
  }
}

void nmsubD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7204 "src/hotspot/cpu/riscv/riscv.ad"

    __ fnmsub_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
                as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
                as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 11982 "ad_riscv.cpp"
  }
}

void nmsubD_reg_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7204 "src/hotspot/cpu/riscv/riscv.ad"

    __ fnmsub_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
                as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
                as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 12003 "ad_riscv.cpp"
  }
}

void nmaddF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7223 "src/hotspot/cpu/riscv/riscv.ad"

    __ fnmadd_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
                as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
                as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 12024 "ad_riscv.cpp"
  }
}

void nmaddF_reg_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7223 "src/hotspot/cpu/riscv/riscv.ad"

    __ fnmadd_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
                as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
                as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 12045 "ad_riscv.cpp"
  }
}

void nmaddD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7242 "src/hotspot/cpu/riscv/riscv.ad"

    __ fnmadd_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
                as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
                as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 12066 "ad_riscv.cpp"
  }
}

void nmaddD_reg_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7242 "src/hotspot/cpu/riscv/riscv.ad"

    __ fnmadd_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
                as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
                as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 12087 "ad_riscv.cpp"
  }
}

void maxF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7259 "src/hotspot/cpu/riscv/riscv.ad"

    __ minmax_FD(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
                 as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                 false /* is_double */, false /* is_min */);
  
#line 12107 "ad_riscv.cpp"
  }
}

void minF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7275 "src/hotspot/cpu/riscv/riscv.ad"

    __ minmax_FD(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
                 as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                 false /* is_double */, true /* is_min */);
  
#line 12127 "ad_riscv.cpp"
  }
}

void maxD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7291 "src/hotspot/cpu/riscv/riscv.ad"

    __ minmax_FD(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
                 as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                 true /* is_double */, false /* is_min */);
  
#line 12147 "ad_riscv.cpp"
  }
}

void minD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7307 "src/hotspot/cpu/riscv/riscv.ad"

    __ minmax_FD(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
                 as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                 true /* is_double */, true /* is_min */);
  
#line 12167 "ad_riscv.cpp"
  }
}

void divF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7322 "src/hotspot/cpu/riscv/riscv.ad"

    __ fdiv_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 12186 "ad_riscv.cpp"
  }
}

void divD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7337 "src/hotspot/cpu/riscv/riscv.ad"

    __ fdiv_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 12205 "ad_riscv.cpp"
  }
}

void negF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7352 "src/hotspot/cpu/riscv/riscv.ad"

    __ fneg_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12222 "ad_riscv.cpp"
  }
}

void negD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7366 "src/hotspot/cpu/riscv/riscv.ad"

    __ fneg_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12239 "ad_riscv.cpp"
  }
}

void absI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7384 "src/hotspot/cpu/riscv/riscv.ad"

    __ sraiw(t0, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), 0x1f);
    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), t0);
    __ xorr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(0)->reg(ra_,this)/* dst */), t0);
  
#line 12257 "ad_riscv.cpp"
  }
}

void absL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7403 "src/hotspot/cpu/riscv/riscv.ad"

    __ srai(t0, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), 0x3f);
    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), t0);
    __ xorr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(0)->reg(ra_,this)/* dst */), t0);
  
#line 12275 "ad_riscv.cpp"
  }
}

void absF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7417 "src/hotspot/cpu/riscv/riscv.ad"

    __ fabs_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12292 "ad_riscv.cpp"
  }
}

void absD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7430 "src/hotspot/cpu/riscv/riscv.ad"

    __ fabs_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12309 "ad_riscv.cpp"
  }
}

void sqrtF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7443 "src/hotspot/cpu/riscv/riscv.ad"

    __ fsqrt_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12326 "ad_riscv.cpp"
  }
}

void sqrtD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7456 "src/hotspot/cpu/riscv/riscv.ad"

    __ fsqrt_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12343 "ad_riscv.cpp"
  }
}

void andI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7476 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 12363 "ad_riscv.cpp"
  }
}

void andI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7493 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ andi(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (int32_t)(opnd_array(2)->constant()));
  
#line 12383 "ad_riscv.cpp"
  }
}

void orI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7510 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 12403 "ad_riscv.cpp"
  }
}

void orI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7527 "src/hotspot/cpu/riscv/riscv.ad"

    __ ori(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           (int32_t)(opnd_array(2)->constant()));
  
#line 12422 "ad_riscv.cpp"
  }
}

void xorI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7543 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ xorr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 12442 "ad_riscv.cpp"
  }
}

void xorI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7560 "src/hotspot/cpu/riscv/riscv.ad"

    __ xori(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (int32_t)(opnd_array(2)->constant()));
  
#line 12461 "ad_riscv.cpp"
  }
}

void andL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7576 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 12481 "ad_riscv.cpp"
  }
}

void andL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7593 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ andi(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (int32_t)(opnd_array(2)->constantL()));
  
#line 12501 "ad_riscv.cpp"
  }
}

void orL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7610 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 12521 "ad_riscv.cpp"
  }
}

void orL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7627 "src/hotspot/cpu/riscv/riscv.ad"

    __ ori(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           (int32_t)(opnd_array(2)->constantL()));
  
#line 12540 "ad_riscv.cpp"
  }
}

void xorL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7643 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ xorr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 12560 "ad_riscv.cpp"
  }
}

void xorL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7660 "src/hotspot/cpu/riscv/riscv.ad"

    __ xori(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (int32_t)(opnd_array(2)->constantL()));
  
#line 12579 "ad_riscv.cpp"
  }
}

void bytes_reverse_intNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7679 "src/hotspot/cpu/riscv/riscv.ad"

    __ revb_w_w(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12596 "ad_riscv.cpp"
  }
}

void bytes_reverse_longNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7693 "src/hotspot/cpu/riscv/riscv.ad"

    __ revb(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12613 "ad_riscv.cpp"
  }
}

void bytes_reverse_unsigned_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7706 "src/hotspot/cpu/riscv/riscv.ad"

    __ revb_h_h_u(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12629 "ad_riscv.cpp"
  }
}

void bytes_reverse_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7719 "src/hotspot/cpu/riscv/riscv.ad"

    __ revb_h_h(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12645 "ad_riscv.cpp"
  }
}

void load_fenceNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7735 "src/hotspot/cpu/riscv/riscv.ad"

    __ membar(MacroAssembler::LoadLoad | MacroAssembler::LoadStore);
  
#line 12661 "ad_riscv.cpp"
  }
}

void membar_acquireNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7748 "src/hotspot/cpu/riscv/riscv.ad"

    __ block_comment("membar_acquire");
    __ membar(MacroAssembler::LoadLoad | MacroAssembler::LoadStore);
  
#line 12678 "ad_riscv.cpp"
  }
}

void membar_acquire_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7762 "src/hotspot/cpu/riscv/riscv.ad"

    __ block_comment("membar_acquire_lock (elided)");
  
#line 12694 "ad_riscv.cpp"
  }
}

void store_fenceNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7775 "src/hotspot/cpu/riscv/riscv.ad"

    __ membar(MacroAssembler::LoadStore | MacroAssembler::StoreStore);
  
#line 12710 "ad_riscv.cpp"
  }
}

void membar_releaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7788 "src/hotspot/cpu/riscv/riscv.ad"

    __ block_comment("membar_release");
    __ membar(MacroAssembler::LoadStore | MacroAssembler::StoreStore);
  
#line 12727 "ad_riscv.cpp"
  }
}

void membar_storestoreNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7802 "src/hotspot/cpu/riscv/riscv.ad"

    __ membar(MacroAssembler::StoreStore);
  
#line 12743 "ad_riscv.cpp"
  }
}

void membar_storestore_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7802 "src/hotspot/cpu/riscv/riscv.ad"

    __ membar(MacroAssembler::StoreStore);
  
#line 12759 "ad_riscv.cpp"
  }
}

void membar_release_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7814 "src/hotspot/cpu/riscv/riscv.ad"

    __ block_comment("membar_release_lock (elided)");
  
#line 12775 "ad_riscv.cpp"
  }
}

void membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7828 "src/hotspot/cpu/riscv/riscv.ad"

    __ block_comment("membar_volatile");
    __ membar(MacroAssembler::StoreLoad);
  
#line 12792 "ad_riscv.cpp"
  }
}

void castX2PNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7845 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    if (opnd_array(0)->reg(ra_,this)/* dst */!= opnd_array(1)->reg(ra_,this,idx1)/* src */) {
      __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    }
  
#line 12811 "ad_riscv.cpp"
  }
}

void castP2XNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7861 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    if (opnd_array(0)->reg(ra_,this)/* dst */!= opnd_array(1)->reg(ra_,this,idx1)/* src */) {
      __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    }
  
#line 12830 "ad_riscv.cpp"
  }
}

void castPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castPPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castLLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castLLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castIINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castIINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void checkCastPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint checkCastPPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castFFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castFFNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castDDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castDDNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castVVNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castVVNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void convI2BoolNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7959 "src/hotspot/cpu/riscv/riscv.ad"

    __ snez(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12937 "ad_riscv.cpp"
  }
}

void convP2BoolNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7974 "src/hotspot/cpu/riscv/riscv.ad"

    __ snez(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12953 "ad_riscv.cpp"
  }
}

void convI2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7989 "src/hotspot/cpu/riscv/riscv.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), zr);
  
#line 12969 "ad_riscv.cpp"
  }
}

void convL2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8001 "src/hotspot/cpu/riscv/riscv.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), zr);
  
#line 12985 "ad_riscv.cpp"
  }
}

void convI2UL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8016 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ zero_extend(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), 32);
  
#line 13003 "ad_riscv.cpp"
  }
}

void convF2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8032 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_d_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 13019 "ad_riscv.cpp"
  }
}

void convD2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8045 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_s_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 13035 "ad_riscv.cpp"
  }
}

void convF2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8060 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_w_s_safe(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src */);
  
#line 13051 "ad_riscv.cpp"
  }
}

void convI2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8073 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_s_w(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 13067 "ad_riscv.cpp"
  }
}

void convF2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8088 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_l_s_safe(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src */);
  
#line 13083 "ad_riscv.cpp"
  }
}

void convL2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8101 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_s_l(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 13099 "ad_riscv.cpp"
  }
}

void convD2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8116 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_w_d_safe(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src */);
  
#line 13115 "ad_riscv.cpp"
  }
}

void convI2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8129 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_d_w(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 13131 "ad_riscv.cpp"
  }
}

void convD2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8144 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_l_d_safe(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src */);
  
#line 13147 "ad_riscv.cpp"
  }
}

void convL2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8157 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_d_l(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 13163 "ad_riscv.cpp"
  }
}

void convP2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8171 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ zero_extend(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, 32);
  
#line 13180 "ad_riscv.cpp"
  }
}

void convN2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8189 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ mv(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 13197 "ad_riscv.cpp"
  }
}

void encodeHeapOopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8202 "src/hotspot/cpu/riscv/riscv.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Register d = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ encode_heap_oop(d, s);
  
#line 13215 "ad_riscv.cpp"
  }
}

void decodeHeapOopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8217 "src/hotspot/cpu/riscv/riscv.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Register d = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ decode_heap_oop(d, s);
  
#line 13233 "ad_riscv.cpp"
  }
}

void decodeHeapOop_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8232 "src/hotspot/cpu/riscv/riscv.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Register d = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ decode_heap_oop_not_null(d, s);
  
#line 13251 "ad_riscv.cpp"
  }
}

void encodeKlass_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8247 "src/hotspot/cpu/riscv/riscv.ad"

    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ encode_klass_not_null(dst_reg, src_reg, t0);
  
#line 13269 "ad_riscv.cpp"
  }
}

void decodeKlass_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8264 "src/hotspot/cpu/riscv/riscv.ad"

    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register tmp_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* tmp */);
    __ decode_klass_not_null(dst_reg, src_reg, tmp_reg);
  
#line 13289 "ad_riscv.cpp"
  }
}

void MoveF2I_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8286 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ lw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(sp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 13306 "ad_riscv.cpp"
  }
}

void MoveI2F_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8305 "src/hotspot/cpu/riscv/riscv.ad"

    __ flw(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), Address(sp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 13322 "ad_riscv.cpp"
  }
}

void MoveD2L_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8323 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ ld(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(sp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 13339 "ad_riscv.cpp"
  }
}

void MoveL2D_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8342 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ fld(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), Address(sp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 13356 "ad_riscv.cpp"
  }
}

void MoveF2I_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8361 "src/hotspot/cpu/riscv/riscv.ad"

    __ fsw(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), Address(sp, opnd_array(0)->disp(ra_,this,0)));
  
#line 13372 "ad_riscv.cpp"
  }
}

void MoveI2F_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8379 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ sw(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), Address(sp, opnd_array(0)->disp(ra_,this,0)));
  
#line 13389 "ad_riscv.cpp"
  }
}

void MoveD2L_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8398 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ fsd(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), Address(sp, opnd_array(0)->disp(ra_,this,0)));
  
#line 13406 "ad_riscv.cpp"
  }
}

void MoveL2D_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8417 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ sd(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), Address(sp, opnd_array(0)->disp(ra_,this,0)));
  
#line 13423 "ad_riscv.cpp"
  }
}

void MoveF2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8436 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmv_x_w(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 13439 "ad_riscv.cpp"
  }
}

void MoveI2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8454 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmv_w_x(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 13455 "ad_riscv.cpp"
  }
}

void MoveD2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8472 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmv_x_d(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 13471 "ad_riscv.cpp"
  }
}

void MoveL2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8490 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmv_d_x(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 13487 "ad_riscv.cpp"
  }
}

void cmpF3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8511 "src/hotspot/cpu/riscv/riscv.ad"

    // we want -1 for unordered or less than, 0 for equal and 1 for greater than.
    __ float_compare(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* op1 */),
                     as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op2 */), -1 /*unordered_result < 0*/);
  
#line 13506 "ad_riscv.cpp"
  }
}

void cmpD3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8531 "src/hotspot/cpu/riscv/riscv.ad"

    // we want -1 for unordered or less than, 0 for equal and 1 for greater than.
    __ double_compare(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* op1 */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op2 */), -1 /*unordered_result < 0*/);
  
#line 13524 "ad_riscv.cpp"
  }
}

void cmpL3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8549 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_l2i(t0, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op2 */));
    __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), t0);
  
#line 13542 "ad_riscv.cpp"
  }
}

void cmpLTMask_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8567 "src/hotspot/cpu/riscv/riscv.ad"

    __ slt(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* p */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* q */));
    __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), zr, as_Register(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 13560 "ad_riscv.cpp"
  }
}

void cmpLTMask_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8583 "src/hotspot/cpu/riscv/riscv.ad"

    __ sraiw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op */), 31);
  
#line 13577 "ad_riscv.cpp"
  }
}

void minI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8610 "src/hotspot/cpu/riscv/riscv.ad"

    Label Lsrc1, Ldone;
    __ ble(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), Lsrc1);
    __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
    __ j(Ldone);
    __ bind(Lsrc1);
    __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */));
    __ bind(Ldone);
  
#line 13600 "ad_riscv.cpp"
  }
}

void maxI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8639 "src/hotspot/cpu/riscv/riscv.ad"

    Label Lsrc1, Ldone;
    __ bge(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), Lsrc1);
    __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
    __ j(Ldone);
    __ bind(Lsrc1);
    __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */));
    __ bind(Ldone);

  
#line 13624 "ad_riscv.cpp"
  }
}

void branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 2190 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Label* L = opnd_array(1)->label();
    __ j(*L);
  
#line 13641 "ad_riscv.cpp"
  }
}

void cmpFlag_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lbl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8683 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* cr */), *(opnd_array(3)->label()));
  
#line 13659 "ad_riscv.cpp"
  }
}

void cmpI_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8702 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 13678 "ad_riscv.cpp"
  }
}

void cmpI_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8721 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 13697 "ad_riscv.cpp"
  }
}

void cmpU_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8741 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 13717 "ad_riscv.cpp"
  }
}

void cmpU_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8761 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 13737 "ad_riscv.cpp"
  }
}

void cmpL_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8782 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 13756 "ad_riscv.cpp"
  }
}

void cmpL_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8801 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 13775 "ad_riscv.cpp"
  }
}

void cmpUL_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8820 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 13795 "ad_riscv.cpp"
  }
}

void cmpUL_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8839 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 13815 "ad_riscv.cpp"
  }
}

void cmpP_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8860 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 13835 "ad_riscv.cpp"
  }
}

void cmpP_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8880 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 13855 "ad_riscv.cpp"
  }
}

void cmpN_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8901 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 13875 "ad_riscv.cpp"
  }
}

void cmpN_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8921 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 13895 "ad_riscv.cpp"
  }
}

void cmpF_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8941 "src/hotspot/cpu/riscv/riscv.ad"

    __ float_cmp_branch(opnd_array(1)->ccode(), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 13914 "ad_riscv.cpp"
  }
}

void cmpF_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8958 "src/hotspot/cpu/riscv/riscv.ad"

    __ float_cmp_branch(opnd_array(1)->ccode(), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 13933 "ad_riscv.cpp"
  }
}

void cmpD_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8976 "src/hotspot/cpu/riscv/riscv.ad"

    __ float_cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::double_branch_mask, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                        as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 13953 "ad_riscv.cpp"
  }
}

void cmpD_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8994 "src/hotspot/cpu/riscv/riscv.ad"

    __ float_cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::double_branch_mask, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                        as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 13973 "ad_riscv.cpp"
  }
}

void cmpI_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9014 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), zr, *(opnd_array(4)->label()));
  
#line 13992 "ad_riscv.cpp"
  }
}

void cmpI_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9033 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), zr, *(opnd_array(4)->label()));
  
#line 14011 "ad_riscv.cpp"
  }
}

void cmpUEqNeLeGt_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9053 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpUEqNeLeGt_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 14030 "ad_riscv.cpp"
  }
}

void cmpUEqNeLeGt_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9073 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpUEqNeLeGt_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 14049 "ad_riscv.cpp"
  }
}

void cmpL_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9093 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), zr, *(opnd_array(4)->label()));
  
#line 14068 "ad_riscv.cpp"
  }
}

void cmpL_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9112 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), zr, *(opnd_array(4)->label()));
  
#line 14087 "ad_riscv.cpp"
  }
}

void cmpULEqNeLeGt_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9132 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpUEqNeLeGt_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 14106 "ad_riscv.cpp"
  }
}

void cmpULEqNeLeGt_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9151 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpUEqNeLeGt_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 14125 "ad_riscv.cpp"
  }
}

void cmpP_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9168 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 14144 "ad_riscv.cpp"
  }
}

void cmpP_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9184 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 14163 "ad_riscv.cpp"
  }
}

void cmpN_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9202 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 14182 "ad_riscv.cpp"
  }
}

void cmpN_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9219 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 14201 "ad_riscv.cpp"
  }
}

void cmpP_narrowOop_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9236 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 14220 "ad_riscv.cpp"
  }
}

void cmpP_narrowOop_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9252 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 14239 "ad_riscv.cpp"
  }
}

void far_cmpFlag_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lbl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9269 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* cr */), *(opnd_array(3)->label()), /* is_far */ true);
  
#line 14257 "ad_riscv.cpp"
  }
}

void far_cmpI_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9287 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14276 "ad_riscv.cpp"
  }
}

void far_cmpI_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9301 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14295 "ad_riscv.cpp"
  }
}

void far_cmpU_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9315 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                       as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14315 "ad_riscv.cpp"
  }
}

void far_cmpU_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9330 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                       as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14335 "ad_riscv.cpp"
  }
}

void far_cmpL_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9345 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14354 "ad_riscv.cpp"
  }
}

void far_cmpLloopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9359 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14373 "ad_riscv.cpp"
  }
}

void far_cmpUL_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9373 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                       as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14393 "ad_riscv.cpp"
  }
}

void far_cmpUL_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9388 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                       as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14413 "ad_riscv.cpp"
  }
}

void far_cmpP_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9406 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                       as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14433 "ad_riscv.cpp"
  }
}

void far_cmpP_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9424 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                       as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14453 "ad_riscv.cpp"
  }
}

void far_cmpN_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9442 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                       as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14473 "ad_riscv.cpp"
  }
}

void far_cmpN_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9460 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14493 "ad_riscv.cpp"
  }
}

void far_cmpF_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9478 "src/hotspot/cpu/riscv/riscv.ad"

    __ float_cmp_branch(opnd_array(1)->ccode(), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                        *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14513 "ad_riscv.cpp"
  }
}

void far_cmpF_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9494 "src/hotspot/cpu/riscv/riscv.ad"

    __ float_cmp_branch(opnd_array(1)->ccode(), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                        *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14533 "ad_riscv.cpp"
  }
}

void far_cmpD_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9511 "src/hotspot/cpu/riscv/riscv.ad"

    __ float_cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::double_branch_mask, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                        as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14553 "ad_riscv.cpp"
  }
}

void far_cmpD_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9527 "src/hotspot/cpu/riscv/riscv.ad"

    __ float_cmp_branch(opnd_array(1)->ccode()| C2_MacroAssembler::double_branch_mask, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                        as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14573 "ad_riscv.cpp"
  }
}

void far_cmpI_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9545 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), zr, *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14592 "ad_riscv.cpp"
  }
}

void far_cmpI_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9562 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), zr, *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14611 "ad_riscv.cpp"
  }
}

void far_cmpUEqNeLeGt_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9579 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpUEqNeLeGt_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14630 "ad_riscv.cpp"
  }
}

void far_cmpUEqNeLeGt_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9597 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpUEqNeLeGt_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14649 "ad_riscv.cpp"
  }
}

void far_cmpULtGe_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {

#line 2196 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Label* L = opnd_array(4)->label();
    switch (opnd_array(1)->ccode()) {
      case(BoolTest::ge):
        __ j(*L);
        break;
      case(BoolTest::lt):
        break;
      default:
        Unimplemented();
    }
  
#line 14677 "ad_riscv.cpp"
  }
}

void far_cmpULtGe_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {

#line 2196 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Label* L = opnd_array(4)->label();
    switch (opnd_array(1)->ccode()) {
      case(BoolTest::ge):
        __ j(*L);
        break;
      case(BoolTest::lt):
        break;
      default:
        Unimplemented();
    }
  
#line 14705 "ad_riscv.cpp"
  }
}

void far_cmpL_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9645 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), zr, *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14724 "ad_riscv.cpp"
  }
}

void far_cmpL_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9662 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), zr, *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14743 "ad_riscv.cpp"
  }
}

void far_cmpULEqNeLeGt_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9679 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpUEqNeLeGt_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14762 "ad_riscv.cpp"
  }
}

void far_cmpULEqNeLeGt_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9696 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpUEqNeLeGt_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14781 "ad_riscv.cpp"
  }
}

void far_cmpULLtGe_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {

#line 2196 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Label* L = opnd_array(4)->label();
    switch (opnd_array(1)->ccode()) {
      case(BoolTest::ge):
        __ j(*L);
        break;
      case(BoolTest::lt):
        break;
      default:
        Unimplemented();
    }
  
#line 14809 "ad_riscv.cpp"
  }
}

void far_cmpULLtGe_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {

#line 2196 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Label* L = opnd_array(4)->label();
    switch (opnd_array(1)->ccode()) {
      case(BoolTest::ge):
        __ j(*L);
        break;
      case(BoolTest::lt):
        break;
      default:
        Unimplemented();
    }
  
#line 14837 "ad_riscv.cpp"
  }
}

void far_cmpP_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9741 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14856 "ad_riscv.cpp"
  }
}

void far_cmpP_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9755 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14875 "ad_riscv.cpp"
  }
}

void far_cmpN_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9770 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14894 "ad_riscv.cpp"
  }
}

void far_cmpN_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9785 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14913 "ad_riscv.cpp"
  }
}

void far_cmpP_narrowOop_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9799 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14932 "ad_riscv.cpp"
  }
}

void far_cmpP_narrowOop_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9813 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 14951 "ad_riscv.cpp"
  }
}

void cmovI_cmpINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9832 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmove(opnd_array(1)->ccode(),
                 as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                 as_Register(opnd_array(4)->reg(ra_,this,idx4)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src */));
  
#line 14973 "ad_riscv.cpp"
  }
}

void cmovI_cmpUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9851 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmove(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask,
                 as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                 as_Register(opnd_array(4)->reg(ra_,this,idx4)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src */));
  
#line 14995 "ad_riscv.cpp"
  }
}

void cmovI_cmpLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9870 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmove(opnd_array(1)->ccode(),
                 as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                 as_Register(opnd_array(4)->reg(ra_,this,idx4)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src */));
  
#line 15017 "ad_riscv.cpp"
  }
}

void cmovL_cmpLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9889 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmove(opnd_array(1)->ccode(),
                 as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                 as_Register(opnd_array(4)->reg(ra_,this,idx4)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src */));
  
#line 15039 "ad_riscv.cpp"
  }
}

void cmovL_cmpULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9908 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmove(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask,
                 as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                 as_Register(opnd_array(4)->reg(ra_,this,idx4)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src */));
  
#line 15061 "ad_riscv.cpp"
  }
}

void cmovI_cmpULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9926 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmove(opnd_array(1)->ccode()| C2_MacroAssembler::unsigned_branch_mask,
                 as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                 as_Register(opnd_array(4)->reg(ra_,this,idx4)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src */));
  
#line 15083 "ad_riscv.cpp"
  }
}

void CallStaticJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 2239 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);

    address addr = (address)opnd_array(1)->method();
    address call = NULL;
    assert_cond(addr != NULL);
    if (!_method) {
      // A call to a runtime wrapper, e.g. new, new_typeArray_Java, uncommon_trap.
      call = __ trampoline_call(Address(addr, relocInfo::runtime_call_type), &cbuf);
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    } else {
      int method_index = resolved_method_index(cbuf);
      RelocationHolder rspec = _optimized_virtual ? opt_virtual_call_Relocation::spec(method_index)
                                                  : static_call_Relocation::spec(method_index);
      call = __ trampoline_call(Address(addr, rspec), &cbuf);
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }

      // Emit stub for static call
      address stub = CompiledStaticCall::emit_to_interp_stub(cbuf);
      if (stub == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    }
  
#line 15126 "ad_riscv.cpp"
  }
  {

#line 2281 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      __ call_Unimplemented();
    }
  
#line 15138 "ad_riscv.cpp"
  }
}

void CallDynamicJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {

#line 2271 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    int method_index = resolved_method_index(cbuf);
    address call = __ ic_call((address)opnd_array(1)->method(), method_index);
    if (call == NULL) {
      ciEnv::current()->record_failure("CodeCache is full");
      return;
    }
  
#line 15159 "ad_riscv.cpp"
  }
  {

#line 2281 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      __ call_Unimplemented();
    }
  
#line 15171 "ad_riscv.cpp"
  }
}

void CallRuntimeDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {

#line 2289 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);

    // some calls to generated routines (arraycopy code) are scheduled
    // by C2 as runtime calls. if so we can call them using a jr (they
    // will be in a reachable segment) otherwise we have to use a jalr
    // which loads the absolute address into a register.
    address entry = (address)opnd_array(1)->method();
    CodeBlob *cb = CodeCache::find_blob(entry);
    if (cb != NULL) {
      address call = __ trampoline_call(Address(entry, relocInfo::runtime_call_type));
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    } else {
      Label retaddr;
      __ la(t1, retaddr);
      __ la(t0, RuntimeAddress(entry));
      // Leave a breadcrumb for JavaFrameAnchor::capture_last_Java_pc()
      __ addi(sp, sp, -2 * wordSize);
      __ sd(t1, Address(sp, wordSize));
      __ jalr(t0);
      __ bind(retaddr);
      __ addi(sp, sp, 2 * wordSize);
    }
  
#line 15210 "ad_riscv.cpp"
  }
}

void CallLeafDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {

#line 2289 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);

    // some calls to generated routines (arraycopy code) are scheduled
    // by C2 as runtime calls. if so we can call them using a jr (they
    // will be in a reachable segment) otherwise we have to use a jalr
    // which loads the absolute address into a register.
    address entry = (address)opnd_array(1)->method();
    CodeBlob *cb = CodeCache::find_blob(entry);
    if (cb != NULL) {
      address call = __ trampoline_call(Address(entry, relocInfo::runtime_call_type));
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    } else {
      Label retaddr;
      __ la(t1, retaddr);
      __ la(t0, RuntimeAddress(entry));
      // Leave a breadcrumb for JavaFrameAnchor::capture_last_Java_pc()
      __ addi(sp, sp, -2 * wordSize);
      __ sd(t1, Address(sp, wordSize));
      __ jalr(t0);
      __ bind(retaddr);
      __ addi(sp, sp, 2 * wordSize);
    }
  
#line 15249 "ad_riscv.cpp"
  }
}

void CallLeafNoFPDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {

#line 2289 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);

    // some calls to generated routines (arraycopy code) are scheduled
    // by C2 as runtime calls. if so we can call them using a jr (they
    // will be in a reachable segment) otherwise we have to use a jalr
    // which loads the absolute address into a register.
    address entry = (address)opnd_array(1)->method();
    CodeBlob *cb = CodeCache::find_blob(entry);
    if (cb != NULL) {
      address call = __ trampoline_call(Address(entry, relocInfo::runtime_call_type));
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    } else {
      Label retaddr;
      __ la(t1, retaddr);
      __ la(t0, RuntimeAddress(entry));
      // Leave a breadcrumb for JavaFrameAnchor::capture_last_Java_pc()
      __ addi(sp, sp, -2 * wordSize);
      __ sd(t1, Address(sp, wordSize));
      __ jalr(t0);
      __ bind(retaddr);
      __ addi(sp, sp, 2 * wordSize);
    }
  
#line 15288 "ad_riscv.cpp"
  }
}

void partialSubtypeCheckNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// sub
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// super
  {

#line 2212 "src/hotspot/cpu/riscv/riscv.ad"

    Register sub_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* sub */);
    Register super_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* super */);
    Register temp_reg = as_Register(R12_enc);
    Register result_reg = as_Register(opnd_array(0)->reg(ra_,this)/* result */);
    Register cr_reg = t1;

    Label miss;
    Label done;
    C2_MacroAssembler _masm(&cbuf);
    __ check_klass_subtype_slow_path(sub_reg, super_reg, temp_reg, result_reg,
                                     NULL, &miss);
    if ((0x1 /*primary()*/)) {
      __ mv(result_reg, zr);
    } else {
      __ mv(cr_reg, zr);
      __ j(done);
    }

    __ bind(miss);
    if (!(0x1 /*primary()*/)) {
      __ li(cr_reg, 1);
    }

    __ bind(done);
  
#line 15327 "ad_riscv.cpp"
  }
}

void partialSubtypeCheckVsZeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// sub
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// super
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 2212 "src/hotspot/cpu/riscv/riscv.ad"

    Register sub_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* sub */);
    Register super_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* super */);
    Register temp_reg = as_Register(R12_enc);
    Register result_reg = as_Register(R15_enc);
    Register cr_reg = t1;

    Label miss;
    Label done;
    C2_MacroAssembler _masm(&cbuf);
    __ check_klass_subtype_slow_path(sub_reg, super_reg, temp_reg, result_reg,
                                     NULL, &miss);
    if ((0x0 /*primary()*/)) {
      __ mv(result_reg, zr);
    } else {
      __ mv(cr_reg, zr);
      __ j(done);
    }

    __ bind(miss);
    if (!(0x0 /*primary()*/)) {
      __ li(cr_reg, 1);
    }

    __ bind(done);
  
#line 15367 "ad_riscv.cpp"
  }
}

void string_compareUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10079 "src/hotspot/cpu/riscv/riscv.ad"

    // Count is in 8-bit bytes; non-Compact chars are 16 bits.
    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      as_Register(R28_enc), as_Register(R29_enc), as_Register(R30_enc),
                      StrIntrinsicNode::UU);
  
#line 15390 "ad_riscv.cpp"
  }
}

void string_compareLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10097 "src/hotspot/cpu/riscv/riscv.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      as_Register(R28_enc), as_Register(R29_enc), as_Register(R30_enc),
                      StrIntrinsicNode::LL);
  
#line 15412 "ad_riscv.cpp"
  }
}

void string_compareULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10114 "src/hotspot/cpu/riscv/riscv.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      as_Register(R28_enc), as_Register(R29_enc), as_Register(R30_enc),
                      StrIntrinsicNode::UL);
  
#line 15434 "ad_riscv.cpp"
  }
}

void string_compareLUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10132 "src/hotspot/cpu/riscv/riscv.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      as_Register(R28_enc), as_Register(R29_enc), as_Register(R30_enc),
                      StrIntrinsicNode::LU);
  
#line 15456 "ad_riscv.cpp"
  }
}

void string_indexofUUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// result
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp1
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp2
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp3
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp4
  unsigned idx10 = idx9 + opnd_array(9)->num_edges(); 	// tmp5
  unsigned idx11 = idx10 + opnd_array(10)->num_edges(); 	// tmp6
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10151 "src/hotspot/cpu/riscv/riscv.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      opnd_array(6)->as_Register(ra_,this,idx6)/* tmp1 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp2 */,
                      opnd_array(8)->as_Register(ra_,this,idx8)/* tmp3 */, opnd_array(9)->as_Register(ra_,this,idx9)/* tmp4 */,
                      opnd_array(10)->as_Register(ra_,this,idx10)/* tmp5 */, opnd_array(11)->as_Register(ra_,this,idx11)/* tmp6 */,
                      opnd_array(5)->as_Register(ra_,this,idx5)/* result */, StrIntrinsicNode::UU);
  
#line 15487 "ad_riscv.cpp"
  }
}

void string_indexofLLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// result
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp1
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp2
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp3
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp4
  unsigned idx10 = idx9 + opnd_array(9)->num_edges(); 	// tmp5
  unsigned idx11 = idx10 + opnd_array(10)->num_edges(); 	// tmp6
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10172 "src/hotspot/cpu/riscv/riscv.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      opnd_array(6)->as_Register(ra_,this,idx6)/* tmp1 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp2 */,
                      opnd_array(8)->as_Register(ra_,this,idx8)/* tmp3 */, opnd_array(9)->as_Register(ra_,this,idx9)/* tmp4 */,
                      opnd_array(10)->as_Register(ra_,this,idx10)/* tmp5 */, opnd_array(11)->as_Register(ra_,this,idx11)/* tmp6 */,
                      opnd_array(5)->as_Register(ra_,this,idx5)/* result */, StrIntrinsicNode::LL);
  
#line 15518 "ad_riscv.cpp"
  }
}

void string_indexofULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// result
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp1
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp2
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp3
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp4
  unsigned idx10 = idx9 + opnd_array(9)->num_edges(); 	// tmp5
  unsigned idx11 = idx10 + opnd_array(10)->num_edges(); 	// tmp6
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10193 "src/hotspot/cpu/riscv/riscv.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      opnd_array(6)->as_Register(ra_,this,idx6)/* tmp1 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp2 */,
                      opnd_array(8)->as_Register(ra_,this,idx8)/* tmp3 */, opnd_array(9)->as_Register(ra_,this,idx9)/* tmp4 */,
                      opnd_array(10)->as_Register(ra_,this,idx10)/* tmp5 */, opnd_array(11)->as_Register(ra_,this,idx11)/* tmp6 */,
                      opnd_array(5)->as_Register(ra_,this,idx5)/* result */, StrIntrinsicNode::UL);
  
#line 15549 "ad_riscv.cpp"
  }
}

void string_indexof_conUUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// result
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp1
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp2
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp3
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10215 "src/hotspot/cpu/riscv/riscv.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    __ string_indexof_linearscan(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                                 opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, zr,
                                 opnd_array(6)->as_Register(ra_,this,idx6)/* tmp1 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp2 */,
                                 opnd_array(8)->as_Register(ra_,this,idx8)/* tmp3 */, opnd_array(9)->as_Register(ra_,this,idx9)/* tmp4 */,
                                 icnt2, opnd_array(5)->as_Register(ra_,this,idx5)/* result */, StrIntrinsicNode::UU);
  
#line 15578 "ad_riscv.cpp"
  }
}

void string_indexof_conLLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// result
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp1
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp2
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp3
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10236 "src/hotspot/cpu/riscv/riscv.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    __ string_indexof_linearscan(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                                 opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, zr,
                                 opnd_array(6)->as_Register(ra_,this,idx6)/* tmp1 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp2 */,
                                 opnd_array(8)->as_Register(ra_,this,idx8)/* tmp3 */, opnd_array(9)->as_Register(ra_,this,idx9)/* tmp4 */,
                                 icnt2, opnd_array(5)->as_Register(ra_,this,idx5)/* result */, StrIntrinsicNode::LL);
  
#line 15607 "ad_riscv.cpp"
  }
}

void string_indexof_conULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// result
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp1
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp2
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp3
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10257 "src/hotspot/cpu/riscv/riscv.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    __ string_indexof_linearscan(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                                 opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, zr,
                                 opnd_array(6)->as_Register(ra_,this,idx6)/* tmp1 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp2 */,
                                 opnd_array(8)->as_Register(ra_,this,idx8)/* tmp3 */, opnd_array(9)->as_Register(ra_,this,idx9)/* tmp4 */,
                                 icnt2, opnd_array(5)->as_Register(ra_,this,idx5)/* result */, StrIntrinsicNode::UL);
  
#line 15636 "ad_riscv.cpp"
  }
}

void stringU_indexof_charNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// ch
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// result
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10278 "src/hotspot/cpu/riscv/riscv.ad"

    __ string_indexof_char(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* ch */,
                           opnd_array(4)->as_Register(ra_,this,idx4)/* result */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */,
                           opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp4 */, false /* isU */);
  
#line 15661 "ad_riscv.cpp"
  }
}

void stringL_indexof_charNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// ch
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// result
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10297 "src/hotspot/cpu/riscv/riscv.ad"

    __ string_indexof_char(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* ch */,
                           opnd_array(4)->as_Register(ra_,this,idx4)/* result */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */,
                           opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp4 */, true /* isL */);
  
#line 15686 "ad_riscv.cpp"
  }
}

void clearArray_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10315 "src/hotspot/cpu/riscv/riscv.ad"

    address tpc = __ zero_words(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */);
    if (tpc == NULL) {
      ciEnv::current()->record_failure("CodeCache is full");
      return;
    }
  
#line 15707 "ad_riscv.cpp"
  }
}

void clearArray_imm_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10336 "src/hotspot/cpu/riscv/riscv.ad"

    __ zero_words(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, (uint64_t)opnd_array(1)->constantL());
  
#line 15724 "ad_riscv.cpp"
  }
}

void string_equalsLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// str2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10351 "src/hotspot/cpu/riscv/riscv.ad"

    // Count is in 8-bit bytes; non-Compact chars are 16 bits.
    __ string_equals(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* str2 */,
                     opnd_array(0)->as_Register(ra_,this)/* result */, opnd_array(3)->as_Register(ra_,this,idx3)/* cnt */, 1);
  
#line 15744 "ad_riscv.cpp"
  }
}

void string_equalsUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// str2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10367 "src/hotspot/cpu/riscv/riscv.ad"

    // Count is in 8-bit bytes; non-Compact chars are 16 bits.
    __ string_equals(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* str2 */,
                     opnd_array(0)->as_Register(ra_,this)/* result */, opnd_array(3)->as_Register(ra_,this,idx3)/* cnt */, 2);
  
#line 15764 "ad_riscv.cpp"
  }
}

void array_equalsBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10384 "src/hotspot/cpu/riscv/riscv.ad"

    __ arrays_equals(opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                     opnd_array(3)->as_Register(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp3 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp4 */,
                     opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(R28_enc), 1);
  
#line 15787 "ad_riscv.cpp"
  }
}

void array_equalsCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10401 "src/hotspot/cpu/riscv/riscv.ad"

    __ arrays_equals(opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                     opnd_array(3)->as_Register(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp3 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp4 */,
                     opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(R28_enc), 2);
  
#line 15810 "ad_riscv.cpp"
  }
}

void safePointNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10420 "src/hotspot/cpu/riscv/riscv.ad"

    __ read_polling_page(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* poll */), 0, relocInfo::poll_type);
  
#line 15826 "ad_riscv.cpp"
  }
}

void tlsLoadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint tlsLoadPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void cmpFastLockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {

#line 2318 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register flag = t1;
    Register oop = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* object */);
    Register box = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* box */);
    Register disp_hdr = as_Register(opnd_array(3)->reg(ra_,this,idx3)/* tmp1 */);
    Register tmp = as_Register(opnd_array(4)->reg(ra_,this,idx4)/* tmp2 */);
    Label cont;
    Label object_has_monitor;

    assert_different_registers(oop, box, tmp, disp_hdr, t0);

    // Load markWord from object into displaced_header.
    __ ld(disp_hdr, Address(oop, oopDesc::mark_offset_in_bytes()));

    if (DiagnoseSyncOnValueBasedClasses != 0) {
      __ load_klass(flag, oop);
      __ lwu(flag, Address(flag, Klass::access_flags_offset()));
      __ andi(flag, flag, JVM_ACC_IS_VALUE_BASED_CLASS, tmp /* tmp */);
      __ bnez(flag, cont, true /* is_far */);
    }

    // Check for existing monitor
    __ andi(t0, disp_hdr, markWord::monitor_value);
    __ bnez(t0, object_has_monitor);

    if (!UseHeavyMonitors) {
      // Set tmp to be (markWord of object | UNLOCK_VALUE).
      __ ori(tmp, disp_hdr, markWord::unlocked_value);

      // Initialize the box. (Must happen before we update the object mark!)
      __ sd(tmp, Address(box, BasicLock::displaced_header_offset_in_bytes()));

      // Compare object markWord with an unlocked value (tmp) and if
      // equal exchange the stack address of our box with object markWord.
      // On failure disp_hdr contains the possibly locked markWord.
      __ cmpxchg(/*memory address*/oop, /*expected value*/tmp, /*new value*/box, Assembler::int64, Assembler::aq,
                 Assembler::rl, /*result*/disp_hdr);
      __ mv(flag, zr);
      __ beq(disp_hdr, tmp, cont); // prepare zero flag and goto cont if we won the cas

      assert(oopDesc::mark_offset_in_bytes() == 0, "offset of _mark is not 0");

      // If the compare-and-exchange succeeded, then we found an unlocked
      // object, will have now locked it will continue at label cont
      // We did not see an unlocked object so try the fast recursive case.

      // Check if the owner is self by comparing the value in the
      // markWord of object (disp_hdr) with the stack pointer.
      __ sub(disp_hdr, disp_hdr, sp);
      __ li(tmp, (intptr_t) (~(os::vm_page_size()-1) | (uintptr_t)markWord::lock_mask_in_place));
      // If (mark & lock_mask) == 0 and mark - sp < page_size, we are stack-locking and goto cont,
      // hence we can store 0 as the displaced header in the box, which indicates that it is a
      // recursive lock.
      __ andr(tmp/*==0?*/, disp_hdr, tmp);
      __ sd(tmp/*==0, perhaps*/, Address(box, BasicLock::displaced_header_offset_in_bytes()));
      __ mv(flag, tmp); // we can use the value of tmp as the result here
    } else {
      __ mv(flag, 1); // Set non-zero flag to indicate 'failure' -> take slow-path
    }

    __ j(cont);

    // Handle existing monitor.
    __ bind(object_has_monitor);
    // The object's monitor m is unlocked iff m->owner == NULL,
    // otherwise m->owner may contain a thread or a stack address.
    //
    // Try to CAS m->owner from NULL to current thread.
    __ add(tmp, disp_hdr, (ObjectMonitor::owner_offset_in_bytes() - markWord::monitor_value));
    __ cmpxchg(/*memory address*/tmp, /*expected value*/zr, /*new value*/xthread, Assembler::int64, Assembler::aq,
             Assembler::rl, /*result*/flag); // cas succeeds if flag == zr(expected)

    // Store a non-null value into the box to avoid looking like a re-entrant
    // lock. The fast-path monitor unlock code checks for
    // markWord::monitor_value so use markWord::unused_mark which has the
    // relevant bit set, and also matches ObjectSynchronizer::slow_enter.
    __ mv(tmp, (address)markWord::unused_mark().value());
    __ sd(tmp, Address(box, BasicLock::displaced_header_offset_in_bytes()));

    __ beqz(flag, cont); // CAS success means locking succeeded

    __ bne(flag, xthread, cont); // Check for recursive locking

    // Recursive lock case
    __ mv(flag, zr);
    __ ld(tmp, Address(disp_hdr, ObjectMonitor::recursions_offset_in_bytes() - markWord::monitor_value));
    __ add(tmp, tmp, 1u);
    __ sd(tmp, Address(disp_hdr, ObjectMonitor::recursions_offset_in_bytes() - markWord::monitor_value));

    __ bind(cont);
  
#line 15946 "ad_riscv.cpp"
  }
}

void cmpFastUnlockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {

#line 2412 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Register flag = t1;
    Register oop = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* object */);
    Register box = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* box */);
    Register disp_hdr = as_Register(opnd_array(3)->reg(ra_,this,idx3)/* tmp1 */);
    Register tmp = as_Register(opnd_array(4)->reg(ra_,this,idx4)/* tmp2 */);
    Label cont;
    Label object_has_monitor;

    assert_different_registers(oop, box, tmp, disp_hdr, flag);

    if (!UseHeavyMonitors) {
      // Find the lock address and load the displaced header from the stack.
      __ ld(disp_hdr, Address(box, BasicLock::displaced_header_offset_in_bytes()));

      // If the displaced header is 0, we have a recursive unlock.
      __ mv(flag, disp_hdr);
      __ beqz(disp_hdr, cont);
    }

    // Handle existing monitor.
    __ ld(tmp, Address(oop, oopDesc::mark_offset_in_bytes()));
    __ andi(t0, disp_hdr, markWord::monitor_value);
    __ bnez(t0, object_has_monitor);

    if (!UseHeavyMonitors) {
      // Check if it is still a light weight lock, this is true if we
      // see the stack address of the basicLock in the markWord of the
      // object.

      __ cmpxchg(/*memory address*/oop, /*expected value*/box, /*new value*/disp_hdr, Assembler::int64, Assembler::relaxed,
                 Assembler::rl, /*result*/tmp);
      __ xorr(flag, box, tmp); // box == tmp if cas succeeds
    } else {
      __ mv(flag, 1); // Set non-zero flag to indicate 'failure' -> take slow path
    }
    __ j(cont);

    assert(oopDesc::mark_offset_in_bytes() == 0, "offset of _mark is not 0");

    // Handle existing monitor.
    __ bind(object_has_monitor);
    STATIC_ASSERT(markWord::monitor_value <= INT_MAX);
    __ add(tmp, tmp, -(int)markWord::monitor_value); // monitor
    __ ld(disp_hdr, Address(tmp, ObjectMonitor::recursions_offset_in_bytes()));

    Label notRecursive;
    __ beqz(disp_hdr, notRecursive); // Will be 0 if not recursive.

    // Recursive lock
    __ addi(disp_hdr, disp_hdr, -1);
    __ sd(disp_hdr, Address(tmp, ObjectMonitor::recursions_offset_in_bytes()));
    __ mv(flag, zr);
    __ j(cont);

    __ bind(notRecursive);
    __ ld(flag, Address(tmp, ObjectMonitor::EntryList_offset_in_bytes()));
    __ ld(disp_hdr, Address(tmp, ObjectMonitor::cxq_offset_in_bytes()));
    __ orr(flag, flag, disp_hdr); // Will be 0 if both are 0.
    __ bnez(flag, cont);
    // need a release store here
    __ la(tmp, Address(tmp, ObjectMonitor::owner_offset_in_bytes()));
    __ membar(MacroAssembler::LoadStore | MacroAssembler::StoreStore);
    __ sd(zr, Address(tmp)); // set unowned

    __ bind(cont);
  
#line 16029 "ad_riscv.cpp"
  }
}

void TailCalljmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// method_oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 2515 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Assembler::CompressibleRegion cr(&_masm);
    Register target_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* jump_target */);
    __ jr(target_reg);
  
#line 16048 "ad_riscv.cpp"
  }
}

void TailjmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// ex_oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 2522 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Assembler::CompressibleRegion cr(&_masm);
    Register target_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* jump_target */);
    // exception oop should be in x10
    // ret addr has been popped into ra
    // callee expects it in x13
    __ mv(x13, ra);
    __ jr(target_reg);
  
#line 16071 "ad_riscv.cpp"
  }
}

void CreateExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint CreateExceptionNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void RethrowExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 2533 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    __ far_jump(RuntimeAddress(OptoRuntime::rethrow_stub()));
  
#line 16100 "ad_riscv.cpp"
  }
}

void RetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 2538 "src/hotspot/cpu/riscv/riscv.ad"

    C2_MacroAssembler _masm(&cbuf);
    Assembler::CompressibleRegion cr(&_masm);
    __ ret();
  
#line 16117 "ad_riscv.cpp"
  }
}

void ShouldNotReachHereNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10558 "src/hotspot/cpu/riscv/riscv.ad"

    if (is_reachable()) {
      __ stop(_halt_reason);
    }
  
#line 16135 "ad_riscv.cpp"
  }
}

void loadVNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 109 "src/hotspot/cpu/riscv/riscv_v.ad"

    VectorRegister dst_reg = as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(C2_MacroAssembler(&cbuf), false, dst_reg,
              Matcher::vector_element_basic_type(this), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 16153 "ad_riscv.cpp"
  }
}

void storeVNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 121 "src/hotspot/cpu/riscv/riscv_v.ad"

    VectorRegister src_reg = as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(C2_MacroAssembler(&cbuf), true, src_reg,
              Matcher::vector_element_basic_type(this, opnd_array(2)), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 16172 "ad_riscv.cpp"
  }
}

void vabsBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 137 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    __ vrsub_vi(as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* tmp */), 0, as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    __ vmax_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* tmp */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 16191 "ad_riscv.cpp"
  }
}

void vabsSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 151 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    __ vrsub_vi(as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* tmp */), 0, as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    __ vmax_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* tmp */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 16210 "ad_riscv.cpp"
  }
}

void vabsINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 165 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vrsub_vi(as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* tmp */), 0, as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    __ vmax_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* tmp */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 16229 "ad_riscv.cpp"
  }
}

void vabsLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 179 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vrsub_vi(as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* tmp */), 0, as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    __ vmax_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* tmp */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 16248 "ad_riscv.cpp"
  }
}

void vabsFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 191 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vfsgnjx_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 16265 "ad_riscv.cpp"
  }
}

void vabsDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 202 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vfsgnjx_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 16282 "ad_riscv.cpp"
  }
}

void vaddBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 215 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    __ vadd_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16302 "ad_riscv.cpp"
  }
}

void vaddSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 228 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    __ vadd_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16322 "ad_riscv.cpp"
  }
}

void vaddINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 241 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vadd_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16342 "ad_riscv.cpp"
  }
}

void vaddLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 254 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vadd_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16362 "ad_riscv.cpp"
  }
}

void vaddFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 267 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vfadd_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
                as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16382 "ad_riscv.cpp"
  }
}

void vaddDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 280 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vfadd_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
                as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16402 "ad_riscv.cpp"
  }
}

void vandNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 295 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vand_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16422 "ad_riscv.cpp"
  }
}

void vorNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 310 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vor_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16442 "ad_riscv.cpp"
  }
}

void vxorNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 325 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vxor_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16462 "ad_riscv.cpp"
  }
}

void vdivFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 340 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vfdiv_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
                as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16482 "ad_riscv.cpp"
  }
}

void vdivDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 353 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vfdiv_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
                as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16502 "ad_riscv.cpp"
  }
}

void vmaxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 370 "src/hotspot/cpu/riscv/riscv_v.ad"

    BasicType bt = Matcher::vector_element_basic_type(this);
    Assembler::SEW sew = Assembler::elemtype_to_sew(bt);
    __ vsetvli(t0, x0, sew);
    __ vmax_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16523 "ad_riscv.cpp"
  }
}

void vminNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 386 "src/hotspot/cpu/riscv/riscv_v.ad"

    BasicType bt = Matcher::vector_element_basic_type(this);
    Assembler::SEW sew = Assembler::elemtype_to_sew(bt);
    __ vsetvli(t0, x0, sew);
    __ vmin_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 16544 "ad_riscv.cpp"
  }
}

void vmaxFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 404 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ minmax_FD_v(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
                   as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                   false /* is_double */, false /* is_min */);
  
#line 16564 "ad_riscv.cpp"
  }
}

void vmaxDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 418 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ minmax_FD_v(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
                   as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                   true /* is_double */, false /* is_min */);
  
#line 16584 "ad_riscv.cpp"
  }
}

void vminFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 432 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ minmax_FD_v(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
                   as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                   false /* is_double */, true /* is_min */);
  
#line 16604 "ad_riscv.cpp"
  }
}

void vminDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 446 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ minmax_FD_v(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
                   as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                   true /* is_double */, true /* is_min */);
  
#line 16624 "ad_riscv.cpp"
  }
}

void vfmlaFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 462 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vfmacc_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                 as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16644 "ad_riscv.cpp"
  }
}

void vfmlaDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 476 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vfmacc_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                 as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16664 "ad_riscv.cpp"
  }
}

void vfmlsFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 494 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vfnmsac_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                  as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16684 "ad_riscv.cpp"
  }
}

void vfmlsF_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 494 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vfnmsac_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                  as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16704 "ad_riscv.cpp"
  }
}

void vfmlsDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 510 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vfnmsac_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                  as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16724 "ad_riscv.cpp"
  }
}

void vfmlsD_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 510 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vfnmsac_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                  as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16744 "ad_riscv.cpp"
  }
}

void vfnmlaFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 528 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vfnmacc_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                  as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16764 "ad_riscv.cpp"
  }
}

void vfnmlaF_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 528 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vfnmacc_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                  as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16784 "ad_riscv.cpp"
  }
}

void vfnmlaDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 544 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vfnmacc_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                  as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16804 "ad_riscv.cpp"
  }
}

void vfnmlaD_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 544 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vfnmacc_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                  as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16824 "ad_riscv.cpp"
  }
}

void vfnmlsFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 560 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vfmsac_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                 as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16844 "ad_riscv.cpp"
  }
}

void vfnmlsDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 574 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vfmsac_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                 as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16864 "ad_riscv.cpp"
  }
}

void vmlaBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 589 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    __ vmacc_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16884 "ad_riscv.cpp"
  }
}

void vmlaB_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst_src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 589 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    __ vmacc_vv(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst_src1 */),
                as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src3 */));
  
#line 16904 "ad_riscv.cpp"
  }
}

void vmlaSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 602 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    __ vmacc_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16924 "ad_riscv.cpp"
  }
}

void vmlaS_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst_src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 602 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    __ vmacc_vv(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst_src1 */),
                as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src3 */));
  
#line 16944 "ad_riscv.cpp"
  }
}

void vmlaINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 615 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vmacc_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 16964 "ad_riscv.cpp"
  }
}

void vmlaI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst_src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 615 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vmacc_vv(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst_src1 */),
                as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src3 */));
  
#line 16984 "ad_riscv.cpp"
  }
}

void vmlaLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 628 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vmacc_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 17004 "ad_riscv.cpp"
  }
}

void vmlaL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst_src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 628 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vmacc_vv(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst_src1 */),
                as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src3 */));
  
#line 17024 "ad_riscv.cpp"
  }
}

void vmlsBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 643 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    __ vnmsac_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                 as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 17044 "ad_riscv.cpp"
  }
}

void vmlsSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 656 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    __ vnmsac_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                 as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 17064 "ad_riscv.cpp"
  }
}

void vmlsINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 669 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vnmsac_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                 as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 17084 "ad_riscv.cpp"
  }
}

void vmlsLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst_src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 682 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vnmsac_vv(as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst_src1 */),
                 as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 17104 "ad_riscv.cpp"
  }
}

void vmulBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 696 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    __ vmul_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 17123 "ad_riscv.cpp"
  }
}

void vmulSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 708 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    __ vmul_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 17142 "ad_riscv.cpp"
  }
}

void vmulINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 720 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vmul_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 17161 "ad_riscv.cpp"
  }
}

void vmulLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 732 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vmul_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 17180 "ad_riscv.cpp"
  }
}

void vmulFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 744 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vfmul_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
                as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 17199 "ad_riscv.cpp"
  }
}

void vmulDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 756 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vfmul_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
                as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 17218 "ad_riscv.cpp"
  }
}

void vnegFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 770 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vfneg_v(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 17235 "ad_riscv.cpp"
  }
}

void vnegDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 781 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vfneg_v(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 17252 "ad_riscv.cpp"
  }
}

void reduce_addBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 798 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    __ vmv_s_x(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */);
    __ vredsum_vs(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                  as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
    __ vmv_x_s(opnd_array(0)->as_Register(ra_,this)/* dst */, as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 17274 "ad_riscv.cpp"
  }
}

void reduce_addSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 816 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    __ vmv_s_x(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */);
    __ vredsum_vs(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                  as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
    __ vmv_x_s(opnd_array(0)->as_Register(ra_,this)/* dst */, as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 17296 "ad_riscv.cpp"
  }
}

void reduce_addINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 834 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vmv_s_x(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */);
    __ vredsum_vs(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                  as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
    __ vmv_x_s(opnd_array(0)->as_Register(ra_,this)/* dst */, as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 17318 "ad_riscv.cpp"
  }
}

void reduce_addLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 852 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vmv_s_x(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */);
    __ vredsum_vs(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                  as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
    __ vmv_x_s(opnd_array(0)->as_Register(ra_,this)/* dst */, as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 17340 "ad_riscv.cpp"
  }
}

void reduce_addFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1_dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 869 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vfmv_s_f(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src1_dst */);
    __ vfredosum_vs(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                    as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
    __ vfmv_f_s(opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src1_dst */, as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 17362 "ad_riscv.cpp"
  }
}

void reduce_addDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1_dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 886 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vfmv_s_f(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src1_dst */);
    __ vfredosum_vs(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                    as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
    __ vfmv_f_s(opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src1_dst */, as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 17384 "ad_riscv.cpp"
  }
}

void vreduce_maxBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 903 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    __ vredmax_vs(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
    __ vmv_x_s(opnd_array(0)->as_Register(ra_,this)/* dst */, as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
    Label Ldone;
    __ ble(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Ldone);
    __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */));
    __ bind(Ldone);
  
#line 17408 "ad_riscv.cpp"
  }
}

void vreduce_maxSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 921 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    __ vredmax_vs(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
    __ vmv_x_s(opnd_array(0)->as_Register(ra_,this)/* dst */, as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
    Label Ldone;
    __ ble(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Ldone);
    __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */));
    __ bind(Ldone);
  
#line 17432 "ad_riscv.cpp"
  }
}

void vreduce_maxINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 939 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vmv_s_x(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */);
    __ vredmax_vs(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
    __ vmv_x_s(opnd_array(0)->as_Register(ra_,this)/* dst */, as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 17453 "ad_riscv.cpp"
  }
}

void vreduce_maxLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 954 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vmv_s_x(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */);
    __ vredmax_vs(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
    __ vmv_x_s(opnd_array(0)->as_Register(ra_,this)/* dst */, as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 17474 "ad_riscv.cpp"
  }
}

void vreduce_minBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 970 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    __ vredmin_vs(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
    __ vmv_x_s(opnd_array(0)->as_Register(ra_,this)/* dst */, as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
    Label Ldone;
    __ bge(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Ldone);
    __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */));
    __ bind(Ldone);
  
#line 17498 "ad_riscv.cpp"
  }
}

void vreduce_minSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 988 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    __ vredmin_vs(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
    __ vmv_x_s(opnd_array(0)->as_Register(ra_,this)/* dst */, as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
    Label Ldone;
    __ bge(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Ldone);
    __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */));
    __ bind(Ldone);
  
#line 17522 "ad_riscv.cpp"
  }
}

void vreduce_minINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1006 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vmv_s_x(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */);
    __ vredmin_vs(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
    __ vmv_x_s(opnd_array(0)->as_Register(ra_,this)/* dst */, as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 17543 "ad_riscv.cpp"
  }
}

void vreduce_minLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1021 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vmv_s_x(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */);
    __ vredmin_vs(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */), as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
    __ vmv_x_s(opnd_array(0)->as_Register(ra_,this)/* dst */, as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* tmp */));
  
#line 17564 "ad_riscv.cpp"
  }
}

void vreduce_maxFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1038 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ reduce_minmax_FD_v(opnd_array(3)->as_FloatRegister(ra_,this,idx3)/* dst */,
                          opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src1 */, as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                          as_VectorRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp1 */), as_VectorRegister(opnd_array(5)->reg(ra_,this,idx5)/* tmp2 */),
                          false /* is_double */, false /* is_min */);
  
#line 17587 "ad_riscv.cpp"
  }
}

void vreduce_maxDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1053 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ reduce_minmax_FD_v(opnd_array(3)->as_FloatRegister(ra_,this,idx3)/* dst */,
                          opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src1 */, as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                          as_VectorRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp1 */), as_VectorRegister(opnd_array(5)->reg(ra_,this,idx5)/* tmp2 */),
                          true /* is_double */, false /* is_min */);
  
#line 17610 "ad_riscv.cpp"
  }
}

void vreduce_minFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1070 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ reduce_minmax_FD_v(opnd_array(3)->as_FloatRegister(ra_,this,idx3)/* dst */,
                          opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src1 */, as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                          as_VectorRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp1 */), as_VectorRegister(opnd_array(5)->reg(ra_,this,idx5)/* tmp2 */),
                          false /* is_double */, true /* is_min */);
  
#line 17633 "ad_riscv.cpp"
  }
}

void vreduce_minDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1085 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ reduce_minmax_FD_v(opnd_array(3)->as_FloatRegister(ra_,this,idx3)/* dst */,
                          opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src1 */, as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                          as_VectorRegister(opnd_array(4)->reg(ra_,this,idx4)/* tmp1 */), as_VectorRegister(opnd_array(5)->reg(ra_,this,idx5)/* tmp2 */),
                          true /* is_double */, true /* is_min */);
  
#line 17656 "ad_riscv.cpp"
  }
}

void vroundDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rmode
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1100 "src/hotspot/cpu/riscv/riscv_v.ad"

    switch (opnd_array(2)->constant()) {
      case RoundDoubleModeNode::rmode_rint:
        __ csrwi(CSR_FRM, C2_MacroAssembler::rne);
        __ vfcvt_rtz_x_f_v(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
        break;
      case RoundDoubleModeNode::rmode_floor:
        __ csrwi(CSR_FRM, C2_MacroAssembler::rdn);
        __ vfcvt_rtz_x_f_v(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
        break;
      case RoundDoubleModeNode::rmode_ceil:
        __ csrwi(CSR_FRM, C2_MacroAssembler::rup);
        __ vfcvt_rtz_x_f_v(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
        break;
      default:
        ShouldNotReachHere();
        break;
    }
  
#line 17689 "ad_riscv.cpp"
  }
}

void replicateBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1128 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    __ vmv_v_x(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 17706 "ad_riscv.cpp"
  }
}

void replicateSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1139 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    __ vmv_v_x(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 17723 "ad_riscv.cpp"
  }
}

void replicateINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1150 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vmv_v_x(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 17740 "ad_riscv.cpp"
  }
}

void replicateLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1161 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vmv_v_x(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 17757 "ad_riscv.cpp"
  }
}

void replicateB_imm5Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1172 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    __ vmv_v_i(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->constant());
  
#line 17774 "ad_riscv.cpp"
  }
}

void replicateS_imm5Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1183 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    __ vmv_v_i(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->constant());
  
#line 17791 "ad_riscv.cpp"
  }
}

void replicateI_imm5Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1194 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vmv_v_i(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->constant());
  
#line 17808 "ad_riscv.cpp"
  }
}

void replicateL_imm5Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1205 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vmv_v_i(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->constantL());
  
#line 17825 "ad_riscv.cpp"
  }
}

void replicateFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1216 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vfmv_v_f(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src */);
  
#line 17842 "ad_riscv.cpp"
  }
}

void replicateDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1227 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vfmv_v_f(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src */);
  
#line 17859 "ad_riscv.cpp"
  }
}

void vasrBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1244 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    // if shift > BitsPerByte - 1, clear the low BitsPerByte - 1 bits
    __ vmsgtu_vi(v0, as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */), BitsPerByte - 1);
    __ vsra_vi(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               BitsPerByte - 1, Assembler::v0_t);
    // otherwise, shift
    __ vmnot_m(v0, v0);
    __ vsra_vv(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */), Assembler::v0_t);
  
#line 17885 "ad_riscv.cpp"
  }
}

void vasrSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1266 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    // if shift > BitsPerShort - 1, clear the low BitsPerShort - 1 bits
    __ vmsgtu_vi(v0, as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */), BitsPerShort - 1);
    __ vsra_vi(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               BitsPerShort - 1, Assembler::v0_t);
    // otherwise, shift
    __ vmnot_m(v0, v0);
    __ vsra_vv(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */), Assembler::v0_t);
  
#line 17911 "ad_riscv.cpp"
  }
}

void vasrINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1284 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vsra_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 17930 "ad_riscv.cpp"
  }
}

void vasrLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1296 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vsra_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
         as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 17949 "ad_riscv.cpp"
  }
}

void vlslBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1312 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    // if shift > BitsPerByte - 1, clear the element
    __ vmsgtu_vi(v0, as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */), BitsPerByte - 1);
    __ vxor_vv(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), Assembler::v0_t);
    // otherwise, shift
    __ vmnot_m(v0, v0);
    __ vsll_vv(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */), Assembler::v0_t);
  
#line 17975 "ad_riscv.cpp"
  }
}

void vlslSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1334 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    // if shift > BitsPerShort - 1, clear the element
    __ vmsgtu_vi(v0, as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */), BitsPerShort - 1);
    __ vxor_vv(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), Assembler::v0_t);
    // otherwise, shift
    __ vmnot_m(v0, v0);
    __ vsll_vv(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */), Assembler::v0_t);
  
#line 18001 "ad_riscv.cpp"
  }
}

void vlslINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1352 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vsll_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 18020 "ad_riscv.cpp"
  }
}

void vlslLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1364 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vsll_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 18039 "ad_riscv.cpp"
  }
}

void vlsrBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1380 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    // if shift > BitsPerByte - 1, clear the element
    __ vmsgtu_vi(v0, as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */), BitsPerByte - 1);
    __ vxor_vv(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), Assembler::v0_t);
    // otherwise, shift
    __ vmnot_m(v0, v0);
    __ vsrl_vv(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */), Assembler::v0_t);
  
#line 18065 "ad_riscv.cpp"
  }
}

void vlsrSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1402 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    // if shift > BitsPerShort - 1, clear the element
    __ vmsgtu_vi(v0, as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */), BitsPerShort - 1);
    __ vxor_vv(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), Assembler::v0_t);
    // otherwise, shift
    __ vmnot_m(v0, v0);
    __ vsrl_vv(as_VectorRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */), Assembler::v0_t);
  
#line 18091 "ad_riscv.cpp"
  }
}

void vlsrINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1421 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vsrl_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 18110 "ad_riscv.cpp"
  }
}

void vlsrLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1434 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vsrl_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 18129 "ad_riscv.cpp"
  }
}

void vasrB_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1446 "src/hotspot/cpu/riscv/riscv_v.ad"

    uint32_t con = (unsigned)opnd_array(2)->constant()& 0x1f;
    __ vsetvli(t0, x0, Assembler::e8);
    if (con == 0) {
      __ vor_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
                as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
      return;
    }
    if (con >= BitsPerByte) con = BitsPerByte - 1;
    __ vsra_vi(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), con);
  
#line 18154 "ad_riscv.cpp"
  }
}

void vasrS_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1464 "src/hotspot/cpu/riscv/riscv_v.ad"

    uint32_t con = (unsigned)opnd_array(2)->constant()& 0x1f;
    __ vsetvli(t0, x0, Assembler::e16);
    if (con == 0) {
      __ vor_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
                as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
      return;
    }
    if (con >= BitsPerShort) con = BitsPerShort - 1;
    __ vsra_vi(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), con);
  
#line 18179 "ad_riscv.cpp"
  }
}

void vasrI_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1482 "src/hotspot/cpu/riscv/riscv_v.ad"

    uint32_t con = (unsigned)opnd_array(2)->constant()& 0x1f;
    __ vsetvli(t0, x0, Assembler::e32);
    if (con == 0) {
      __ vor_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
                as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
      return;
    }
    __ vsra_vi(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), con);
  
#line 18203 "ad_riscv.cpp"
  }
}

void vasrL_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1500 "src/hotspot/cpu/riscv/riscv_v.ad"

    uint32_t con = (unsigned)opnd_array(2)->constant()& 0x1f;
    __ vsetvli(t0, x0, Assembler::e64);
    if (con == 0) {
      __ vor_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
                as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
      return;
    }
    __ vsra_vi(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), con);
  
#line 18227 "ad_riscv.cpp"
  }
}

void vlsrB_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1517 "src/hotspot/cpu/riscv/riscv_v.ad"

    uint32_t con = (unsigned)opnd_array(2)->constant()& 0x1f;
    __ vsetvli(t0, x0, Assembler::e8);
    if (con == 0) {
      __ vor_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
                as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
      return;
    }
    if (con >= BitsPerByte) {
      __ vxor_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
                 as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
      return;
    }
    __ vsrl_vi(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), con);
  
#line 18256 "ad_riscv.cpp"
  }
}

void vlsrS_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1539 "src/hotspot/cpu/riscv/riscv_v.ad"

    uint32_t con = (unsigned)opnd_array(2)->constant()& 0x1f;
    __ vsetvli(t0, x0, Assembler::e16);
    if (con == 0) {
      __ vor_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
                as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
      return;
    }
    if (con >= BitsPerShort) {
      __ vxor_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
                 as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
      return;
    }
    __ vsrl_vi(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), con);
  
#line 18285 "ad_riscv.cpp"
  }
}

void vlsrI_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1561 "src/hotspot/cpu/riscv/riscv_v.ad"

    uint32_t con = (unsigned)opnd_array(2)->constant()& 0x1f;
    __ vsetvli(t0, x0, Assembler::e32);
    if (con == 0) {
      __ vor_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
                as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
      return;
    }
    __ vsrl_vi(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), con);
  
#line 18309 "ad_riscv.cpp"
  }
}

void vlsrL_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1579 "src/hotspot/cpu/riscv/riscv_v.ad"

    uint32_t con = (unsigned)opnd_array(2)->constant()& 0x1f;
    __ vsetvli(t0, x0, Assembler::e64);
    if (con == 0) {
      __ vor_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
                as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
      return;
    }
    __ vsrl_vi(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), con);
  
#line 18333 "ad_riscv.cpp"
  }
}

void vlslB_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1596 "src/hotspot/cpu/riscv/riscv_v.ad"

    uint32_t con = (unsigned)opnd_array(2)->constant()& 0x1f;
    __ vsetvli(t0, x0, Assembler::e8);
    if (con >= BitsPerByte) {
      __ vxor_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
                 as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
      return;
    }
    __ vsll_vi(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), con);
  
#line 18357 "ad_riscv.cpp"
  }
}

void vlslS_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1613 "src/hotspot/cpu/riscv/riscv_v.ad"

    uint32_t con = (unsigned)opnd_array(2)->constant()& 0x1f;
    __ vsetvli(t0, x0, Assembler::e16);
    if (con >= BitsPerShort) {
      __ vxor_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
                 as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
      return;
    }
    __ vsll_vi(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), con);
  
#line 18381 "ad_riscv.cpp"
  }
}

void vlslI_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1630 "src/hotspot/cpu/riscv/riscv_v.ad"

    uint32_t con = (unsigned)opnd_array(2)->constant()& 0x1f;
    __ vsetvli(t0, x0, Assembler::e32);
    __ vsll_vi(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), con);
  
#line 18400 "ad_riscv.cpp"
  }
}

void vlslL_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1643 "src/hotspot/cpu/riscv/riscv_v.ad"

    uint32_t con = (unsigned)opnd_array(2)->constant()& 0x1f;
    __ vsetvli(t0, x0, Assembler::e64);
    __ vsll_vi(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), con);
  
#line 18419 "ad_riscv.cpp"
  }
}

void vshiftcntBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1656 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    __ vmv_v_x(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* cnt */));
  
#line 18436 "ad_riscv.cpp"
  }
}

void vshiftcntB_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1656 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    __ vmv_v_x(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* cnt */));
  
#line 18453 "ad_riscv.cpp"
  }
}

void vshiftcntSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1669 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    __ vmv_v_x(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* cnt */));
  
#line 18470 "ad_riscv.cpp"
  }
}

void vshiftcntS_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1669 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    __ vmv_v_x(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* cnt */));
  
#line 18487 "ad_riscv.cpp"
  }
}

void vshiftcntINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1681 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vmv_v_x(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* cnt */));
  
#line 18504 "ad_riscv.cpp"
  }
}

void vshiftcntI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1681 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vmv_v_x(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* cnt */));
  
#line 18521 "ad_riscv.cpp"
  }
}

void vshiftcntLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1693 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vmv_v_x(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* cnt */));
  
#line 18538 "ad_riscv.cpp"
  }
}

void vshiftcntL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1693 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vmv_v_x(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* cnt */));
  
#line 18555 "ad_riscv.cpp"
  }
}

void vsqrtFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1706 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vfsqrt_v(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 18572 "ad_riscv.cpp"
  }
}

void vsqrtDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1717 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vfsqrt_v(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 18589 "ad_riscv.cpp"
  }
}

void vsubBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1730 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e8);
    __ vsub_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 18608 "ad_riscv.cpp"
  }
}

void vsubSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1742 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e16);
    __ vsub_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 18627 "ad_riscv.cpp"
  }
}

void vsubINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1754 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vsub_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 18646 "ad_riscv.cpp"
  }
}

void vsubLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1766 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vsub_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
               as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 18665 "ad_riscv.cpp"
  }
}

void vsubFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1778 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e32);
    __ vfsub_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
                as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 18684 "ad_riscv.cpp"
  }
}

void vsubDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1790 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ vsetvli(t0, x0, Assembler::e64);
    __ vfsub_vv(as_VectorRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_VectorRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
                as_VectorRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 18703 "ad_riscv.cpp"
  }
}

void vstring_equalsLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// str2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// v1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// v2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// v3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1807 "src/hotspot/cpu/riscv/riscv_v.ad"

    // Count is in 8-bit bytes; non-Compact chars are 16 bits.
    __ string_equals_v(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* str2 */,
                       opnd_array(0)->as_Register(ra_,this)/* result */, opnd_array(3)->as_Register(ra_,this,idx3)/* cnt */, 1);
  
#line 18726 "ad_riscv.cpp"
  }
}

void vstring_equalsUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// str2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// v1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// v2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// v3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1824 "src/hotspot/cpu/riscv/riscv_v.ad"

    // Count is in 8-bit bytes; non-Compact chars are 16 bits.
    __ string_equals_v(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* str2 */,
                       opnd_array(0)->as_Register(ra_,this)/* result */, opnd_array(3)->as_Register(ra_,this,idx3)/* cnt */, 2);
  
#line 18749 "ad_riscv.cpp"
  }
}

void varray_equalsBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// v1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// v2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// v3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1840 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ arrays_equals_v(opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                       opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(R28_enc), 1);
    
#line 18770 "ad_riscv.cpp"
  }
}

void varray_equalsCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// v1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// v2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// v3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1855 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ arrays_equals_v(opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                       opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(R28_enc), 2);
  
#line 18791 "ad_riscv.cpp"
  }
}

void vstring_compareUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// v1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// v2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// v3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// v4
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// v5
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1872 "src/hotspot/cpu/riscv/riscv_v.ad"

    // Count is in 8-bit bytes; non-Compact chars are 16 bits.
    __ string_compare_v(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                        opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                        as_Register(R28_enc), as_Register(R29_enc),
                        StrIntrinsicNode::UU);
  
#line 18819 "ad_riscv.cpp"
  }
}

void vstring_compareLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// v1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// v2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// v3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// v4
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// v5
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1891 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ string_compare_v(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                        opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                        as_Register(R28_enc), as_Register(R29_enc),
                        StrIntrinsicNode::LL);
  
#line 18846 "ad_riscv.cpp"
  }
}

void vstring_compareULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// v1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// v2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// v3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// v4
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// v5
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1910 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ string_compare_v(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                        opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                        as_Register(R28_enc), as_Register(R29_enc),
                        StrIntrinsicNode::UL);
  
#line 18873 "ad_riscv.cpp"
  }
}

void vstring_compareLUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// v1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// v2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// v3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// v4
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// v5
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1928 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ string_compare_v(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                        opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                        as_Register(R28_enc), as_Register(R29_enc),
                        StrIntrinsicNode::LU);
  
#line 18900 "ad_riscv.cpp"
  }
}

void vstring_inflateNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// v1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// v2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// v3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1946 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ byte_array_inflate_v(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp */);
  
#line 18922 "ad_riscv.cpp"
  }
}

void vencode_iso_arrayNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// result
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// v1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// v2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// v3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1962 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ encode_iso_array_v(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
                          opnd_array(4)->as_Register(ra_,this,idx4)/* result */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp */);
  
#line 18946 "ad_riscv.cpp"
  }
}

void vstring_compressNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// result
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// v1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// v2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// v3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1979 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ char_array_compress_v(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
                             opnd_array(4)->as_Register(ra_,this,idx4)/* result */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp */);
  
#line 18970 "ad_riscv.cpp"
  }
}

void vcount_positivesNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// len
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// result
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// v1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// v2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// v3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 1994 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ count_positives_v(opnd_array(1)->as_Register(ra_,this,idx1)/* ary */, opnd_array(2)->as_Register(ra_,this,idx2)/* len */, opnd_array(3)->as_Register(ra_,this,idx3)/* result */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp */);
  
#line 18992 "ad_riscv.cpp"
  }
}

void vstringU_indexof_charNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// ch
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// result
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// v1
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// v2
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// v3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2012 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ string_indexof_char_v(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* ch */,
                             opnd_array(4)->as_Register(ra_,this,idx4)/* result */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */,
                             false /* isL */);
  
#line 19018 "ad_riscv.cpp"
  }
}

void vstringL_indexof_charNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// ch
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// result
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// v1
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// v2
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// v3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2032 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ string_indexof_char_v(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* ch */,
                             opnd_array(4)->as_Register(ra_,this,idx4)/* result */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */,
                             true /* isL */);
  
#line 19044 "ad_riscv.cpp"
  }
}

void vclearArray_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vReg1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vReg2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vReg3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2051 "src/hotspot/cpu/riscv/riscv_v.ad"

    __ clear_array_v(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */);
  
#line 19064 "ad_riscv.cpp"
  }
}

void rorI_imm_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 35 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ roriw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), opnd_array(2)->constant()& 0x1f);
  
#line 19081 "ad_riscv.cpp"
  }
}

void rorL_imm_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 49 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ rori(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), opnd_array(2)->constant()& 0x3f);
  
#line 19098 "ad_riscv.cpp"
  }
}

void rorI_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 62 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ rorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 19115 "ad_riscv.cpp"
  }
}

void rorL_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 74 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ ror(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 19132 "ad_riscv.cpp"
  }
}

void rolI_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 86 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ rolw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 19149 "ad_riscv.cpp"
  }
}

void rolL_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 98 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ rol(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 19166 "ad_riscv.cpp"
  }
}

void convP2I_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 112 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ zext_w(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19182 "ad_riscv.cpp"
  }
}

void convB2I_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 127 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ sext_b(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19200 "ad_riscv.cpp"
  }
}

void convI2S_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 142 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ sext_h(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19218 "ad_riscv.cpp"
  }
}

void convS2UI_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 157 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ zext_h(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19235 "ad_riscv.cpp"
  }
}

void convI2UL_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 172 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ zext_w(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19252 "ad_riscv.cpp"
  }
}

void bytes_reverse_int_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 187 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ revb_w_w(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19268 "ad_riscv.cpp"
  }
}

void bytes_reverse_long_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 201 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ rev8(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19284 "ad_riscv.cpp"
  }
}

void bytes_reverse_unsigned_short_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 215 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ revb_h_h_u(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19300 "ad_riscv.cpp"
  }
}

void bytes_reverse_short_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 229 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ revb_h_h(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19316 "ad_riscv.cpp"
  }
}

void shaddP_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// imm
  {
    C2_MacroAssembler _masm(&cbuf);

#line 244 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ shadd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             t0,
             opnd_array(3)->constant());
  
#line 19338 "ad_riscv.cpp"
  }
}

void shaddP_reg_reg_ext_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// imm
  {
    C2_MacroAssembler _masm(&cbuf);

#line 262 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ shadd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             t0,
             opnd_array(3)->constant());
  
#line 19360 "ad_riscv.cpp"
  }
}

void shaddL_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// imm
  {
    C2_MacroAssembler _masm(&cbuf);

#line 281 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ shadd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             t0,
             opnd_array(3)->constant());
  
#line 19382 "ad_riscv.cpp"
  }
}

void shaddL_reg_reg_b_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 281 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ shadd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
             t0,
             opnd_array(2)->constant());
  
#line 19404 "ad_riscv.cpp"
  }
}

void shaddL_reg_reg_ext_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// imm
  {
    C2_MacroAssembler _masm(&cbuf);

#line 299 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ shadd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             t0,
             opnd_array(3)->constant());
  
#line 19426 "ad_riscv.cpp"
  }
}

void shaddL_reg_reg_ext_b_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 299 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ shadd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
             t0,
             opnd_array(2)->constant());
  
#line 19448 "ad_riscv.cpp"
  }
}

void countLeadingZerosI_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 318 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ clzw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19464 "ad_riscv.cpp"
  }
}

void countLeadingZerosL_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 332 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ clz(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19480 "ad_riscv.cpp"
  }
}

void countTrailingZerosI_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 346 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ ctzw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19496 "ad_riscv.cpp"
  }
}

void countTrailingZerosL_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 360 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ ctz(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19512 "ad_riscv.cpp"
  }
}

void popCountI_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 375 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ cpopw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19528 "ad_riscv.cpp"
  }
}

void popCountL_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 390 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ cpop(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19544 "ad_riscv.cpp"
  }
}

void minI_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 405 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ min(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 19561 "ad_riscv.cpp"
  }
}

void maxI_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 419 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ max(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 19578 "ad_riscv.cpp"
  }
}

void absI_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 437 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ negw(t0, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    __ max(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), t0);
  
#line 19595 "ad_riscv.cpp"
  }
}

void absL_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 455 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ neg(t0, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    __ max(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), t0);
  
#line 19612 "ad_riscv.cpp"
  }
}

void andnI_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 471 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ andn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 19632 "ad_riscv.cpp"
  }
}

void andnI_reg_reg_b_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 471 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ andn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */));
  
#line 19652 "ad_riscv.cpp"
  }
}

void andnL_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 487 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ andn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 19672 "ad_riscv.cpp"
  }
}

void andnL_reg_reg_b_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 487 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ andn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */));
  
#line 19692 "ad_riscv.cpp"
  }
}

void ornI_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 504 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 19712 "ad_riscv.cpp"
  }
}

void ornI_reg_reg_b_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 504 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */));
  
#line 19732 "ad_riscv.cpp"
  }
}

void ornL_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 520 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 19752 "ad_riscv.cpp"
  }
}

void ornL_reg_reg_b_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 520 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */));
  
#line 19772 "ad_riscv.cpp"
  }
}

void compareAndSwapP_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 41 "src/hotspot/cpu/riscv/gc/shenandoah/shenandoah_riscv64.ad"

    Register tmp = opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */;
    __ mv(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   Assembler::relaxed /* acquire */, Assembler::rl /* release */,
                                                   false /* is_cae */, opnd_array(0)->as_Register(ra_,this)/* res */);
  
#line 19795 "ad_riscv.cpp"
  }
}

void compareAndSwapN_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 62 "src/hotspot/cpu/riscv/gc/shenandoah/shenandoah_riscv64.ad"

    Register tmp = opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */;
    __ mv(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   Assembler::relaxed /* acquire */, Assembler::rl /* release */,
                                                   false /* is_cae */, opnd_array(0)->as_Register(ra_,this)/* res */);
  
#line 19818 "ad_riscv.cpp"
  }
}

void compareAndSwapPAcq_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 84 "src/hotspot/cpu/riscv/gc/shenandoah/shenandoah_riscv64.ad"

    Register tmp = opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */;
    __ mv(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   Assembler::aq /* acquire */, Assembler::rl /* release */,
                                                   false /* is_cae */, opnd_array(0)->as_Register(ra_,this)/* res */);
  
#line 19841 "ad_riscv.cpp"
  }
}

void compareAndSwapNAcq_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 106 "src/hotspot/cpu/riscv/gc/shenandoah/shenandoah_riscv64.ad"

    Register tmp = opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */;
    __ mv(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   Assembler::aq /* acquire */, Assembler::rl /* release */,
                                                   false /* is_cae */, opnd_array(0)->as_Register(ra_,this)/* res */);
  
#line 19864 "ad_riscv.cpp"
  }
}

void compareAndExchangeN_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 126 "src/hotspot/cpu/riscv/gc/shenandoah/shenandoah_riscv64.ad"

    Register tmp = opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */;
    __ mv(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   Assembler::relaxed /* acquire */, Assembler::rl /* release */,
                                                   true /* is_cae */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 19888 "ad_riscv.cpp"
  }
}

void compareAndExchangeP_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 146 "src/hotspot/cpu/riscv/gc/shenandoah/shenandoah_riscv64.ad"

    Register tmp = opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */;
    __ mv(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   Assembler::relaxed /* acquire */, Assembler::rl /* release */,
                                                   true /* is_cae */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 19912 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapN_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 167 "src/hotspot/cpu/riscv/gc/shenandoah/shenandoah_riscv64.ad"

    Register tmp = opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */;
    __ mv(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    // Weak is not current supported by ShenandoahBarrierSet::cmpxchg_oop
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   Assembler::relaxed /* acquire */, Assembler::rl /* release */,
                                                   false /* is_cae */, opnd_array(0)->as_Register(ra_,this)/* res */);
  
#line 19936 "ad_riscv.cpp"
  }
}

void compareAndExchangeNAcq_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 189 "src/hotspot/cpu/riscv/gc/shenandoah/shenandoah_riscv64.ad"

    Register tmp = opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */;
    __ mv(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */);
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   Assembler::aq /* acquire */, Assembler::rl /* release */,
                                                   true /* is_cae */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 19960 "ad_riscv.cpp"
  }
}

void compareAndExchangePAcq_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 210 "src/hotspot/cpu/riscv/gc/shenandoah/shenandoah_riscv64.ad"

    Register tmp = opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */;
    __ mv(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */);
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   Assembler::aq /* acquire */, Assembler::rl /* release */,
                                                   true /* is_cae */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 19984 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapP_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 230 "src/hotspot/cpu/riscv/gc/shenandoah/shenandoah_riscv64.ad"

    Register tmp = opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */;
    __ mv(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   Assembler::relaxed /* acquire */, Assembler::rl /* release */,
                                                   false /* is_cae */, opnd_array(0)->as_Register(ra_,this)/* res */);
  
#line 20007 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapNAcq_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 252 "src/hotspot/cpu/riscv/gc/shenandoah/shenandoah_riscv64.ad"

    Register tmp = opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */;
    __ mv(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    // Weak is not current supported by ShenandoahBarrierSet::cmpxchg_oop
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   Assembler::aq /* acquire */, Assembler::rl /* release */,
                                                   false /* is_cae */, opnd_array(0)->as_Register(ra_,this)/* res */);
  
#line 20031 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapPAcq_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 275 "src/hotspot/cpu/riscv/gc/shenandoah/shenandoah_riscv64.ad"

    Register tmp = opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */;
    __ mv(tmp, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */); // Must not clobber oldval.
    // Weak is not current supported by ShenandoahBarrierSet::cmpxchg_oop
    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm, opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, tmp, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   Assembler::aq /* acquire */, Assembler::rl /* release */,
                                                   false /* is_cae */, opnd_array(0)->as_Register(ra_,this)/* res */);
  
#line 20055 "ad_riscv.cpp"
  }
}

void zLoadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 65 "src/hotspot/cpu/riscv/gc/z/z_riscv64.ad"

    const Address ref_addr (as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1));
    __ ld(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, ref_addr);
    z_load_barrier(_masm, this, ref_addr, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, t0 /* tmp */, barrier_data());
  
#line 20074 "ad_riscv.cpp"
  }
}

void zCompareAndSwapPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 85 "src/hotspot/cpu/riscv/gc/z/z_riscv64.ad"

    Label failed;
    guarantee(opnd_array(1)->index(ra_,this,idx1)== -1 && opnd_array(1)->disp(ra_,this,idx1)== 0, "impossible encoding");
    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               Assembler::relaxed /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
               true /* result_as_bool */);
    __ beqz(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, failed);
    __ mv(t0, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */);
    __ bind(failed);
    if (barrier_data() != ZLoadBarrierElided) {
      Label good;
      __ ld(t1, Address(xthread, ZThreadLocalData::address_bad_mask_offset()), t1 /* tmp */);
      __ andr(t1, t1, t0);
      __ beqz(t1, good);
      z_load_barrier_slow_path(_masm, this, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */), t0 /* ref */, t1 /* tmp */);
      __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
                 Assembler::relaxed /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                 true /* result_as_bool */);
      __ bind(good);
    }
  
#line 20111 "ad_riscv.cpp"
  }
}

void zCompareAndSwapP_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 85 "src/hotspot/cpu/riscv/gc/z/z_riscv64.ad"

    Label failed;
    guarantee(opnd_array(1)->index(ra_,this,idx1)== -1 && opnd_array(1)->disp(ra_,this,idx1)== 0, "impossible encoding");
    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               Assembler::relaxed /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
               true /* result_as_bool */);
    __ beqz(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, failed);
    __ mv(t0, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */);
    __ bind(failed);
    if (barrier_data() != ZLoadBarrierElided) {
      Label good;
      __ ld(t1, Address(xthread, ZThreadLocalData::address_bad_mask_offset()), t1 /* tmp */);
      __ andr(t1, t1, t0);
      __ beqz(t1, good);
      z_load_barrier_slow_path(_masm, this, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */), t0 /* ref */, t1 /* tmp */);
      __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
                 Assembler::relaxed /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                 true /* result_as_bool */);
      __ bind(good);
    }
  
#line 20148 "ad_riscv.cpp"
  }
}

void zCompareAndSwapPAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 121 "src/hotspot/cpu/riscv/gc/z/z_riscv64.ad"

    Label failed;
    guarantee(opnd_array(1)->index(ra_,this,idx1)== -1 && opnd_array(1)->disp(ra_,this,idx1)== 0, "impossible encoding");
    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               Assembler::aq /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
               true /* result_as_bool */);
    __ beqz(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, failed);
    __ mv(t0, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */);
    __ bind(failed);
    if (barrier_data() != ZLoadBarrierElided) {
      Label good;
      __ ld(t1, Address(xthread, ZThreadLocalData::address_bad_mask_offset()), t1 /* tmp */);
      __ andr(t1, t1, t0);
      __ beqz(t1, good);
      z_load_barrier_slow_path(_masm, this, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */), t0 /* ref */, t1 /* tmp */);
      __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
                 Assembler::aq /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                 true /* result_as_bool */);
      __ bind(good);
    }
  
#line 20185 "ad_riscv.cpp"
  }
}

void zCompareAndSwapPAcq_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 121 "src/hotspot/cpu/riscv/gc/z/z_riscv64.ad"

    Label failed;
    guarantee(opnd_array(1)->index(ra_,this,idx1)== -1 && opnd_array(1)->disp(ra_,this,idx1)== 0, "impossible encoding");
    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               Assembler::aq /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
               true /* result_as_bool */);
    __ beqz(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, failed);
    __ mv(t0, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */);
    __ bind(failed);
    if (barrier_data() != ZLoadBarrierElided) {
      Label good;
      __ ld(t1, Address(xthread, ZThreadLocalData::address_bad_mask_offset()), t1 /* tmp */);
      __ andr(t1, t1, t0);
      __ beqz(t1, good);
      z_load_barrier_slow_path(_masm, this, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */), t0 /* ref */, t1 /* tmp */);
      __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
                 Assembler::aq /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                 true /* result_as_bool */);
      __ bind(good);
    }
  
#line 20222 "ad_riscv.cpp"
  }
}

void zCompareAndExchangePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 155 "src/hotspot/cpu/riscv/gc/z/z_riscv64.ad"

    guarantee(opnd_array(1)->index(ra_,this,idx1)== -1 && opnd_array(1)->disp(ra_,this,idx1)== 0, "impossible encoding");
    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               Assembler::relaxed /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
    if (barrier_data() != ZLoadBarrierElided) {
      Label good;
      __ ld(t0, Address(xthread, ZThreadLocalData::address_bad_mask_offset()));
      __ andr(t0, t0, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
      __ beqz(t0, good);
      z_load_barrier_slow_path(_masm, this, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */), opnd_array(4)->as_Register(ra_,this,idx4)/* res */, t0 /* tmp */);
      __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
                 Assembler::relaxed /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
      __ bind(good);
    }
  
#line 20253 "ad_riscv.cpp"
  }
}

void zCompareAndExchangePAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    C2_MacroAssembler _masm(&cbuf);

#line 183 "src/hotspot/cpu/riscv/gc/z/z_riscv64.ad"

    guarantee(opnd_array(1)->index(ra_,this,idx1)== -1 && opnd_array(1)->disp(ra_,this,idx1)== 0, "impossible encoding");
    __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               Assembler::aq /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
    if (barrier_data() != ZLoadBarrierElided) {
      Label good;
      __ ld(t0, Address(xthread, ZThreadLocalData::address_bad_mask_offset()));
      __ andr(t0, t0, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
      __ beqz(t0, good);
      z_load_barrier_slow_path(_masm, this, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */), opnd_array(4)->as_Register(ra_,this,idx4)/* res */, t0 /* tmp */);
      __ cmpxchg(opnd_array(1)->as_Register(ra_,this,idx1)/* mem */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
                 Assembler::aq /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
      __ bind(good);
    }
  
#line 20284 "ad_riscv.cpp"
  }
}

void zGetAndSetPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// prev
  {
    C2_MacroAssembler _masm(&cbuf);

#line 211 "src/hotspot/cpu/riscv/gc/z/z_riscv64.ad"

    __ atomic_xchg(opnd_array(3)->as_Register(ra_,this,idx3)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
    z_load_barrier(_masm, this, Address(noreg, 0), opnd_array(3)->as_Register(ra_,this,idx3)/* prev */, t0 /* tmp */, barrier_data());
  
#line 20303 "ad_riscv.cpp"
  }
}

void zGetAndSetPAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// prev
  {
    C2_MacroAssembler _masm(&cbuf);

#line 228 "src/hotspot/cpu/riscv/gc/z/z_riscv64.ad"

    __ atomic_xchgal(opnd_array(3)->as_Register(ra_,this,idx3)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
    z_load_barrier(_masm, this, Address(noreg, 0), opnd_array(3)->as_Register(ra_,this,idx3)/* prev */, t0 /* tmp */, barrier_data());
  
#line 20322 "ad_riscv.cpp"
  }
}

const MachOper* loadBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadRangeNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadNNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadNKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadFNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmCM0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmCM0_orderedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmB0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeCNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmC0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmI0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmL0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storePNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmP0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeNNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmN0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeFNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeNKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPLockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeIConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapBNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapSNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapINode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapLNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapPNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapNNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapBAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapSAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapIAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapLAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapPAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapNAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeBNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeSNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeINode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeLNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeNNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangePNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeBAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeSAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeIAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeLAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeNAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangePAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapBNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapSNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapINode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapLNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapNNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapPNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapBAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapSAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapIAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapLAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapNAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapPAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setINode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setLNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setNNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setPNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setIAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setLAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setNAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setPAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addL_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLiNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLi_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addINode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addI_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIiNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIi_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addL_no_resAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLiAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLi_no_resAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addI_no_resAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIiAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIi_no_resAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadVNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeVNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapP_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapN_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapPAcq_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapNAcq_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeN_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeP_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapN_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeNAcq_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangePAcq_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapP_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapNAcq_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapPAcq_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* zLoadPNode::memory_operand() const { return _opnds[1]; }
const MachOper* zCompareAndSwapPNode::memory_operand() const { return _opnds[1]; }
const MachOper* zCompareAndSwapP_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* zCompareAndSwapPAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* zCompareAndSwapPAcq_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* zCompareAndExchangePNode::memory_operand() const { return _opnds[1]; }
const MachOper* zCompareAndExchangePAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* zGetAndSetPNode::memory_operand() const { return _opnds[1]; }
const MachOper* zGetAndSetPAcqNode::memory_operand() const { return _opnds[1]; }


const bool Matcher::has_match_rule(int opcode) {
  assert(_last_machine_leaf < opcode && opcode < _last_opcode, "opcode in range");
  return _hasMatchRule[opcode];
}

const bool Matcher::_hasMatchRule[_last_opcode] = {
    false,  // Node
    false,  // Set
    false,  // RegN
    false,  // RegI
    false,  // RegP
    false,  // RegF
    false,  // RegD
    false,  // RegL
    false,  // VecA
    false,  // VecS
    false,  // VecD
    false,  // VecX
    false,  // VecY
    false,  // VecZ
    false,  // RegVectMask
    false,  // RegFlags
    false,  // _last_machine_leaf
    true ,  // AbsD
    true ,  // AbsF
    true ,  // AbsI
    true ,  // AbsL
    true ,  // AddD
    true ,  // AddF
    true ,  // AddI
    true ,  // AddL
    true ,  // AddP
    false,  // Allocate
    false,  // AllocateArray
    true ,  // AndI
    true ,  // AndL
    false,  // ArrayCopy
    true ,  // AryEq
    false,  // AtanD
    true ,  // Binary
    false,  // Blackhole
    true ,  // Bool
    false,  // BoxLock
    true ,  // ReverseBytesI
    true ,  // ReverseBytesL
    true ,  // ReverseBytesUS
    true ,  // ReverseBytesS
    false,  // ReverseBytesV
    false,  // CProj
    false,  // CacheWB
    false,  // CacheWBPreSync
    false,  // CacheWBPostSync
    true ,  // CallDynamicJava
    false,  // CallJava
    true ,  // CallLeaf
    true ,  // CallLeafNoFP
    false,  // CallLeafVector
    true ,  // CallRuntime
    true ,  // CallStaticJava
    true ,  // CastDD
    true ,  // CastFF
    true ,  // CastII
    true ,  // CastLL
    true ,  // CastVV
    true ,  // CastX2P
    true ,  // CastP2X
    true ,  // CastPP
    false,  // Catch
    false,  // CatchProj
    true ,  // CheckCastPP
    true ,  // ClearArray
    false,  // CompressBits
    false,  // ExpandBits
    false,  // ConstraintCast
    false,  // CMoveD
    false,  // CMoveVD
    false,  // CMoveF
    false,  // CMoveVF
    true ,  // CMoveI
    true ,  // CMoveL
    false,  // CMoveP
    false,  // CMoveN
    true ,  // CmpN
    true ,  // CmpD
    true ,  // CmpD3
    true ,  // CmpF
    true ,  // CmpF3
    true ,  // CmpI
    true ,  // CmpL
    true ,  // CmpL3
    true ,  // CmpLTMask
    true ,  // CmpP
    true ,  // CmpU
    true ,  // CmpUL
    true ,  // CompareAndSwapB
    true ,  // CompareAndSwapS
    true ,  // CompareAndSwapI
    true ,  // CompareAndSwapL
    true ,  // CompareAndSwapP
    true ,  // CompareAndSwapN
    true ,  // WeakCompareAndSwapB
    true ,  // WeakCompareAndSwapS
    true ,  // WeakCompareAndSwapI
    true ,  // WeakCompareAndSwapL
    true ,  // WeakCompareAndSwapP
    true ,  // WeakCompareAndSwapN
    true ,  // CompareAndExchangeB
    true ,  // CompareAndExchangeS
    true ,  // CompareAndExchangeI
    true ,  // CompareAndExchangeL
    true ,  // CompareAndExchangeP
    true ,  // CompareAndExchangeN
    false,  // GetAndAddB
    false,  // GetAndAddS
    true ,  // GetAndAddI
    true ,  // GetAndAddL
    false,  // GetAndSetB
    false,  // GetAndSetS
    true ,  // GetAndSetI
    true ,  // GetAndSetL
    true ,  // GetAndSetP
    true ,  // GetAndSetN
    false,  // Con
    true ,  // ConN
    true ,  // ConNKlass
    true ,  // ConD
    true ,  // ConF
    true ,  // ConI
    true ,  // ConL
    true ,  // ConP
    true ,  // Conv2B
    true ,  // ConvD2F
    true ,  // ConvD2I
    true ,  // ConvD2L
    true ,  // ConvF2D
    true ,  // ConvF2I
    true ,  // ConvF2L
    true ,  // ConvI2D
    true ,  // ConvI2F
    true ,  // ConvI2L
    true ,  // ConvL2D
    true ,  // ConvL2F
    true ,  // ConvL2I
    false,  // CountedLoop
    true ,  // CountedLoopEnd
    false,  // OuterStripMinedLoop
    false,  // OuterStripMinedLoopEnd
    false,  // LongCountedLoop
    false,  // LongCountedLoopEnd
    true ,  // CountLeadingZerosI
    true ,  // CountLeadingZerosL
    false,  // CountLeadingZerosV
    true ,  // CountTrailingZerosI
    true ,  // CountTrailingZerosL
    false,  // CountTrailingZerosV
    true ,  // CreateEx
    true ,  // DecodeN
    true ,  // DecodeNKlass
    true ,  // DivD
    true ,  // DivF
    true ,  // DivI
    true ,  // DivL
    false,  // UDivI
    false,  // UDivL
    false,  // DivMod
    false,  // DivModI
    false,  // DivModL
    false,  // UDivModI
    false,  // UDivModL
    true ,  // EncodeISOArray
    true ,  // EncodeP
    true ,  // EncodePKlass
    true ,  // FastLock
    true ,  // FastUnlock
    true ,  // FmaD
    true ,  // FmaF
    true ,  // Goto
    true ,  // Halt
    true ,  // CountPositives
    true ,  // If
    false,  // RangeCheck
    false,  // IfFalse
    false,  // IfTrue
    false,  // Initialize
    false,  // JProj
    false,  // Jump
    false,  // JumpProj
    true ,  // LShiftI
    true ,  // LShiftL
    true ,  // LoadB
    true ,  // LoadUB
    true ,  // LoadUS
    true ,  // LoadD
    false,  // LoadD_unaligned
    true ,  // LoadF
    true ,  // LoadI
    true ,  // LoadKlass
    true ,  // LoadNKlass
    true ,  // LoadL
    false,  // LoadL_unaligned
    true ,  // LoadPLocked
    true ,  // LoadP
    true ,  // LoadN
    true ,  // LoadRange
    true ,  // LoadS
    false,  // Lock
    false,  // Loop
    false,  // LoopLimit
    false,  // Mach
    false,  // MachNullCheck
    false,  // MachProj
    false,  // MulAddS2I
    true ,  // MaxI
    false,  // MaxL
    true ,  // MaxD
    true ,  // MaxF
    true ,  // MemBarAcquire
    true ,  // LoadFence
    true ,  // MemBarAcquireLock
    false,  // MemBarCPUOrder
    true ,  // MemBarRelease
    true ,  // StoreFence
    true ,  // StoreStoreFence
    true ,  // MemBarReleaseLock
    true ,  // MemBarVolatile
    true ,  // MemBarStoreStore
    false,  // MergeMem
    true ,  // MinI
    false,  // MinL
    true ,  // MinF
    true ,  // MinD
    false,  // ModD
    false,  // ModF
    true ,  // ModI
    true ,  // ModL
    false,  // UModI
    false,  // UModL
    true ,  // MoveI2F
    true ,  // MoveF2I
    true ,  // MoveL2D
    true ,  // MoveD2L
    false,  // IsInfiniteF
    false,  // IsInfiniteD
    true ,  // MulD
    true ,  // MulF
    true ,  // MulHiL
    false,  // UMulHiL
    true ,  // MulI
    true ,  // MulL
    false,  // Multi
    false,  // NegI
    false,  // NegL
    true ,  // NegD
    true ,  // NegF
    false,  // NeverBranch
    false,  // OnSpinWait
    false,  // Opaque1
    false,  // OpaqueLoopInit
    false,  // OpaqueLoopStride
    false,  // Opaque2
    false,  // Opaque3
    false,  // Opaque4
    false,  // ProfileBoolean
    true ,  // OrI
    true ,  // OrL
    false,  // OverflowAddI
    false,  // OverflowSubI
    false,  // OverflowMulI
    false,  // OverflowAddL
    false,  // OverflowSubL
    false,  // OverflowMulL
    false,  // PCTable
    false,  // Parm
    true ,  // PartialSubtypeCheck
    false,  // SubTypeCheck
    false,  // Phi
    true ,  // PopCountI
    true ,  // PopCountL
    false,  // PopCountVI
    false,  // PopCountVL
    false,  // PopulateIndex
    false,  // PrefetchAllocation
    false,  // Proj
    true ,  // RShiftI
    true ,  // RShiftL
    false,  // Region
    true ,  // Rethrow
    true ,  // Return
    false,  // ReverseI
    false,  // ReverseL
    false,  // ReverseV
    false,  // Root
    false,  // RoundDouble
    false,  // RoundDoubleMode
    true ,  // RoundDoubleModeV
    false,  // RoundFloat
    true ,  // RotateLeft
    false,  // RotateLeftV
    true ,  // RotateRight
    false,  // RotateRightV
    true ,  // SafePoint
    false,  // SafePointScalarObject
    true ,  // ShenandoahCompareAndExchangeP
    true ,  // ShenandoahCompareAndExchangeN
    true ,  // ShenandoahCompareAndSwapN
    true ,  // ShenandoahCompareAndSwapP
    true ,  // ShenandoahWeakCompareAndSwapN
    true ,  // ShenandoahWeakCompareAndSwapP
    false,  // ShenandoahIUBarrier
    false,  // ShenandoahLoadReferenceBarrier
    false,  // SCMemProj
    false,  // CopySignD
    false,  // CopySignF
    false,  // SignumD
    false,  // SignumF
    false,  // SignumVF
    false,  // SignumVD
    true ,  // SqrtD
    false,  // SqrtF
    false,  // RoundF
    false,  // RoundD
    false,  // Start
    false,  // StartOSR
    true ,  // StoreB
    true ,  // StoreC
    true ,  // StoreCM
    true ,  // StorePConditional
    true ,  // StoreIConditional
    true ,  // StoreLConditional
    true ,  // StoreD
    true ,  // StoreF
    true ,  // StoreI
    true ,  // StoreL
    true ,  // StoreP
    true ,  // StoreN
    true ,  // StoreNKlass
    true ,  // StrComp
    true ,  // StrCompressedCopy
    true ,  // StrEquals
    true ,  // StrIndexOf
    true ,  // StrIndexOfChar
    true ,  // StrInflatedCopy
    true ,  // SubD
    true ,  // SubF
    true ,  // SubI
    true ,  // SubL
    true ,  // TailCall
    true ,  // TailJump
    false,  // MacroLogicV
    true ,  // ThreadLocal
    false,  // Unlock
    false,  // URShiftB
    false,  // URShiftS
    true ,  // URShiftI
    true ,  // URShiftL
    true ,  // XorI
    true ,  // XorL
    false,  // Vector
    true ,  // AddVB
    true ,  // AddVS
    true ,  // AddVI
    true ,  // AddReductionVI
    true ,  // AddVL
    true ,  // AddReductionVL
    true ,  // AddVF
    true ,  // AddReductionVF
    true ,  // AddVD
    true ,  // AddReductionVD
    true ,  // SubVB
    true ,  // SubVS
    true ,  // SubVI
    true ,  // SubVL
    true ,  // SubVF
    true ,  // SubVD
    true ,  // MulVB
    true ,  // MulVS
    true ,  // MulVI
    false,  // MulReductionVI
    true ,  // MulVL
    false,  // MulReductionVL
    true ,  // MulVF
    false,  // MulReductionVF
    true ,  // MulVD
    false,  // MulReductionVD
    false,  // MulAddVS2VI
    true ,  // FmaVD
    true ,  // FmaVF
    true ,  // DivVF
    true ,  // DivVD
    true ,  // AbsVB
    true ,  // AbsVS
    true ,  // AbsVI
    true ,  // AbsVL
    true ,  // AbsVF
    true ,  // AbsVD
    false,  // NegVI
    false,  // NegVL
    true ,  // NegVF
    true ,  // NegVD
    true ,  // SqrtVD
    true ,  // SqrtVF
    true ,  // LShiftCntV
    true ,  // RShiftCntV
    true ,  // LShiftVB
    true ,  // LShiftVS
    true ,  // LShiftVI
    true ,  // LShiftVL
    true ,  // RShiftVB
    true ,  // RShiftVS
    true ,  // RShiftVI
    true ,  // RShiftVL
    true ,  // URShiftVB
    true ,  // URShiftVS
    true ,  // URShiftVI
    true ,  // URShiftVL
    true ,  // AndV
    false,  // AndReductionV
    true ,  // OrV
    false,  // OrReductionV
    true ,  // XorV
    false,  // XorReductionV
    true ,  // MinV
    true ,  // MaxV
    true ,  // MinReductionV
    true ,  // MaxReductionV
    false,  // CompressV
    false,  // CompressM
    false,  // ExpandV
    true ,  // LoadVector
    false,  // LoadVectorGather
    false,  // LoadVectorGatherMasked
    true ,  // StoreVector
    false,  // StoreVectorScatter
    false,  // StoreVectorScatterMasked
    false,  // LoadVectorMasked
    false,  // StoreVectorMasked
    false,  // VectorCmpMasked
    false,  // VectorMaskGen
    false,  // VectorMaskOp
    false,  // VectorMaskTrueCount
    false,  // VectorMaskFirstTrue
    false,  // VectorMaskLastTrue
    false,  // VectorMaskToLong
    false,  // VectorLongToMask
    false,  // Pack
    false,  // PackB
    false,  // PackS
    false,  // PackI
    false,  // PackL
    false,  // PackF
    false,  // PackD
    false,  // Pack2L
    false,  // Pack2D
    true ,  // ReplicateB
    true ,  // ReplicateS
    true ,  // ReplicateI
    true ,  // ReplicateL
    true ,  // ReplicateF
    true ,  // ReplicateD
    false,  // RoundVF
    false,  // RoundVD
    false,  // Extract
    false,  // ExtractB
    false,  // ExtractUB
    false,  // ExtractC
    false,  // ExtractS
    false,  // ExtractI
    false,  // ExtractL
    false,  // ExtractF
    false,  // ExtractD
    false,  // Digit
    false,  // LowerCase
    false,  // UpperCase
    false,  // Whitespace
    false,  // VectorBox
    false,  // VectorBoxAllocate
    false,  // VectorUnbox
    false,  // VectorMaskWrapper
    false,  // VectorMaskCmp
    false,  // VectorMaskCast
    false,  // VectorTest
    false,  // VectorBlend
    false,  // VectorRearrange
    false,  // VectorLoadMask
    false,  // VectorLoadShuffle
    false,  // VectorLoadConst
    false,  // VectorStoreMask
    false,  // VectorReinterpret
    false,  // VectorCast
    false,  // VectorCastB2X
    false,  // VectorCastS2X
    false,  // VectorCastI2X
    false,  // VectorCastL2X
    false,  // VectorCastF2X
    false,  // VectorCastD2X
    false,  // VectorUCastB2X
    false,  // VectorUCastS2X
    false,  // VectorUCastI2X
    false,  // VectorInsert
    false,  // MaskAll
    false,  // AndVMask
    false,  // OrVMask
    false   // XorVMask
};


int Compile::sync_stack_slots() const { return 1 * VMRegImpl::slots_per_word; }

uint Matcher::stack_alignment_in_bytes() { return StackAlignmentInBytes; }

OptoReg::Name Matcher::return_addr() const { return OptoReg::stack2reg(- 2 +
              align_up((Compile::current()->in_preserve_stack_slots() +
                        Compile::current()->fixed_slots()),
                       stack_alignment_in_slots())); }

uint Compile::varargs_C_out_slots_killed() const { return frame::arg_reg_save_area_bytes / BytesPerInt; }

OptoRegPair Matcher::return_value(uint ideal_reg) {

#line 2650 "src/hotspot/cpu/riscv/riscv.ad"

    assert(ideal_reg >= Op_RegI && ideal_reg <= Op_RegL,
           "only return normal values");

    static const int lo[Op_RegL + 1] = { // enum name
      0,                                 // Op_Node
      0,                                 // Op_Set
      R10_num,                           // Op_RegN
      R10_num,                           // Op_RegI
      R10_num,                           // Op_RegP
      F10_num,                           // Op_RegF
      F10_num,                           // Op_RegD
      R10_num                            // Op_RegL
    };

    static const int hi[Op_RegL + 1] = { // enum name
      0,                                 // Op_Node
      0,                                 // Op_Set
      OptoReg::Bad,                      // Op_RegN
      OptoReg::Bad,                      // Op_RegI
      R10_H_num,                         // Op_RegP
      OptoReg::Bad,                      // Op_RegF
      F10_H_num,                         // Op_RegD
      R10_H_num                          // Op_RegL
    };

    return OptoRegPair(hi[ideal_reg], lo[ideal_reg]);
  
#line 20999 "ad_riscv.cpp"

}

OptoRegPair Matcher::c_return_value(uint ideal_reg) {

#line 2650 "src/hotspot/cpu/riscv/riscv.ad"

    assert(ideal_reg >= Op_RegI && ideal_reg <= Op_RegL,
           "only return normal values");

    static const int lo[Op_RegL + 1] = { // enum name
      0,                                 // Op_Node
      0,                                 // Op_Set
      R10_num,                           // Op_RegN
      R10_num,                           // Op_RegI
      R10_num,                           // Op_RegP
      F10_num,                           // Op_RegF
      F10_num,                           // Op_RegD
      R10_num                            // Op_RegL
    };

    static const int hi[Op_RegL + 1] = { // enum name
      0,                                 // Op_Node
      0,                                 // Op_Set
      OptoReg::Bad,                      // Op_RegN
      OptoReg::Bad,                      // Op_RegI
      R10_H_num,                         // Op_RegP
      OptoReg::Bad,                      // Op_RegF
      F10_H_num,                         // Op_RegD
      R10_H_num                          // Op_RegL
    };

    return OptoRegPair(hi[ideal_reg], lo[ideal_reg]);
  
#line 21034 "ad_riscv.cpp"

}

OptoReg::Name Matcher::inline_cache_reg() { return OptoReg::Name(R31_num); }

int Matcher::inline_cache_reg_encode() { return _regEncode[inline_cache_reg()]; }

OptoReg::Name Matcher::interpreter_frame_pointer_reg() { return OptoReg::Name(R8_num); }

OptoReg::Name Matcher::c_frame_pointer() const { return OptoReg::Name(R2_num); }

// Number of callee-save + always-save registers
int  Matcher::number_of_saved_registers() {
  return 0;
};

bool Compile::needs_deep_clone_jvms() { return false; }

// Check consistency of C++ compilation with ADLC options:
// Check adlc -DLINUX=1
#ifndef LINUX
#  error "LINUX must be defined"
#endif // LINUX
// Check adlc -D_GNU_SOURCE=1
#ifndef _GNU_SOURCE
#  error "_GNU_SOURCE must be defined"
#endif // _GNU_SOURCE
// Check adlc -DRISCV64=1
#ifndef RISCV64
#  error "RISCV64 must be defined"
#endif // RISCV64
// Check adlc -D_LP64=1
#ifndef _LP64
#  error "_LP64 must be defined"
#endif // _LP64
